/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/**
 * @author hyassine
 */
dojo.provide("com.ibm.team.scm.web.client.util");

dojo.require("dojox.string.Builder");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("dojo.date");
dojo.require("dojo.i18n");
dojo.require("net.jazz.ajax.ui.internal.PageRegistry");
dojo.require("jazz.app.ResourceLink");
dojo.require("jazz.util.date");
dojo.require("com.ibm.team.scm.web.ui.internal.widgets.OverlayIcon");
dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function() {

var checkNotNull			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNonNegative		= net.jazz.ajax.toolbox.lang.checkNonNegative;
var PageRegistry = net.jazz.ajax.ui.internal.PageRegistry;
var ICON_URI_PREFIX = "../com.ibm.team.scm.web/graphics/icons/";
var messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");

com.ibm.team.scm.web.client.util= {

	_basicPrefix: "#action=com.ibm.team.scm.",
	
	//Action URI prefixes.
	_browseStreamURIPrefix: "#action=com.ibm.team.scm.browseStream&id=",
	_browseWorkspaceURIPrefix: "#action=com.ibm.team.scm.browseWorkspace&id=",
	_browseElementURIPrefix: "#action=com.ibm.team.scm.browseElement&workspaceItemId=",
	_viewComponentHistoryURIPrefix: "#action=com.ibm.team.scm.viewComponentHistory&workspaceItemId=",
	_viewItemHistoryURIPrefix: "#action=com.ibm.team.scm.viewItemHistory&workspaceItemId=",
	_viewChangeSetURIPrefix: "#action=com.ibm.team.scm.viewChangeSet&workspaceItemId=",
	_viewChangeSetItemURIPrefix: "#action=com.ibm.team.scm.viewChangeSetItem&workspaceItemId=",
	_viewSnapshotsURIPrefix: "#action=com.ibm.team.scm.viewSnapshots&workspaceItemId=",
	_showTeamChangesURIPrefix: "#action=com.ibm.team.scm.showTeamChanges",
	_contributorProfilePrefix: "#action=com.ibm.team.repository.viewUserProfile&itemId=",
	_workItemLocationPattern: /^itemName\/.*WorkItem\/(.*)$/,
	_getFileDownloadURIPrefix: "../../service/com.ibm.team.filesystem.service.internal.rest.IFilesystemContentService/",
	_workItemPrefix: "#action=com.ibm.team.workitem.viewWorkItem&id=",
	_defaultServicePrefix: "../../resource/",
	
	//Icons locations.
	_streamIconSrc: ICON_URI_PREFIX + "stream_obj.gif",
	_workspaceIconSrc: ICON_URI_PREFIX + "workspace_obj.gif",
	_componentIconSrc: ICON_URI_PREFIX + "compont_obj.gif",
	_folderIconSrc: ICON_URI_PREFIX + "fldr_obj.gif",
	_folderLockBlueIconSrc: ICON_URI_PREFIX + "folder_lock_blue.gif",
	_folderLockOrangeIconSrc: ICON_URI_PREFIX + "folder_lock_orange.gif",
	_fileIconSrc: ICON_URI_PREFIX + "file_obj.gif",
	_fileLockBlueIconSrc: ICON_URI_PREFIX + "file_lock_blue.gif",
	_fileLockOrangeIconSrc: ICON_URI_PREFIX + "file_lock_orange.gif",
	_historyIconSrc: ICON_URI_PREFIX + "history_view.gif",
	_snapshotIconSrc: ICON_URI_PREFIX + "snapshot_obj.gif",
	_deleteEditIconSrc: ICON_URI_PREFIX + "delete_edit.gif",
	_downloadCoIconSrc: ICON_URI_PREFIX + "download_co.gif",
	_lockDownloadCoIconSrc: ICON_URI_PREFIX + "lockdownload_co.gif",
	_uploadCoIconSrc: ICON_URI_PREFIX + "upload_co.gif",
	_includeObjIconSrc: ICON_URI_PREFIX + "includ_obj.gif",
	_crtWorkItemIconSrc: ICON_URI_PREFIX + "crtworkitem.gif",
	_restoreIconSrc: ICON_URI_PREFIX + "restore.gif",
	_removeIconSrc: ICON_URI_PREFIX + "remove.gif",
	_locksIconSrc: ICON_URI_PREFIX + "locks.gif",	
	_unlockIconSrc: ICON_URI_PREFIX + "unlock.gif",
	_locksDisabledIconSrc: ICON_URI_PREFIX + "locks_d.gif",	
	_unlockDisabledIconSrc: ICON_URI_PREFIX + "unlock_d.gif",
	_lockBlueOverlayIconSrc: ICON_URI_PREFIX + "lock_blue_ovr.gif",
	_lockOrangeOverlayIconSrc: ICON_URI_PREFIX + "lock_orange_ovr.gif",
	_toggleClosed: ICON_URI_PREFIX + "arrow_closed.gif",
	_toggleOpen: ICON_URI_PREFIX + "arrow_open.gif",
	
	// Line Delimiter enums (mapping to enum defined in com.ibm.team.filesystem.common.FileLineDelimiter
	LINE_DELIMITER_NONE: 0,	
	LINE_DELIMITER_LF: 1,	
	LINE_DELIMITER_CR: 2,	
	LINE_DELIMITER_CRLF: 3,	
	LINE_DELIMITER_PLATFORM: 4,
	// Map common mime types and extensions (hint)	
	CONTENT_TYPE2EXTENSION : {
		"application/x-javascript" : "|js|",
		"application/java-archive" : "|jar|",
		"application/x-latex" : "|latex|",
		"application/octet-stream" : "|bin|com|dump|exe|o|",
		"application/msaccess" : "|mdb|",
		"application/msexcel" : "|xla|xls|xlt|xlw|",
		"application/msword" : "|doc|dot|word|",
		"application/mspowerpoint" : "|pps|ppt|",
		"application/msproject" : "|mpp|",
		"application/pdf" : "|pdf|",
		"application/postscript" : "|ps|eps|",
		"application/rtf" : "|rtf|",
		"application/x-tar" : "|tar|",
		"application/x-gzip" : "|gz|gzip|",
		"application/x-shockwave-flash" : "|swf|",
		"application/vnd.ms-word.document.macroEnabled.12" : "|docm|",
		"application/vnd.openxmlformats-officedocument.wordprocessingml.document" : "|docx|",
		"application/vnd.ms-word.template.macroEnabled.12" : "|dotm|",
		"application/vnd.openxmlformats-officedocument.wordprocessingml.template" : "|dotx|",
		"application/vnd.ms-powerpoint.template.macroEnabled.12" : "|potm|",
		"application/vnd.openxmlformats-officedocument.presentationml.template" : "|potx|",
		"application/vnd.ms-powerpoint.addin.macroEnabled.12" : "|ppam|",
		"application/vnd.ms-powerpoint.slideshow.macroEnabled.12" : "|ppsm|",
		"application/vnd.openxmlformats-officedocument.presentationml.slideshow" : "|ppsx|",
		"application/vnd.ms-powerpoint.presentation.macroEnabled.12" : "|pptm|",
		"application/vnd.openxmlformats-officedocument.presentationml.presentation" : "|pptx|",
		"application/vnd.ms-excel.addin.macroEnabled.12" : "|xlam|",
		"application/vnd.ms-excel.sheet.binary.macroEnabled.12" : "|xlsb|",
		"application/vnd.ms-excel.sheet.macroEnabled.12" : "|xlsm|",
		"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet" : "|xlsx|",
		"application/vnd.ms-excel.template.macroEnabled.12" : "|xltm|",
		"application/vnd.openxmlformats-officedocument.spreadsheetml.template" : "|xltx|",
		"application/vnd.oasis.opendocument.text  " : "|odt|",
		"application/vnd.oasis.opendocument.text-template" : "|ott|",
		"application/vnd.oasis.opendocument.text-web" : "|oth|",
		"application/vnd.oasis.opendocument.text-master" : "|odm|",
		"application/vnd.oasis.opendocument.graphics" : "|odg|",
		"application/vnd.oasis.opendocument.graphics-template" : "|otg|",
		"application/vnd.oasis.opendocument.presentation" : "|odp|",
		"application/vnd.oasis.opendocument.presentation-template" : "|otp|",
		"application/vnd.oasis.opendocument.spreadsheet" : "|ods|",
		"application/vnd.oasis.opendocument.spreadsheet-template" : "|ots|",
		"application/vnd.oasis.opendocument.chart" : "|odc|",
		"application/vnd.oasis.opendocument.formula" : "|odf|",
		"application/vnd.oasis.opendocument.database" : "|odb|",
		"application/vnd.oasis.opendocument.image" : "|odi|",
		"application/vnd.openofficeorg.extension" : "|oxt|",
		"application/zip" : "|zip|",
		"audio/mpeg3" : "|mp3|",
		"audio/wav" : "|wav|",
		"image/bmp" : "|bmp|",
		"image/gif" : "|gif|",
		"image/jpeg" : "|jpeg|jpg|",
		"image/png" : "|png|",
		"image/tiff" : "|tif|tiff|",
		"text/css" : "|css|",
		"text/html" : "|htm|html|htmls|shtml|",
		"text/plain" : "|c|c++|cc|cpp|cxx|def|h|java|log|pl|text|txt|csh|sh",
		"text/xml" : "|xml|",
		"video/mpeg" : "|m1v|m2v|mp2|mpg|",
		"video/quicktime" : "|mov|qt|"
	},
		
	//////////////////////////////////////////////////////////////////////////////////
	// URI Retrieval
	///////////////////////////////////////////////////////////////////////////////////
	
	//Return the URI for file content view/download.
	// stateId is optional. If missing, latest state in configuration will be downloaded.
	getFileDownloadURI: function(workspaceId, componentId, fileName, fileItemId, stateId) {
		var result;
		checkNotNull(componentId, "componentId");
		checkNotNull(fileName, "fileName");
		checkNotNull(fileItemId, "fileItemId");
		
		result = this._getFileDownloadURIPrefix + workspaceId + "/" + encodeURIComponent(componentId) + "/" + encodeURIComponent(fileName) + "?" + "itemId=" + encodeURIComponent(fileItemId);
		if (stateId) {
			result += "&" + "stateId=" + encodeURIComponent(stateId);
		}
		return result;
	},
	
	//Return the URI for viewing a stream.
	getBrowseStreamURI: function(id) {
		checkNotNull(id, "id");

		return this._browseStreamURIPrefix + encodeURIComponent(id);
	},
	
	//Return the URI for viewing a workspace.
	getBrowseWorkspaceURI: function(id) {
		checkNotNull(id, "id");

		return this._browseWorkspaceURIPrefix + encodeURIComponent(id);
	},
	
	//Return the URI for viewing active change sets within a team.
	getShowTeamChangesURI: function() {
		return this._showTeamChangesURIPrefix;
	},
	
	//Return the URI for viewing a component, folder, or file.
	getBrowseElementURI: function(workspaceId, componentId, path) {
		checkNotNull(componentId, "componentId");
		checkNotNull(path, "path");
		
		return this._browseElementURIPrefix + encodeURIComponent(workspaceId) + "&componentItemId=" + encodeURIComponent(componentId) + "&path=" + encodeURIComponent(path);
	},
	
	getBrowseElementTabURI: function(workspaceId, componentId, path, tab, n) {
		var uri = this.getBrowseElementURI(workspaceId, componentId, path);
		checkNotNull(tab, "tab");		
		uri += "&tab=" + encodeURIComponent(tab);
		
		if(n)
			uri += "&n=" + encodeURIComponent(n);
			
		return uri;
	},
	
	getBrowseElementTabURIWithFirst: function(workspaceId, componentId, path, tab, first, n) {
		var uri = this.getBrowseElementTabURI(workspaceId, componentId, path, tab);
		checkNotNull(first, "first");		
		uri += "&first=" + encodeURIComponent(first);
		
		if(n)
			uri += "&n=" + encodeURIComponent(n);
			
		return uri;
	},
	
	getBrowseElementTabURIWithLast: function(workspaceId, componentId, path, tab, last, n) {
		var uri = this.getBrowseElementTabURI(workspaceId, componentId, path, tab);
		checkNotNull(last, "last");		
		uri += "&last=" + encodeURIComponent(last);
		
		if(n)
			uri += "&n=" + encodeURIComponent(n);
			
		return uri;
	},
	
	//Return the URI for viewing a component's history.
	getComponentHistoryURI: function(workspaceId, componentId, n) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(componentId, "componentId");
		
		//If a paging size (n) is provided.
		if (n) {
			return this._viewComponentHistoryURIPrefix + encodeURIComponent(workspaceId) + 
			"&componentItemId=" + encodeURIComponent(componentId) +
			"&n=" + encodeURIComponent(n);
		}
		else {
			return this._viewComponentHistoryURIPrefix + encodeURIComponent(workspaceId) + 
			"&componentItemId=" + encodeURIComponent(componentId);
		}
	},
	
	//Return the URI for viewing older change sets within the history view.
	getComponentHistoryURIWithFirst: function(workspaceId, componentId, first, n) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(componentId, "componentId");
		
		//If a paging size (n) is provided.
		if (n) {
			return this._viewComponentHistoryURIPrefix + encodeURIComponent(workspaceId) + 
			"&componentItemId=" + encodeURIComponent(componentId) +
			"&first=" + encodeURIComponent(first) +
			"&n=" + encodeURIComponent(n);
		}
		else {
			return this._viewComponentHistoryURIPrefix + encodeURIComponent(workspaceId) + 
			"&componentItemId=" + encodeURIComponent(componentId) +
			"&first=" + encodeURIComponent(first);
		}	
	},
	
	//Return the URI for viewing newer (more recent) change sets within the history view.
	getComponentHistoryURIWithLast: function(workspaceId, componentId, last, n) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(componentId, "componentId");
		
		//If a paging size (n) is provided.
		if (n) {
			return this._viewComponentHistoryURIPrefix + encodeURIComponent(workspaceId) + 
			"&componentItemId=" + encodeURIComponent(componentId) +
			"&last=" + encodeURIComponent(last) +
			"&n=" + encodeURIComponent(n);
		}
		else {
			return this._viewComponentHistoryURIPrefix + encodeURIComponent(workspaceId) + 
			"&componentItemId=" + encodeURIComponent(componentId) +
			"&last=" + encodeURIComponent(last);
		}
	},
	
	//Return the URI for viewing a folder or file's history.
	getItemHistoryURI: function(workspaceId, componentId, oid) {
		checkNotNull(workspaceId, "workspaceId");
		checkNotNull(componentId, "componentId");
		checkNotNull(oid, "oid");
		
		return this._viewItemHistoryURIPrefix + encodeURIComponent(workspaceId) + 
		"&componentItemId=" + encodeURIComponent(componentId) +
		"&oid=" + encodeURIComponent(oid);
	},
	
	//Return the URI for viewing a particular change set.
	getChangeSetURI: function(workspaceId, changeSetId, fromHistory) {
		checkNotNull(changeSetId, "changeSetId");
		checkNotNull(fromHistory, "fromHistory");
		
		return this._viewChangeSetURIPrefix + encodeURIComponent(workspaceId) + "&changeSetItemId=" + encodeURIComponent(changeSetId) + "&fromHistory=" + encodeURIComponent(fromHistory);
	},
	
	//Return the URI for viewing a particular change set with an anchor to a specific diff within the change set (diffId).
	getChangeSetURIWithLocation: function(workspaceId, changeSetId, fromHistory, diffId) {
		checkNotNull(changeSetId, "changeSetId");
		checkNotNull(fromHistory, "fromHistory");
		checkNotNull(diffId, "diffId");
		
		return this._viewChangeSetURIPrefix + encodeURIComponent(workspaceId) + "&changeSetItemId=" + encodeURIComponent(changeSetId) + "&fromHistory=" + encodeURIComponent(fromHistory) + "&diffId=" + encodeURIComponent(diffId);
	},
	
	//Return the URI for viewing an item within a change set (before and after states of files and folders).
	getViewChangeSetItemURI: function(workspaceId, componentId, itemType, itemId, stateId) {
		checkNotNull(componentId, "componentId");
		checkNotNull(itemType, "itemType");
		checkNotNull(itemId, "itemId");

        if (workspaceId === null) {
            workspaceId = "";
        }
		if(stateId) {
			return this._viewChangeSetItemURIPrefix + workspaceId + "&componentItemId=" 
						+ encodeURIComponent(componentId) + "&itemType=" + encodeURIComponent(itemType)
						+ "&itemId=" + encodeURIComponent(itemId) + "&stateId=" + encodeURIComponent(stateId);
		}
		else {
			return this._viewFileByItemIdPrefix + encodeURIComponent(workspaceId) + "&componentItemId=" 
						+ encodeURIComponent(componentId) + "&itemType=" + encodeURIComponent(itemType)
						+ "&itemId=" + encodeURIComponent(itemId);
		}
	},
	
	//Return the URI for viewing snapshots of a workspace/stream 
	getViewSnapshotsURI: function(workspaceId) {
		checkNotNull(workspaceId, "workspaceId");
		
		return this._viewSnapshotsURIPrefix + encodeURIComponent(workspaceId);
	},
	
	//Create and return a DOM element for a contributor.
	createContributorElement: function(/* ContributorDTO */ contributor) {
		if (!contributor) {
			return document.createTextNode("<unknown>");
		}
		var anchor = document.createElement("a");
		anchor.href = this.getContributorURI(contributor);
		var text = document.createTextNode(contributor.name);
		anchor.appendChild(text);
		return anchor;
	},
	
	//Return the URI to use for a contributor.
	getContributorURI: function(/* ContributorDTO */ contributor) {
		return this._contributorProfilePrefix + contributor.itemId;
	},
	
	//Create and return a DOM element for a reason (e.g. a work item).
	// Plug the hover to preview the work item
	createReasonElement: function(/* ReasonDTO */ reason) {
		var anchor= document.createElement("a");
		anchor.href = this.getReasonURI(reason);
		var workItemId = this.getReasonWorkItemId(reason);
		var uri = this.getWorkItemURI(workItemId);
		//anchor.title= reason.label;
		var text= document.createTextNode(this.restoreTags(reason.label));
		anchor.appendChild(text);
		var linkArgs = {lazyFetch: true, uri: uri, presentationUri: anchor.href};
		new jazz.app.ResourceLink(linkArgs, anchor);
		return anchor;
	},
	// return resource URI for given work item id
	getWorkItemURI: function(workItemId) {
		var baseURI = com.ibm.team.repository.web.transport.TeamServerClient._getServicePrefix();
		return baseURI + "../resource/itemName/com.ibm.team.workitem.WorkItem/" + workItemId;
	},
	restoreTags: function(value) {
		var rawText = value;
		rawText= rawText.replace(/\&gt;/gi,">");
		rawText= rawText.replace(/\&lt;/gi,"<");
		rawText= rawText.replace(/\&amp;/gi,"&");
		rawText= rawText.replace(/\&tilde;/gi,"~");
		rawText= rawText.replace(/\&quot;/gi,"\"");
		rawText= rawText.replace(/\&circ;/gi,"^");
		rawText= rawText.replace(/\&nbsp;/gi," ");
		rawText= rawText.replace(/<br\/>/gi,"\n");
		rawText= rawText.replace(/\&#09;/g, "\t");
		return rawText;
	},
	//Return the URI to use for a reason (e.g. a work item).
	getReasonURI: function(/* ReasonDTO */ reason) {
		if (this.workItemsInstalled()) {
		    var match = this._workItemLocationPattern.exec(reason.location);
		    if (match) {
		    	return this._workItemPrefix + encodeURIComponent(match[1]);
		    }
		}
		return this._defaultServicePrefix + encodeURIComponent(reason.location);
	},
	// Return the work item id
	getReasonWorkItemId: function(/* ReasonDTO */ reason) {
		if (this.workItemsInstalled()) {
		    var match = this._workItemLocationPattern.exec(reason.location);
		    if (match) {
		    	return match[1];
		    }
		}
		return -1;
	},	
	createFlowTargetElement: function(stream) {
		var anchor = document.createElement("a");
		anchor.href = this.getBrowseStreamURI(stream.itemId);
		var img = document.createElement("img");
		img.src = this.getIconSrc("stream");
		img.className = "headerIcon";
		anchor.appendChild(img);
		var text = document.createTextNode(stream.name);
		anchor.appendChild(text);
		return anchor;
	},
	
	dashboardInstalled: function() {
		if (this._dashboardInstalled == undefined) {
			this._dashboardInstalled = PageRegistry.findPageWithId("com.ibm.team.dashboard") != null;
		}
		return this._dashboardInstalled;
	},
	
	workItemsInstalled: function() {
		if (this._workItemsInstalled == undefined) {
			this._workItemsInstalled = PageRegistry.findPageWithId("com.ibm.team.workitem") != null;
		}
		return this._workItemsInstalled;
	},	
	
	///////////////////////////////////////////////////////////////////////////////////
	// Other Utility Services
	///////////////////////////////////////////////////////////////////////////////////
	
	//Return the icon location path depending on the specified iconName parameter.
	getIconSrc: function(iconName){
		if(iconName == "stream") {
			return this._streamIconSrc;
		}
		else if(iconName == "workspace") {
			return this._workspaceIconSrc;
		}
		else if(iconName == "component") {
			return this._componentIconSrc;
		}
		else if(iconName == "folder") {
			return this._folderIconSrc;
		}
		else if(iconName == "folderLockedByMe") {
			return this._folderLockBlueIconSrc;
		}
		else if(iconName == "folderLockedByOther") {
			return this._folderLockOrangeIconSrc;
		}
		else if(iconName == "file") {
			return this._fileIconSrc;
		}
		else if(iconName == "fileLockedByMe") {
			return this._fileLockBlueIconSrc;
		}
		else if(iconName == "fileLockedByOther") {
			return this._fileLockOrangeIconSrc;
		}
		else if(iconName == "history") {
			return this._historyIconSrc;
		}
		else if(iconName == "snapshot") {
			return this._snapshotIconSrc;
		}
		else if(iconName == "download_co") {
			return this._downloadCoIconSrc;
		}
		else if(iconName == "lockdownload_co") {
			return this._lockDownloadCoIconSrc;
		}		
		else if(iconName == "upload_co") {
			return this._uploadCoIconSrc;
		}
		else if(iconName == "include_obj") {
			return this._includeObjIconSrc;
		}
		else if(iconName == "crtWorkItem") {
			return this._crtWorkItemIconSrc;
		}
		else if(iconName == "delete_edit") {
			return this._deleteEditIconSrc;
		}
		else if(iconName == "restore") {
			return this._restoreIconSrc;
		}
		else if(iconName == "remove") {
			return this._removeIconSrc;
		} 
		else if(iconName == "locks") {
			return this._locksIconSrc;
		} 
		else if(iconName == "unlock") {
			return this._unlockIconSrc;
		}
		else if(iconName == "lockByMeOverlay")
			return this._lockBlueOverlayIconSrc;
		else if(iconName == "lockByOtherOverlay")
			return this._lockOrangeOverlayIconSrc;
		else if(iconName == "toggleClosed")
			return this._toggleClosed;
		else if(iconName == "toggleOpen")
			return this._toggleOpen;
	},
	
	getDisabledIconSrc: function(iconName) {
		if(iconName == "locks") {
			return this._locksDisabledIconSrc;
		} 
		else if(iconName == "unlock") {
			return this._unlockDisabledIconSrc;
		}
	},
	
	//Return a formatted date string given a date object.
	_getDateString: function(timestamp) {
		if(timestamp.length == 28) {
			timestamp = timestamp.substring(0,19) + "-" + timestamp.substring(24, 26) + ":" + timestamp.substring(26,28);
		}
		var dateObject = dojo.date.stamp.fromISOString(timestamp);
		return jazz.util.date.formatDif(dateObject, true, true);
	},
	
	//Return an array containing the list of items constituting the specified path.
	parsePathIntoArray: function(path) {
		var subElements = [];
		var count = 1;
		var i = 0;
		var seperatorIndex = 0;

		while(count<path.length) {
			seperatorIndex = path.indexOf("/", count);
			//If a '/' seperator is found, extract the item prior to the seperator.
			if(seperatorIndex!=-1) {
				subElements[i] = path.slice(count, seperatorIndex);
				i++;
				count = seperatorIndex + 1;
			}
			//If no more '/' seperators remain, extract the last item and end the parsing process.
			else {
				subElements[i] = path.slice(count);
				count = path.length;
			}
		}
		
		return subElements;
	},
	
	// Return the parent path for the given path
	parseParentPath: function (path) {
		var separatorIndex = path.lastIndexOf("/");
		if (separatorIndex == -1) {
			return "";
		}
		if(separatorIndex == 0)
			return "/";
		return path.substring(0, separatorIndex);
	},
	
	// Return the filename for the given path that uses
	// forward or back slashes as a separator
	parseFileName: function (path) {
		var separatorIndex = path.lastIndexOf("/");
		if (separatorIndex == -1) {
			separatorIndex = path.lastIndexOf("\\");
		}
		if (separatorIndex == -1) {
			return path;
		}
		return path.substring(separatorIndex + 1);
	},
	getExtension: function (path) {
		var separatorIndex = path.lastIndexOf(".");
		if (separatorIndex === -1) {
			return "";			
		}
		return path.substring(separatorIndex + 1);
	},
	// Return content type typically matching the given
	// extension.
	// Returns null if extension unknown (caller should then default
	// to application/octet-stream)
	// E.g. "c" returns "text/plain" 
	guessContentTypeFromExtension: function (extension) {
		var mimeType, query;
		query = "|"+extension+"|";
		for (mimeType in this.CONTENT_TYPE2EXTENSION) {
			if (this.CONTENT_TYPE2EXTENSION[mimeType].indexOf(query) != -1) {
				return mimeType;
			}
		}
		return null;
	},
	/**
	 * true if given content type is known to be text like, false otherwise.
	 */
	isKnownTextContentType: function (contentType) {
		if (contentType.indexOf("text/") === 0) {
			return true;			
		}
		if (contentType === "application/xml") {
			return true;			
		}
		// "application/javascript" and "application/ecmascript" are RFC 4329 suggested media types
		// "application/x-javascript" also is widely accepted by many browsers (but IE). 
		if (contentType === "application/javascript" || contentType === "application/ecmascript" || contentType === "application/x-javascript") {
			return true;
		} 
		return false;
	},
	//Parse a diff and return an array of the set of changes within the diff.
	//Each entry in the returned array contains a change beginning with @@ -#,# +#,# @@
	// If the diff contains more than 1000 changes, return an array with an explicative message
	parseDiff: function(diff) {
		var changes = [];
		var count = 1;
		var i = 0;
		var regexIndex;
		count = diff.indexOf("@@", 0);
		while(count<diff.length && count!=-1) {
			if (i > 1000) {
				return [ messages.util_TooManyDifferences ];
			}
			regexIndex = diff.indexOf("@@", count+2);
			if(diff.indexOf("@@", regexIndex + 1)!=-1) {
				regexIndex = diff.indexOf("@@", regexIndex + 1);
				changes[i] = diff.slice(count, regexIndex);
				count = regexIndex;
				i++;
			}
			else {
				changes[i] = diff.slice(count, diff.length);
				count = diff.length;
			}
		}		
		return changes;
	},
	
	getLoggedInContributor: function() {
		return com.ibm.team.repository.web.client.session.getAuthenticatedContributor();
	},
	/**
	 * @param {Object} bytes a number of size in bytes
	 * @param {Object} showBytes whether to show size in bytes if it is less than 1KB. false will show 1KB if the size is less than 1024.
	 */
	getSizeDisplay: function(sizeInBytes, showByteSize) {
			if (sizeInBytes / 1024 < 1) { // less than 1 KB
				if(showByteSize)
					return dojo.string.substitute(messages.FileSizeUnit_Bytes, [sizeInBytes]);
				else
					return dojo.string.substitute(messages.FileSizeUnit_KB, [1]); 
			}
			else if (sizeInBytes / 1048576 < 1) // less than 1 MB
				return dojo.string.substitute(messages.FileSizeUnit_KB, [Math.round(sizeInBytes / 1024)]);
			else if (sizeInBytes / 1073741824 < 1) // less than 1 GB
				return dojo.string.substitute(messages.FileSizeUnit_MB, [Math.round(sizeInBytes / 1048576)]);
			else if (sizeInBytes / 1099511627776 < 1) // less than 1 TB
				return dojo.string.substitute(messages.FileSizeUnit_GB, [Math.round(sizeInBytes / 1073741824)]);
			else 
				return dojo.string.substitute(messages.FileSizeUnit_TB, [Math.round(sizeInBytes / 1099511627776)]);
	},
	
	/**
	 * Returns an "img" DOM node for the base icon with appropriate overlay for locking.
	 * @param {Object} baseIconSrc the path to the base icon
	 * @param {Object} isLocked  boolean. true if the artifact represented by the base icon is locked.
	 * @param {Object} byContributor if isLocked is true, then this is the ContributorDTO of the lock owner.  Otherwise it is ignored.
	 */
	getIconWithLockingState: function(baseIconSrc, isLocked, byContributor) {
		if(isLocked === true) {
			var loggedInContributor = this.getLoggedInContributor();
			var overlayIcon = new com.ibm.team.scm.web.ui.internal.widgets.OverlayIcon({
				icon: baseIconSrc,
				overlay: loggedInContributor.itemId == byContributor.itemId ? this.getIconSrc("lockByMeOverlay") : this.getIconSrc("lockByOtherOverlay"),
				layout: com.ibm.team.scm.web.ui.internal.widgets.OverlayIcon.TOP_LEFT});
			return overlayIcon.domNode;
		}
		else {
			var icon = document.createElement("img");
			icon.src = baseIconSrc; 
			return icon;
		}
	},
	/**
	 * Return a decorated file or folder icon.
	 * 
	 * args: an object with the following attributes.
 	 * fileExtension: e.g. txt, png (optional, default to binary).  
	 * fileName: full path / name (e.g. src/readme.txt) (optional, default to binary)
	 * 			if both fileExtension and fileName are specified, fileExtension is used.
 	 * isFolder (optional - default to false)
 	 * isLockedByMe (optional)
 	 * isLockedByOther (optional)
 	 * isLocked (optional - see getIconWithLockingState)
 	 * isSymbolicLink (optional)
 	 * lockedBy (optional - see getIconWithLockingState)
 	 * isExecutable (optional)
	 */
	getDecoratedResourceIcon: function (args) {
		var baseIconSrc, isLocked, byContributor, extension;
		if (args.isFolder) {
			baseIconSrc = this.getIconSrc("folder");
		} else if (args.isSymbolicLink) {
			baseIconSrc = ICON_URI_PREFIX + "symbolic_link.gif";
		} else {
			if (args.fileExtension) {
				extension = args.fileExtension;
			} else if (args.fileName) {
				extension = this.getExtension(args.fileName);
			}
			if (extension === "txt") {
				baseIconSrc = ICON_URI_PREFIX + "textfile_obj.gif";
			} 
			else if (extension === "html" || extension === "htm") {
					baseIconSrc = ICON_URI_PREFIX + "htmlfile_obj.gif";
			}
			else if (extension === "pdf") {
					baseIconSrc = ICON_URI_PREFIX + "pdf_obj.gif";
			}
			else if (extension === "java") {
					baseIconSrc = ICON_URI_PREFIX + "javafile_obj.gif";
			}
			else if (extension === "zip") {
					baseIconSrc = ICON_URI_PREFIX + "zip_obj.gif";
			}
			else if (extension === "doc" || extension === "docx") {
					baseIconSrc = ICON_URI_PREFIX + "wordprocess_obj.gif";
			}
			else if (extension === "ppt") {
					baseIconSrc = ICON_URI_PREFIX + "presentation_obj.gif";
			}
			else if (extension === "xls") {
					baseIconSrc = ICON_URI_PREFIX + "spreadsheet_obj.gif";
			}
			if (!baseIconSrc) {
				baseIconSrc = ICON_URI_PREFIX + "binary_obj.gif";
			}
		}
		isLocked = args.isLocked;
		if (isLocked === undefined) {
			isLocked = args.isLockedByMe || args.isLockedByOther;
		}
		byContributor = args.lockedBy;
		if (!byContributor) {
			byContributor = this.getLoggedInContributor();		
		}
		return this.getIconWithLockingState(baseIconSrc, isLocked, byContributor);
	},
	/**
	 * Return corresponding value, or null if key not found in given dto
	 */
	getVersionableDTOPropertyValue: function (versionableDTO, key) {
		return this.getVersionablePropertyValue(versionableDTO.versionable, key);
	},
	/**
	 * Return corresponding value, or null if key not found in given versionable
	 */
	getVersionablePropertyValue: function (versionable, key) {
		if (versionable.properties) {
			for (i = 0; i < versionable.properties.length; i += 1) {
				if (versionable.properties[i].key === key) {
					return versionable.properties[i].value;
				}
			}
		}
		return null;
	}
};
})();
