/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.viewlets.web.ui.internal.BuildsViewlet");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.JazzFeedViewlet");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.build.web.client.internal.BuildClient");
dojo.require("com.ibm.team.build.web.util.BuildUtil");

dojo.require("jazz.util.date");
dojo.require("jazz.util.html");
dojo.require("jazz.app.ResourceLink");

dojo.requireLocalization("com.ibm.team.build.viewlets.web.ui.internal", "BuildsMessages");

(function() {

/*
 * Aliases
 */
var JazzFeedViewlet = com.ibm.team.dashboard.viewlets.web.ui.internal.JazzFeedViewlet;
var FeedClient = com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var BuildClient = com.ibm.team.build.web.client.internal.BuildClient;
var BuildUtil = com.ibm.team.build.web.util.BuildUtil;

var messages;

/*
 * A viewlet that shows recent builds.
 */
dojo.declare("com.ibm.team.build.viewlets.web.ui.internal.BuildsViewlet",
	JazzFeedViewlet, {

	constructor: function() {
		messages = dojo.i18n.getLocalization("com.ibm.team.build.viewlets.web.ui.internal", "BuildsMessages");
		this.setTrackReadMarks(false);
	},
	
	createTooltip: function(linkNode, entry) {
		var uri = entry.items[0].linkURL;
		var arg = {uri: entry.items[0].linkURL, presentationUri: BuildUtil.getResultHref(uri.substring(uri.lastIndexOf("/")+1)), retainLinkText: true};
		return new jazz.app.ResourceLink(arg,linkNode);
	},

	getFilters: function() {
		return ["com.ibm.team.build.internal.ui.dashboard.BuildResultChangeEventFilter"];
	},
	
	getNoItemsMessage: function() {
		return this.getTags() ? messages.noBuildsWithTags : messages.noBuilds;
	},
	
	getShowActiveBuilds: function() {
		return this.getPreference("showActive");
	},

	getTagsKey: function() {
		return "build.IBuildResult.tags";
	},
	
	getTags: function() {
		return this.getPreference("tags");
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		var scopeChanged = false;
		var updateTitle = false;
		if (newSettings.scope != oldSettings.scope || newSettings.scopeItemId != oldSettings.scopeItemId) {
			updateTitle = true;
			scopeChanged = true;
		}
		if (updateTitle || newSettings.userTitle != oldSettings.userTitle)
			this._updateTitle(true);
		if (scopeChanged || (newSettings.scope != DashboardConstants.SCOPE_CONTRIBUTOR && newSettings.scopeSubteams != oldSettings.scopeSubteams)
			|| oldSettings.preferences.showActive != newSettings.preferences.showActive || oldSettings.preferences.tags != newSettings.preferences.tags) {
			this.reset(true);
		}
		else if (oldSettings.preferences.items != newSettings.preferences.items) {
			this.reset(false);
		}
		else {
			this.update();
			this.updateTotalPageFooter();
		}
	},
	
	_updateTitle: function(persist) {
		this.getSite().setTitle((this.getScopeItem() && this.getScopeItem().name) ? dojo.string.substitute(messages.scopedTitle, [this.getScopeItem().name]) : messages.title, persist);
	},
	
	fetchTooltip: function (entry, srh) {
		var item = entry.items[0];
		var url = item.linkURL;
		var itemId = url.substring(url.lastIndexOf("/")+1);
		var callback = dojo.hitch(this,function(response){
			this._handleBuildResultCallback(response,item, srh);
		});
		var deferred = BuildClient.getResultPresentation({resultItemId: itemId, callback: callback, errback: callback});
		this.getServiceTracker().register(deferred, srh);
	},
	
	_handleBuildResultCallback: function(response, item, srh){
		if (response instanceof Error) {
			item.description = " ";
		}
		else {
			var date = new Date(response.buildResult.buildStartTime);
			var tipHtml = "<div class=\"com-ibm-team-build-BuildsViewletTooltip\">";
			tipHtml += "<div class=\"section\"><span class=\"heading\">" + messages.startTime + "</span>: " + jazz.util.date.formatDif(date, true, true) + "</div>";
			tipHtml += "<div class=\"section\"><span class=\"heading\">" + messages.duration + "</span>: " + jazz.util.date.formatDuration(response.buildResult.buildTimeTaken) + "</div>";
			var tags = response.buildResult.tags ? jazz.util.html.escape(response.buildResult.tags) : "";
			tipHtml += "<div class=\"section\"><span class=\"heading\">" + messages.tags + "</span>: "
			if (tags === "") {
				tipHtml += "<span class=\"none-text\">" + messages.none + "</span>";
			} else {
				tipHtml += tags;
			}
			tipHtml += "</div>";
			var logHtml = "<div class=\"section\"><span class=\"heading\">" + messages.logs + "</span>:";
			var downloadHtml = "<div class=\"section\"><span class=\"heading\">" + messages.downloads + "</span>:";
			if (response.contributions) {
				for (var i = 0; i < response.contributions.length; i++) {
					delete filename;
					delete uri;
					if (response.contributions[i].contentUri)
						var uri = "../../" + response.contributions[i].contentUri;
					else if (response.contributions[i].contribution.extendedContributionProperties) {
						for (var j = 0; j < response.contributions[i].contribution.extendedContributionProperties.length; j++) {
							if (response.contributions[i].contribution.extendedContributionProperties[j].name == "com.ibm.team.build.common.model.IBuildResultContribution.url")
								var uri = response.contributions[i].contribution.extendedContributionProperties[j].value;
						}
					}
					if (!uri)
						continue;
					if (response.contributions[i].contribution.extendedContributionTypeId == "com.ibm.team.build.common.model.IBuildResultContribution.log") {
						var statusIcon = response.contributions[i].contribution.contributionStatus == "ERROR" ? "errorLog" : "log";
						logHtml += "<a href=\"" + uri + "\" class=\"icon " + statusIcon + "\">" + jazz.util.html.escape(response.contributions[i].contribution.label) + "</a>";
						var hasLogs = true;
					} else if (response.contributions[i].contribution.extendedContributionTypeId == "com.ibm.team.build.common.model.IBuildResultContribution.artifact") {
						downloadHtml += "<a href=\"" + uri + "\" class=\"icon artifact\">" + jazz.util.html.escape(response.contributions[i].contribution.label) + "</a>";
						var hasDownloads = true;
					}
				}
			}
			tipHtml += (hasDownloads ? downloadHtml + "</div>" : "") + (hasLogs ? logHtml + "</div>" : "") + "</div>";
			item.description = tipHtml;
		}
		srh.doCallback({entries: [{
			items: [item]
		}]});
	},
	
	/*
	 * Returns the context root path of the repository, e.g. "/jazz".
	 */
	getContextRoot: function() {
		if (!this._contextRoot) {
			// use internal JAF constant (see work item 48210)
			this._contextRoot = net.jazz.ajax._contextRoot;
		}
		return this._contextRoot;
	}
});

})();
