/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.DashboardContainer");

dojo.require("dijit.form.TextBox");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletPage");
dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTabs");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletChooserBlock");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

//
// Aliases
//
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ItemNameRegistry = com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry;
var MetaNav = com.ibm.team.dashboard.web.ui.internal.MetaNav;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var DraggableTabs = com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTabs;
var ViewletChooserBlock = com.ibm.team.dashboard.web.ui.internal.ViewletChooserBlock;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;
var ViewletPage = com.ibm.team.dashboard.web.ui.internal.ViewletPage;
var ViewletRegistry = com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var messages;

/**
 * The widget for the content area (right side) of the dashboard.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.DashboardContainer",
	[Observable, dijit._Widget, dijit._Templated], {
	
	// Variables passed into constructor
	selectedTabId: null,
	allowSharing: true, // optional
	
	// Variables available after setContent is called
	autoSave: null,
	dashboardWidget: null,
	dashboardBundle: null,
	dashboardItem: null,
	editMode: null,
	serviceTracker: new ServiceTracker(),
	tabs: null,
	viewletPages: null,
	
	// Internal variables
	_dashboardActive: false,
	_dashboardContributorInfo: null,
	
	// Template init
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/DashboardContainer.html"),
	
	/*
	 * Early lifecycle functions
	 */
	
	constructor: function(args) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	postCreate: function(){
		this.metaNav = new com.ibm.team.dashboard.web.ui.internal.MetaNav({}, this.metaNav); 
	},
	/*
	 * Sets the dashboard content to the given model object containing
	 * the complete state of the dashboard. Called by Dashboard widget 
	 * once DashboardContainer.domNode is appended to layout.
	 */
	setContent: function(/* DashboardBundleDTO */ dashboardBundle) {
		this.dashboardBundle = dashboardBundle;
		this.dashboardItem = dashboardBundle.dashboard;
		this.viewletPages = new Array(this.dashboardItem.viewletPages.length);
		
		this._dashboardActive = true;
		this._dashboardContributorInfo = dashboardBundle.contributorInfo;

		// register item names
		if (dashboardBundle.itemNames)
			ItemNameRegistry.addNames(dashboardBundle.itemNames);
		if (dashboardBundle.defaults && dashboardBundle.defaults.team)
			ItemNameRegistry.addName(dashboardBundle.defaults.team);
		if (dashboardBundle.defaults && dashboardBundle.defaults.project)
			ItemNameRegistry.addName(dashboardBundle.defaults.project);
		if (dashboardBundle.defaults && dashboardBundle.defaults.contributor)
			ItemNameRegistry.addName(dashboardBundle.defaults.contributor);
		
		// initialize contributor info
		if (!this._dashboardContributorInfo) {
			this._dashboardContributorInfo = dashboardBundle.contributorInfo = {
				dashboard: { itemId: this.dashboardItem.itemId },
				contributor: { itemId: this.dashboardBundle.contributorItemId },
				contextId: this.dashboardBundle.contributorItemId,
				viewletInfo: []
			};
		}
		
		// set editMode and autoSave attributes
		if(this.dashboardBundle.permissions.modify) {
			if (this.isPrivateContributorDashboard()) {
				this.editMode = Constants.EDIT_AUTO_SAVE;
				this.autoSave = true; 
			} else {
				this.editMode = Constants.EDIT_DEFERRED_SAVE;
				this.autoSave = false; 
			}
		} else {
			this.editMode = Constants.READ_ONLY;
			this.autoSave = null;
		}
		
		//yield before instantiating the rest of the UI
		setTimeout(dojo.hitch(this,this._setContent2),0);
	},
	_setContent2: function() {
		if (!this._isDestroyed) {
			//initialize the metanav
			this.metaNav.addListener(dojo.hitch(this, this._onDashboardChange), Constants.PROPERTY_DASHBOARD);
			this.metaNav.init({
				dashboardContainer: this,
				dashboardBundle: this.dashboardBundle,
				editMode: this.editMode,
				selectedTabId: this.selectedTabId,
				allowSharing: this.allowSharing
			});
			
			//create new DraggableTabs container
			var div = document.createElement("div");
			this._tabs.appendChild(div);
			this.tabs = new DraggableTabs(div, {
				parent: this,
				accept: ["tab"],
				dashboardContainer: this,
				dashboardItem: this.dashboardItem,
				dashboardBundle: this.dashboardBundle,
				editing: this.isEditable()
			});
			this.tabs.addListener(dojo.hitch(this, this._onDashboardChange), Constants.PROPERTY_DASHBOARD);
			
			var viewletPageHelpers = this.dashboardItem.viewletPages;
			
			for (var i = 0; i < viewletPageHelpers.length; i++) {
				var viewletPageHelper = viewletPageHelpers[i];
				var id = viewletPageHelper.id;
				this.tabs.add(null, {
					title: viewletPageHelper.title,
					id: id,
					index: i,
					helper: viewletPageHelper,
					type: "tab"
				});
			}
			//show the tab bar and the loading text
			this._tabWrapper.style.display = "block";
			this.tabs.startup(this.selectedTabId);
			
			this.setEditMode(this.editMode, true);
			var selectedTab = this.tabs.selectedTab;
			this.showViewletPage(selectedTab.index);
			this.isInitialized = true;
		}
	},
	
	/*
	 * Public Functions
	 */
	
	isDirty: function() {
		return this._isDirty ? true : false;
	},
	isEditable: function() {
		return (this.editMode == Constants.EDIT_AUTO_SAVE || this.editMode == Constants.EDIT_DEFERRED_SAVE);
	},
	isPrivateContributorDashboard: function() {
		return (this.dashboardItem.scope == DashboardConstants.SCOPE_CONTRIBUTOR && this.dashboardItem["contextId"] == this.dashboardItem.scopeItem.itemId);
	},
	isVisible: function() {
		return this.dashboardWidget.isVisible();
	},
	navigateAwayEvent: function(workbenchNavigation, onBeforeUnload) {
		if (onBeforeUnload) {
			if (!this.autoSave && this.isDirty()) {
				//block the user's navigate away attempt with a popup.
				return messages.unsavedWork;
			} else if (this._isEditingContents()) {
				return messages.editingContents;
			} else {
				return false;
			}
		}
		
		var navigateAwayParams = new Object(); //set defaults for return object
		navigateAwayParams.doNavigate = true;
		navigateAwayParams.destroyWidget = false;
		
		if (this.deleted) return navigateAwayParams;

		if (!this.autoSave && this.isDirty()) {
			var doConfirmDialog = true;
		} else {
			if (this._isEditingContents()) {
				var doConfirmDialog = true;
				var confirmText = messages.navigateAwayPrefix + messages.editingContentsUnload;
			}
		}

		if (!confirmText) var confirmText = messages.navigateAwayPrefix + messages.unsavedChanges;
		
		if (workbenchNavigation) confirmText += messages.canReturn;  //switching to another vertical, they can return to save changes later
		else confirmText += messages.willDiscard; //switching to another dashboard, they will lose changes if they continue.
		
		confirmText += messages.navigateAwaySuffix; 

		if (doConfirmDialog) {
			if (confirm(confirmText)) {
				//user wants to discard changes
				this._releaseLock();
				if (workbenchNavigation && this.isDirty()) {
					navigateAwayParams.destroyWidget = false;
				} else if (workbenchNavigation){
					if (this.currentPage) this.currentPage.setActive(false);
				}
			} else {
				navigateAwayParams.doNavigate = false;
			}
		}
		if (navigateAwayParams.doNavigate) {
			this._dashboardActive = false;
			if (this._blockerDiv) this._blockerDiv.style.display = "none";
			if (this.zoomDiv) this.zoomDiv.style.display = "none";
			if (this.currentPage) {
				this.currentPage.setActive(false);
				if (this.currentPage.killPopup) this.currentPage.killPopup(); 			
			}
		}
		return navigateAwayParams;
	},
	closeViewletChooser: function(event) {
		if (event){
			dojo.stopEvent(event);
		}
		if (!this._addViewletAreaExpanded) {
			return;
		}
		this._addViewletAreaExpanded = false;
		
		var args = {node: this._viewletChooser, duration: 200};
		var chooserWipeOutAnimation = dojo.fx.wipeOut(args);
		chooserWipeOutAnimation.play();
		this.connect(chooserWipeOutAnimation,"onEnd",this._afterChooserWipeOutAnimation);
	},
	openViewletChooser: function(event,noAnim) {
		if (event){
			dojo.stopEvent(event);
		}
		if (this._addViewletAreaExpanded && this._viewletChooser.style.display != "none") {
			return;
		}
		this._createViewletChooser();
		this._addViewletAreaExpanded = true;
		
		if (noAnim) {
			this._viewletChooserBlock.startup();
			this._viewletChooser.style.display = "block";
		} else {
			var args = {node: this._viewletChooser, duration: 200};
			var chooserWipeInAnimation = dojo.fx.wipeIn(args);
			this.connect(chooserWipeInAnimation,"onEnd",this._afterChooserWipeInAnimation);	
			chooserWipeInAnimation.play();
		}
		this._selectAddViewletLink();
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
	},
	// prompts the users and reloads dashboard based on choice.
	refresh: function(event) {
		if (event) 
			dojo.stopEvent(event);
		if (this.isDirty() && !confirm(messages.confirm)) {  //if the dashboard is dirty, prompt the user that they will lose changes
			return;	
		}
		if (this.errorState || !this.dashboardItem || this.isDirty()) {
			this.reload();
		} else {
			this._refresh();
		}
	},
	// forces a reload of the current dashboard - doesn't check for dirty state
	reload: function() {
		var params = Util.getDashboardItemParams(this.dashboardBundle, this.tabs.selectedTabId);
		params.force = true;
		this.dashboardWidget.viewDashboard(params, this.dashboardWidget.projectArea);
	},
	duplicateTab: function(event) {
		if (event)
			dojo.stopEvent(event);
		this.handleCloneTab(false);
	},
	copyTab: function(event) {
		if (event)
			dojo.stopEvent(event);
		this.handleCloneTab(true);
	},
	handleCloneTab: function(copy) {
		if (this.currentPage) {
			var viewletPages = this.dashboardItem.viewletPages;
			for (var i = 0; i < viewletPages.length; i++) {
				if (viewletPages[i].id == this.selectedTabId) {
					if (copy) {
						// create a copy to add scope info where applicable
						var page = dojo.clone(viewletPages[i]);
						// find default scoped viewlts and fix them
						for (var j = 0; j < page.columns.length; j++) {
							if (!page.columns[j].viewlets)
								continue;
							for (var k = 0; k < page.columns[j].viewlets.length; k++) {
								var viewlet = page.columns[j].viewlets[k];
								if (typeof(viewlet.scope) == 'undefined') {
									viewlet.scope = this.dashboardItem.scope;
									viewlet.scopeItem = this.dashboardItem.scopeItem;
								}
							}
						}
						try {
							globalStorage[location.hostname]["com-ibm-team-dashboard-clipboard"] = dojo.toJson(page);
						} catch (e) {
							alert(messages.featureNotSupported);
						}
					} else {
						this.addPageFromHelper(viewletPages[i], i+1);
					}
					break;
				}
			}
		}
	},
	saveDashboard: function() {
		this._saveDashboard(this.dashboardItem);
	},
	setActive: function() {
		this._dashboardActive = true;
		if (this._blockerDiv) this._blockerDiv.style.display = "block";
		if (this.zoomDiv) this.zoomDiv.style.display = "block";
		if (this._updateZoomMargins) this._updateZoomMargins();
		if (this.currentPage) this.currentPage.setActive(true);
	},	
	setDirty: function(isDirty) {
		if (typeof isDirty == "undefined") isDirty = true;
		this._isDirty = isDirty;
		this.metaNav.setDirty(isDirty);
		if (isDirty && !this.autoSave) {
			this._acquireLock();
		} else {
			// release lock when exiting dirty state
			this._releaseLock();
		}
	},
	setEditMode: function (editMode, noPropagate) {
		this.editMode = editMode;
		if (this.isEditable()) {
			this.autoSave = (editMode == Constants.EDIT_AUTO_SAVE) ? true : false; 
			if (this.dashboardBundle && this.dashboardBundle.permissions.addTab) {
				this._createNewTabLink();
			}
			this._showAddViewletLink();
			if (!noPropagate) this.tabs.setEditMode(true);
		} else {
			this._removeNewTabLink();
			this._hideAddViewletLink();
			if (!noPropagate) this.tabs.setEditMode(false);
		}
		
		if (!noPropagate) {
			this.metaNav.setEditMode(editMode);
			for (var i = 0; i < this.viewletPages.length; i++ ) {
				if (this.viewletPages[i]) {
					this.viewletPages[i].setEditMode(this.isEditable());
				}
			}
		}
	},
	
	addPageFromHelper: function(helper, tabIndex) {
		// make a copy of the helper incase the original is still in use
		var helper = dojo.clone(helper);
		// determine how many UUIDs we need
		var count = 1;
		for (var i = 0; i < helper.columns.length; i++) {
			var viewlets = helper.columns[i].viewlets;
			if (viewlets) {
				count += viewlets.length;
			}
		}
		var serviceObject = {
			self: this,
			success: function(uuids) {
				var index = 0;
				helper.id = uuids[index++];
				for (var i = 0; i < helper.columns.length; i++) {
					var viewlets = helper.columns[i].viewlets;
					if (viewlets) {
						for (var j = 0; j < viewlets.length; j++) {
							viewlets[j].id = uuids[index++];
						}
					}
				}
				this.self.doCreateViewletPage(helper, tabIndex);
			},
			failure: function(errorObject) {
				console.error("ViewletPage#addViewlet remote error: " + errorObj.message);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		DashboardClient.generateUUID(count, srh);
	},
	
	/*
	 *   Public functions: Viewlet page management
	 */
	createViewletPage: function(event){
		if (event) {
			dojo.stopEvent(event);
		}
		// first get a UUID
		var serviceObject = {
			self: this,
			success: function(uuids) {
				var viewletPageHelper = {
					id: uuids[0],
					title: messages.newTab,
					columns: [{},{},{}]
				};
				this.self.doCreateViewletPage(viewletPageHelper);
			},
			failure: function(errorObject) {
				console.error("ViewletPage#addViewlet remote error: " + errorObj.message);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		DashboardClient.generateUUID(1, srh);
	},
	doCreateViewletPage: function(viewletPageHelper, index) {
		if (typeof index !== "number")
			index = this.tabs.widgets.length;
		this.dashboardItem.viewletPages.splice(index,0,viewletPageHelper);
		this.viewletPages.splice(index,0,null); // create a space in the viewletPages array, so the page will be reinitialized
		this._onDashboardChange();
		this.tabs.add(null,{title: viewletPageHelper.title, id: viewletPageHelper.id, index: index, helper: viewletPageHelper, type: "tab"});
		this.tabs._selectTabAction(this.tabs.widgets[index]);
	},
	deleteViewletPage: function(index) {
		if (this.viewletPages[index]) {
			this.viewletPages[index].destroy();
			this.viewletPages.splice(index, 1);
			this.dashboardItem.viewletPages.splice(index, 1);
			this._onDashboardChange();
			this.showViewletPage(0);
		}
	},
	moveViewletPage: function(startIndex, endIndex) {
		if (this.viewletPages[startIndex]) {
			var page = this.viewletPages[startIndex];
			this.viewletPages.splice(startIndex,1);
			this.viewletPages.splice(endIndex, 0, page);
			
			//update model
			var viewletPages = this.dashboardItem.viewletPages;
			var page = viewletPages[startIndex];
			viewletPages.splice(startIndex, 1);
			viewletPages.splice(endIndex, 0, page);
			this._onDashboardChange();
		}
	},
	isPageActive: function(pageId) {
		return this._dashboardActive && pageId == this.selectedTabId;
	},
	refreshCurrentTab: function() {
		this.setPageDirty(this.tabs.selectedTab.index);
		this.showViewletPage(this.tabs.selectedTab.index);		
	},
	restoreDashboard: function() {
		var projectAreaItemId = getCurrentProjectArea().itemId;
		var srh = new ServiceResponseHandler(this, "_restoreDashboard2", "_logError");
		DashboardClient.createDashboard(projectAreaItemId, this.dashboardItem.scope, this.dashboardItem.scopeItem.itemId, this.dashboardItem.role, false, srh, this.serviceTracker);
	},
	_restoreDashboard2: function(/* ResponseDTO */ responseDTO) {
		if (responseDTO.status == "OK") {
			var newDashboardItem = responseDTO.data.dashboard;
			this.dashboardItem.title = newDashboardItem.title;
			this.dashboardItem.viewletPages = newDashboardItem.viewletPages;
			var srh = new ServiceResponseHandler(this, "_restoreDashboard3", "_logError");
			DashboardClient.saveDashboard(this.dashboardItem, true, srh, this.serviceTracker);
		}
		else {
			alert(dojo.string.substitute(messages.restoreError, [responseDTO.msg]));
		}
	},
	_restoreDashboard3: function(/* ResponseDTO */ responseDTO) {
		if (responseDTO.status == "OK") {
			this.propertyChanged("title", this.dashboardItem.title);
			this.reload();
		}
		else {
			alert(dojo.string.substitute(messages.restoreError, [responseDTO.msg]));
		}
	},
	setPageDirty: function(index) {
		if (this.viewletPages[index]){
			this.viewletPages[index].dirty = true;	
		}
	},
	showViewletPage: function(index) {
		var init = false;
		var vp = this.viewletPages[index];
		this.tabs.selectTab(index);
		if (this.currentPage && this.currentPage != this.viewletPages[index]) {
			this.currentPage.setActive(false);
			if (this.currentPage.domNode) {
				this.currentPage.domNode.style.display = "none";
			}
		}
		if (!vp || vp.dirty) {
			if (vp && vp.destroyRendering) {
				vp.destroyRendering();
			}
			this._viewletPageWrapper.appendChild(this._createViewletPage(this.tabs.widgets[index].helper, index).domNode);
			init = true;
		} else {
			vp.domNode.style.display = "";
		}
		this.currentPage = this.viewletPages[index];
		this.selectedTabId = this.dashboardItem.viewletPages[index].id;
		if (init) {
			this.currentPage.init();
		}
	},
	showViewletPageById: function(pageId) {
		var viewletPages = this.dashboardItem.viewletPages;
		if (!pageId) {
			pageId = viewletPages[0].id;
		}
		for (var x = 0; x < viewletPages.length; x++) {
			if (viewletPages[x].id == pageId) {
				this.showViewletPage(x);
				break;
			}
		}
	},
	
	/*
	 * Private Functions
	 */
	
	_createViewletPage: function(/* ViewletPage */ viewletPageHelper,/* Integer */ index) {
		if (this.viewletPages[index]) {
			this.viewletPages[index].destroy();
		}
		this.viewletPages[index] = new ViewletPage({viewletPageHelper: viewletPageHelper, dashboardContainer: this});
		this.viewletPages[index].addListener(dojo.hitch(this, this._onDashboardChange), Constants.PROPERTY_DASHBOARD);
		this.viewletPages[index].addListener(dojo.hitch(this, this._onContributorInfoChange), Constants.PROPERTY_CONTRIBUTOR_INFO);
		return this.viewletPages[index];
	},
	/*
	 * Destroys all viewlet widgets so that they can be garbage collected.
	 */
	destroy: function() {
		this._releaseLock();
		// abort any remaining requests
		if (this._serviceTracker) this._serviceTracker.abortActiveRequests();
		// are we in the middle of loading?
		if (this._yieldTimeout) {
			// cancel loading
			clearTimeout(this._yieldTimeout);
			delete this._yieldTimeout;
		}
		if (this._currentTab)
			this._currentTab.destroy();
		for (var i = 0; this.viewletPages && i < this.viewletPages.length; i++) {
			if (this.viewletPages[i]) {
				this.viewletPages[i].destroy();
				this.viewletPages[i] = null;
			}
		}
		this.metaNav.destroy();
		if (this.tabs) {
			this.tabs.destroy();	
		}
		if (this._viewletChooserBlock) {
			this._viewletChooserBlock.destroy();
		}
		this.inherited(arguments);
		this._isDestroyed = true;
	},
	_isEditingContents: function() {
		//traverse the viewlet pages looking for viewlets that are in "Edit Contents" mode
		var viewletPages = this.viewletPages;
		for (var i = 0; viewletPages && i < viewletPages.length; i++) {
			if (viewletPages[i]) {
				var columns = this.viewletPages[i].columns;
				for (var j = 0; columns && j < columns.length; j++) {
					var widgets = this.viewletPages[i].columns[j].widgets;
					for (var k = 0; widgets && k < widgets.length; k++) {
						var widget = widgets[k];
						if (widget && widget._viewlet) {
							if (widget._viewlet.isEditing()) {
								return true;
							}
						}
					} 
				}	
			}											
		}
		return false;
	},
	/*
	 * Utility empty callback for success responses we don't need to
	 * react to.
	 */
	_empty: function() {
		
	},	
	/*
	 * Utility method to log remote errors when calling services.
	 */	
	_logError: function(errorObj) {
		console.error("DashboardContainer.js remote error: " + errorObj.message);
	},
	
	/*
	 * Saves the given dashboard.
	 */
	_saveDashboard: function(/* DashboardItem */ dashboardItem) {
		// ensure that only one outgoing save at a time
		if (!this._isSaveInProgress) {
			this._isSaveInProgress = true;
			var srh = new ServiceResponseHandler(this, "_handleSave", "_handleSaveError");
			DashboardClient.saveDashboard(dashboardItem, false, srh, this.serviceTracker);
			delete this.dashboardItemPendingSave;
		}
		else {
			// clone by serialize/unserialize
			this.dashboardItemPendingSave = dojo.fromJson(dojo.toJson(dashboardItem));
		}
	},
	
	/*
	 * Handles save response (ok/stale/locked/error).
	 */
	_handleSave: function(/* ResponseDTO */ responseDTO) {
		switch(responseDTO.status) {
			case "OK":
				// update client model timestamp
				this.dashboardItem.modified = responseDTO.msg;
				if (this.dashboardItemPendingSave) {
					this.dashboardItemPendingSave.modified = responseDTO.msg;
				}
				// successful save; release lock
				if (!this.autoSave) {
					var srh = new ServiceResponseHandler(this, "_empty", "_empty");
					DashboardClient.unlockDashboard(this.dashboardItem.itemId, srh, this.serviceTracker);
				}
				break;
			case "STALE":
				if (responseDTO.msg) {
					// msg present means modified by someone else
					alert(dojo.string.substitute(messages.itemSaveStaleSpecific, [responseDTO.msg]));
				}
				else {
					// no msg means modified by current user
					alert(messages.itemSaveStaleGeneral);
				}
				break;
			case "LOCKED":
				alert(dojo.string.substitute(messages.itemSaveLocked, [responseDTO.msg]));
				break;
			case "ERROR":
				alert(responseDTO.msg);
				break;
		}
		if (responseDTO.status != "OK") {
			// didn't work; still dirty
			this.setDirty(true);
		}
		this._isSaveInProgress = false;
		
		// check for pending saves
		if (this.dashboardItemPendingSave) {
			this._saveDashboard(this.dashboardItemPendingSave);
		}
	},
	
	/*
	 * Handles save error response.
	 */
	_handleSaveError: function(/* Error */ error) {
		this._logError(error);
		this._isSaveInProgress = false;
		
		// check for pending saves
		if (this.dashboardItemPendingSave) {
			this._saveDashboard(this.dashboardItemPendingSave);
		}
	},

	/*
	 * Called after the user has modified contributor info.
	 */
	_onContributorInfoChange: function() {
		// only save non-default data
		var prunedInfo = {
			dashboard: this._dashboardContributorInfo.dashboard,
			contributor: this._dashboardContributorInfo.contributor,
			contextId: this._dashboardContributorInfo.contextId,
			viewletInfo: []
		};
		var viewletInfo = this._dashboardContributorInfo.viewletInfo;
		var len = viewletInfo.length;
		for (var i=0;i<len;i++) {
			var v = viewletInfo[i];
			if (v.collapsed || v.memento) {
				prunedInfo.viewletInfo.push(v);
			}
		}
		
		// update server state
		var srh = new ServiceResponseHandler(this, "_empty", "_logError");
		if (prunedInfo.viewletInfo.length > 0) {
			// it has content; save it
			DashboardClient.saveDashboardContributorInfo(prunedInfo, srh, this.serviceTracker);
		}
		else {
			// empty (defaults); delete it to save space
			DashboardClient.deleteDashboardContributorInfo(prunedInfo.dashboard.itemId, srh, this.serviceTracker);
		}
	},
	/*
	 * Called after the user has modified the dashboard.
	 */
	_onDashboardChange: function() {
		if (this.autoSave) {
			this.saveDashboard();
		}
		else {
			this.setDirty(true);
		}
	},
	
	/*
	 * Acquires a dashboard lock for the current dashboard, and maintains it
	 * using a keep-alive interval until released.
	 */
	_acquireLock: function(/* boolean */ override) {
		var serviceObj = {
			_self: this,
			success: function(/* ResponseDTO */ responseDTO) {
				switch(responseDTO.status) {
					case "STALE":
						if (responseDTO.msg) {
							// we know who modified it
							alert(dojo.string.substitute(messages.itemEditStaleSpecific, [ responseDTO.msg ]));
						}
						else {
							// don't know who modified it
							alert(messages.itemEditStaleGeneral);
						}
						this._self.reload();
						break;
					case "LOCKED":
						if (confirm(dojo.string.substitute(messages.itemEditLocked, [ responseDTO.msg ]))) {
							// this time, set override to true
							var srh = new ServiceResponseHandler(this, "success", "failure");
							this._self._acquireLock(true);
						} else {
							this._self.reload();
						}
						break;
				}
			},
			failure: function(error) {
				alert(dojo.string.substitute(messages.itemEditError, [ error.message ]));
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		var item = this.dashboardItem;
		DashboardClient.lockDashboard(item.itemId, item.modified, override, srh, this.serviceTracker);
		
		// start keep-alive if not already started
		if (!this._keepAlive) {
			this._keepAlive = setInterval(dojo.hitch(this, function() {
				var serviceObj = {
					success: function(/* ResponseDTO */ responseDTO) {
						// silently cancel keep-alive on problem
						if (responseDTO.status != "OK") {
							if (this._keepAlive) {
								clearInterval(this._keepAlive);
								delete this._keepAlive;
							}
						}
					},
					failure: function(error) {
						// silently cancel keep-alive on error
						if (this._keepAlive) {
							clearInterval(this._keepAlive);
							delete this._keepAlive;
						}
					}
				};
				var srh2 = new ServiceResponseHandler(serviceObj, "success", "failure");
				var item = this.dashboardItem;
				DashboardClient.lockDashboard(item.itemId, item.modified, false, srh2, this.serviceTracker);
			}), 30*1000);
		}
	},
	
	/*
	 * Releases the dashboard lock and cancels the keep-alive interval.
	 */
	_releaseLock: function() {
		if (this._keepAlive) {
			clearInterval(this._keepAlive);
			delete this._keepAlive;
			var srh = new ServiceResponseHandler(this, "_empty", "_empty");
			DashboardClient.unlockDashboard(this.dashboardItem.itemId, srh, this.serviceTracker);
		}
	},
	_refresh: function() {
		for (var i = 0; i < this.viewletPages.length; i++) {
			this.setPageDirty(i);
		}
		this.refreshCurrentTab();
	},
	_createNewTabLink: function() {
		//create new tab link
		if (!this._newTabLink) {
			var newTabLink = document.createElement("a");
			newTabLink.className = "add-new-tab";
			newTabLink.href = "#";
			newTabLink.title = messages.addNewTab;
			var newTabImg = document.createElement("img");
			newTabImg.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/add-tab.png");
			newTabImg.alt = messages.addNewTab;
			newTabLink.appendChild(newTabImg);
			this._tabs.firstChild.appendChild(newTabLink);
			this._newTabLinkEvent = this.connect(newTabLink,"onclick",this.createViewletPage);
			this._newTabLink = newTabLink;
		}
	},
	_removeNewTabLink: function(){
		if (this._newTabLink && this._newTabLink.parentNode) {
			this.disconnect(this.newTabLinkEvent);
			this._newTabLink.parentNode.removeChild(this._newTabLink);
			this._newTabLink = null;
		}
	},
	_showAddViewletLink: function() {
		this._addViewletLink.style.display = "";
		dojo.removeClass(this._addViewletLink, "selected");
	},
	_hideAddViewletLink: function() {
		this._addViewletLink.style.display = "none";	
	},
	_selectAddViewletLink: function() {
		dojo.addClass(this._addViewletLink, "selected");
	},
	_createViewletChooser: function() {
		// If the point has children already, then the viewlet chooser was
		// already created 
		if (this._addViewletContent.hasChildNodes()) {
			return;
		} 
		// Create the new viewlet chooser
		var scope = this.dashboardItem.scope;
		var scopeItemId = this.dashboardItem.scopeItem.itemId;
		this._viewletChooserBlock = new ViewletChooserBlock({dashboardContainer: this, scope: scope, scopeItemId: scopeItemId}); 
		// Add the new child node
		this._addViewletContent.appendChild(this._viewletChooserBlock.domNode);
	},
	_afterChooserWipeInAnimation: function() {
		setTimeout(dojo.hitch(this._viewletChooserBlock, this._viewletChooserBlock.startup), 10);
	},
	_afterChooserWipeOutAnimation: function() {
		this._showAddViewletLink();
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");		
	},
	_showFirebugWarning: function() {
		if (this._firebugWarning) return;
		this._firebugWarning = true;
		if (window.console && window.console.firebug) {
			this.metaNav.showMessage({
				message: this.messages.firebugWarning
			});	
		}
	},
	_showBlockerDiv: function(opacity, zIndex) {
		if (!this._blockerDiv) {
			this._blockerDiv = document.createElement("div");
			this._blockerDiv.className = "jazz-ui-Dialog-blockerDiv";			
		}

		var de = document.documentElement;
		var height = de.clientHeight;
		var scrollHeight = de.scrollHeight;
		
		this._blockerDiv.style.height = ((scrollHeight > height) ? scrollHeight : height) + "px";
		
		if (opacity == null) {
			opacity = 0.06;
		}
		var args = {opacity: 0, top: 0, left: 0}; 
		args.zIndex = zIndex ? zIndex : 0;
		dojo.style(this._blockerDiv,args);

		var appendTo = dojo.byId("content-cell") || document.body;
		appendTo.appendChild(this._blockerDiv);
		
		if (opacity !== 0) {
			var blockerAnim = dojo.animateProperty({node: this._blockerDiv, duration: 280,
				properties:	{opacity: {start: 0, end: opacity}
			}});
			
			blockerAnim.play();
		}
	},
	_hideBlockerDiv: function() {
		if (this._blockerDiv && this._blockerDiv.parentNode)  {
			this._blockerDiv.parentNode.removeChild(this._blockerDiv);
		}
	},
	_addViewletImgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/content/icons/add-viewlet-tab.png") + ((dojo.dependency !== undefined) ? "/_ver"+dojo.dependency.getWebBundleVersion("com.ibm.team.dashboard.web") : "/_ver")
});
})();
