/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.LeftNav");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.build.web.util.BuildUtil");
dojo.require("com.ibm.team.build.web.ui.BuildConstants");
dojo.require("com.ibm.team.build.web.ui.internal.BuildQueue");
dojo.require("com.ibm.team.build.web.ui.internal.BuildEngines");
dojo.require("com.ibm.team.build.web.ui.internal.DefinitionList");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.util.format");
dojo.require("jazz.util.html");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function() {

var BuildUtil = com.ibm.team.build.web.util.BuildUtil;
var BuildConstants = com.ibm.team.build.web.ui.BuildConstants;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

var MAX_RECENT = 5;
var SELECTED_ARTIFACT_CLASS = "selected";
var LI_ID_SUFFIX = "-li";

dojo.declare("com.ibm.team.build.web.ui.internal.LeftNav",
	dijit._Widget, {
	
	findTarget: function(args) {
		if (!args)
			return null;
		return this.findListNode(args.id);
	},
	
	createRecentSection: function() {
		this.recentSection = new com.ibm.team.build.web.ui.internal.LeftNavRecent({
			parent: this,
			buildPage: this.buildPage
		});
		var recentNavSection = this.layout.addNavigationSection(this.recentSection, messages.recentlyViewed, true);
		this.recentSection.populate(recentNavSection);
	},

	postCreate: function() {
		this.createRecentSection();
		this.createSharedSection();
	},
	
	createSharedSection: function() {
		this.sharedSection = new com.ibm.team.build.web.ui.internal.LeftNavShared({
			parent: this
		});
		var sharedNavSection = this.layout.addNavigationSection(this.sharedSection, this.getSharedSectionTitle(), true);
		this.sharedSection.populate(sharedNavSection);
		this.isLoaded = true;
	},
	
	getSharedSectionTitle: function() {
		return messages.projsAndTeams;
	},
	
	translateArgs: function(rawArgs) {
		return rawArgs;
	},
	
	itemDeleted: function (itemId) {
		if (this.recentSection)
			this.recentSection.removeExplicit(itemId);
	},
	
	updateExplicitHighlighting: function(/* boolean */ clearFlag) {
		if (this.isExplicit)
			this._doHighlighting(this.isExplicit);
		if (clearFlag)
			this.isExplicit = false;
	},
	
	updateHighlighting: function(args) {
		if (this.isExplicit) {
			this._doHighlighting(this.isExplicit);
			this.isExplicit = false;
			return;
		}
		if (this._updateHighlightingTimer) {
			clearTimeout(this._updateHighlightingTimer);
			delete this._updateHighlightingTimer;
		}
		// postpone highlighting until the left nav is fully loaded
		if (!this.isLoaded) {
			this._startHighlightTimer(args);
			return;
		}
		
		var rawArgs = args;
		args = this.translateArgs(args);
		
		var foundTarget = this.findTarget(args);
		// if we found a node, do the highlighting, otherwise clear it
		if (foundTarget) {
			this._doHighlighting(foundTarget);
		} else {
			// if we did not find a node, the recently viewed entry has not been created yet
			// try again later
			this._startHighlightTimer(rawArgs);
		}
	},
	
	/* domNode */ findListNode: function(/* string */ id) {
		return dojo.byId(id + LI_ID_SUFFIX);
	},
	
	explicitHighlight: function(/* Node */ node) {
		if (this._highlightedNode != node)
			this.isExplicit = node;
		if (this._highlightedNode && this._highlightedNode.firstChild.href.substring(this._highlightedNode.firstChild.href.indexOf("#action=")) == node.firstChild.href.substring(node.firstChild.href.indexOf("#action=")))
			this.updateHighlighting();
		else
			this.updateExplicitHighlighting();
	},
	
	_startHighlightTimer: function(args) {
		this._updateHighlightingTimer = setTimeout(dojo.hitch(this,function(){
			this.updateHighlighting(args);
		}), 100);
	},
	
	_clearHighlighting: function() {
		if (this._highlightedNode) {
			dojo.removeClass(this._highlightedNode,SELECTED_ARTIFACT_CLASS);
			this._highlightedNode = null;
		}
	},
	
	_doHighlighting: function(newNode){
		if (this._highlightedNode != newNode) {
			if (this._highlightedNode)
				dojo.removeClass(this._highlightedNode,SELECTED_ARTIFACT_CLASS);
			dojo.addClass(newNode,SELECTED_ARTIFACT_CLASS);
			this._highlightedNode = newNode;
		}
	},
	
	destroy: function() {
		if (this.sharedSection)
			this.sharedSection.destroy();
		this.inherited(arguments);
	}
	
});

dojo.declare("com.ibm.team.build.web.ui.internal.LeftNavRecent", dijit._Widget, {
	populate: function(/* NavSection */ navSection) {
		this.navSection = navSection;
		navSection.setVisible(false);
		this._count = 0;
		this._artifacts = {};
		if (this.buildPage)
			var widget = this.buildPage.getContentWidget();
		this.pageVisited(widget);
	},
	
	pageVisited: function(widget) {
		// if there is no getArtifactInfo function, the page should not appear in recently viewed
		if (!widget || !widget.getArtifactInfo)
			return;
		// get the itemId
		var serviceObj = {
			failure: function(error) {
				console.error(error);
			},
			success: dojo.hitch(this,this.processPageVisit)
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		widget.getArtifactInfo(srh);
	},
	
	processPageVisit: function(info) {
		if (this._artifacts[info.itemId]) {
			this._artifacts[info.itemId].lastVisit = new Date().getTime();
			return;
		}
		var existingSection = this.parent.findListNode(info.itemId);
		this._count++;
		this.navSection.setVisible(true);
		var sectionRow =  this.navSection.addSectionRow(jazz.util.format.insertWordBreaks(info.title,18, true), info.href, info.iconUrl, this._topSection);
		if (this._count > MAX_RECENT)
			this._removeOldest();
		if (!existingSection)
			sectionRow.id = info.itemId + LI_ID_SUFFIX;
		this.connect(sectionRow.firstChild, "onclick", dojo.hitch(this.parent, this.parent.explicitHighlight, sectionRow));
		this._artifacts[info.itemId] = {
			lastVisit: new Date().getTime(),
			sectionRow: sectionRow
		};
		this._topSection = sectionRow;
	},
	
	getRow: function(itemId) {
		if (this._artifacts && this._artifacts[itemId])
			return this._artifacts[itemId].sectionRow;
		return null;
	},
	
	removeExplicit: function(itemId) {
		var node = this.getRow(itemId);
		delete this._artifacts[itemId];
		if (node) {
			this._count--;
			if (this._topSection == node) {
				if (node.nextSibling)
					this._topSection = node.nextSibling;
				else
					this._topSection = null;
			}
			if (node.parentNode)
				node.parentNode.removeChild(node);
			if (!this._count)
				this.navSection.setVisible(false);
		}
	},
	
	_removeOldest: function() {
		var oldestVisit = new Date().getTime();
		var oldestArtifact;
		for (var i in this._artifacts) {
			if (this._artifacts[i].lastVisit < oldestVisit) {
				oldestArtifact = i;
				oldestVisit = this._artifacts[i].lastVisit;
			}
		}
		this._artifacts[oldestArtifact].sectionRow.parentNode.removeChild(this._artifacts[oldestArtifact].sectionRow);
		this._count--;
		delete this._artifacts[oldestArtifact];
	}
});

dojo.declare("com.ibm.team.build.web.ui.internal.LeftNavShared",
	dijit._Widget, {
		
	populate: function(navSection) {
		this._rootSection = navSection;
		this.addRow(messages.buildDefinitions, BuildUtil.getDefinitionListHref(), dojo.moduleUrl("com.ibm.team.build.web","graphics/common/icons/builds_obj.gif"), "buildDefinitions");
		this.addRow(messages.buildQueue, BuildUtil.getBuildQueueHref(), dojo.moduleUrl("com.ibm.team.build.web","graphics/common/icons/buildque.gif"), "buildQueue");
		this.addRow(messages.buildEngines, BuildUtil.getBuildEnginesHref(), dojo.moduleUrl("com.ibm.team.build.web","graphics/common/icons/normal_engine.gif"), "buildEngines");
	},
	
	addRow: function(title, onTitleClick, iconUrl, id) {
		if (this._rootSection) {
			var row = this._rootSection.addSectionRow(title, onTitleClick, iconUrl, null);
			row.id = id + LI_ID_SUFFIX;
		}
	}
});


})();
