/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportsNavigator");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.reports.web.ui.internal.NewFolderDialog");
dojo.require("com.ibm.team.reports.web.ui.internal.FolderWidget");
dojo.require("com.ibm.team.reports.web.ui.internal.QueryWidget");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");
dojo.require("com.ibm.team.reports.web.ui.internal.utils.DOMUtils");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() { // module container

var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var DOMUtils = com.ibm.team.reports.web.ui.internal.utils.DOMUtils;

var DROPDOWN_HOVER = ImagePool.DD_ARROW_HOV;
var DROPDOWN       = ImagePool.DD_ARROW;

/**
 * Widget to display a tree of report queries.
 */
dojo.declare("com.ibm.team.reports.web.ui.internal.ReportsNavigator", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportsNavigator.html"),
		
		NEW_FOLDER_ICON_CLASS: "newReportFolderIcon",
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
		},

		postCreate: function() {
			this._controller = this.controller;
			this._shared = this.shared;
			this._rootFolder = null;
			this._canDeployQuery = this.canDeployQuery;
			this._canManageFolder = this.canManageFolder;
			this._defaultQuery = this.defaultQuery;
			this._childFolders = [];
			this._childQueries = [];
		},

		initializePulldown: function(navSection) {
			var span = navSection._togglerLink.childNodes[2];
			navSection._togglerLink.removeChild(span);
			
			var table = document.createElement("table");
			table.width = "100%";
			table.cellPadding = "0";
			table.cellSpacing = "0";
			table.className = "compactTable";
			var tbody = document.createElement("tbody");
			table.appendChild(tbody);
			var tr = document.createElement("tr");
			tbody.appendChild(tr);
			var td1 = document.createElement("td");
			td1.appendChild(span);
			tr.appendChild(td1);
			var td = document.createElement("td");
			td.align = "right";
			tr.appendChild(td);
			
			var a = document.createElement("a");
			a.href = "#";
			a.dojoAttachPoint = "_dropDown";
			a.className = "dropDownFloatingLink";
			// Hide the drop-down indicator initially
			a.style.visibility = "hidden";
			this._dropDown = a;
			
			var img = document.createElement("img");
			img.src = DROPDOWN;
			img.dojoAttachPoint = "_dropDownImage";
			img.className = "reportNavIcon";
			this._dropDownImage = img;
			
			a.appendChild(img);
			navSection._togglerLink.appendChild(a);
			td.appendChild(a);
			navSection._togglerLink.appendChild(table);
			
			// Hook the events for the drop-down indicator
			if (this._canManageFolder) {
				this.connect(img, "onmouseover", this._imgMouseOverHandler);
				this.connect(img, "onmouseout", this._imgMouseOutHandler);
				this.connect(img, "onclick", this._imgClickHandler);
			}
			this.connect(navSection._togglerLink, "onmouseover", this._togglerLinkMouseOverHandler);
			this.connect(navSection._togglerLink, "onmouseout", this._togglerLinkMouseOutHandler);
		},
		
		setDefaultQuery: function(defaultQuery) {
			this._defaultQuery = defaultQuery;
			for (var i = 0; i < this._childQueries.length; i++) {
				this._childQueries[i].setDefaultQuery(defaultQuery);
			}
			for (var i = 0; i < this._childFolders.length; i++) {
				this._childFolders[i].setDefaultQuery(defaultQuery);
			}
		},
		
		_imgMouseOverHandler: function(e) {
			e.preventDefault();
			this._dropDownImage.src = DROPDOWN_HOVER;
		},
		
		_imgMouseOutHandler: function(e) {
			e.preventDefault();
			this._dropDownImage.src = DROPDOWN;
		},
		
		_imgClickHandler: function(e) {
			e.preventDefault();
			this._openMenu(e);
			dojo.stopEvent(e);
		},
		
		_togglerLinkMouseOverHandler: function(e) {
			e.preventDefault();
			if (this._menu != null) {
				// We're already in the menu, ignore this
				this._mouseOutOccurred = false;
				return;
			}
			this._dropDown.style.visibility = "visible";
		},
		
		_togglerLinkMouseOutHandler: function(e) {
			e.preventDefault();
			if (this._menu != null) {
				// We're in the menu, don't remove the hover
				// but remember that we left
				this._mouseOutOccurred = true;
				return;
			}
			this._dropDown.style.visibility = "hidden";
		},
		
		removeChild: function(child) {
			for (var i = 0; i < this._childFolders.length; i++) {
				if (this._childFolders[i] == child) {
					this._childFolders.splice(i, 1);
				}
			}
			for (var i = 0; i < this._childQueries.length; i++) {
				if (this._childQueries[i] == child) {
					this._childQueries.splice(i, 1);
				}
			}
			child.domNode.parentNode.removeChild(child.domNode);
			child.destroy();			
		},
		
		setFolders: function(result) {
			this.removeFolders();
			if (result != null) {
				for (var i = 0; i < result.length; i++) {
					this._rootFolder = result[i];
				}
			}
			if (this._rootFolder != null) {
				this._fetchChildren();
			}
		},
		
		setSelection: function(selection) {
			if (!selection) {
				// set empty query uuid which will cause all folders and items to 'deselect'
				selection = {queryUUID: ""};
			}
			// Find and deselect any existing selected items
			for (var i = 0; i < this._childQueries.length; i++) {
				var queryWidget = this._childQueries[i];
				if (queryWidget.query.queryUUID == selection.queryUUID) {
					queryWidgetToSelect = queryWidget;
					queryWidget.setSelected(true);
				} else {
					queryWidget.setSelected(false);
				}
			}
			for (var i = 0; i < this._childFolders.length; i++) {
				var folderWidget = this._childFolders[i];
				folderWidget.setSelection(selection.queryUUID);
			}
		},
		
		removeFolders: function() {
			while (this.rootFoldersDiv.childNodes[0] != null) {
				this.rootFoldersDiv.removeChild(this.rootFoldersDiv.childNodes[0]);
			}
			while (this.rootQueriesDiv.childNodes[0] != null) {
				this.rootQueriesDiv.removeChild(this.rootQueriesDiv.childNodes[0]);
			}
			this._childFolders = [];
		},
		
		addFolder: function(folder) {
			var folderWidget = new com.ibm.team.reports.web.ui.internal.FolderWidget({folder: folder, parentWidget: this, shared: this._shared, expand: false, controller: this._controller, canDeployQuery: this._canDeployQuery, canManageFolder: this._canManageFolder, defaultQuery: this._defaultQuery});
			this.rootQueriesDiv.appendChild(folderWidget.domNode);
			this._childFolders[this._childFolders.length] = folderWidget;
		},
		
		_fetchChildren: function() {
			var args = {
				// Project Area will be added by ReportsClient
				// Owner is intentionally empty
        		parentUUID: this._rootFolder.folderUUID,
        		flags: ReportsClient.ALL
        	};
			var responseHandler = new ServiceResponseHandler(
					this,
					"_addChildFolders",
					"_remoteError");
			ReportsClient.getFolderDescriptors(responseHandler, args);
			var args2 = {
				// Project Area will be added by ReportsClient
				// Owner is intentionally empty
        		folderUUID: this._rootFolder.folderUUID,
        		flags: ReportsClient.EXCLUDE_MICRO
        	};
			var responseHandler2 = new ServiceResponseHandler(
					this,
					"_addChildQueries",
					"_remoteError");
			ReportsClient.getQueryDescriptors(responseHandler2, args2);
		},
			
		/**
		 * Helper function to sort a list of named items
		 * Note: Assume that names are unique
		 */
		_sort: function(items) {
			var names = [];
			for (var i = 0; i < items.length; i++) {
				names[i] = items[i].name;
			}
			names = names.sort();
			var result = [];
			for (var i = 0; i < names.length; i++) {
				result[i] = this._find(names[i], items);
			}
			return result;
		},
		
		_find: function(name, items) {
			for (var i = 0; i < items.length; i++) {
				if (items[i].name == name) {
					return items[i];
				}
			}
			return null;
		},		
		
		_addChildQueries: function(args) {
			if (args != null) {
				// Sort them first.
				var sorted = this._sort(args);
				for (var i = 0; i < sorted.length; i++) {
					this._addQuery(sorted[i]);
				}
			}
		},
		
		_addChildFolders: function(args) {
			if (args != null) {
				// Sort them first.
				var sorted = this._sort(args);
				for (var i = 0; i < sorted.length; i++) {
					this._addFolder(sorted[i]);
				}
			}
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this.rootFoldersDiv, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			alert(error.message);
		},
		
		_addQuery: function(query) {
			var queryWidget = new com.ibm.team.reports.web.ui.internal.QueryWidget({query: query, parentWidget: this, controller: this._controller, canDeployQuery: this._canDeployQuery, defaultQuery: this._defaultQuery});
			this.rootQueriesDiv.appendChild(queryWidget.domNode);
			this._childQueries[this._childQueries.length] = queryWidget;
		},
		
		_addFolder: function(folder) {
			var folderWidget = new com.ibm.team.reports.web.ui.internal.FolderWidget({folder: folder, parentWidget: this, shared: this._shared, expand: false, controller: this._controller, canDeployQuery: this._canDeployQuery, canManageFolder: this._canManageFolder, defaultQuery: this._defaultQuery});
			this.rootFoldersDiv.appendChild(folderWidget.domNode);
			this._childFolders[this._childFolders.length] = folderWidget;
		},
		
		_openMenu: function(e) {
			if (this._menu) {
				this._closeMenu();
				return;
			}
			var menu = new dijit.Menu();
			menu.addChild(new dijit.MenuItem({label: this.messages.newFolderDotDotDot, iconClass: this.NEW_FOLDER_ICON_CLASS, onClick: dojo.hitch(this,this._newFolder)}));
			var coords = dojo.coords(e.target, true);
			dijit.popup.open({ popup: menu, parent: this, x: coords.x + 10, y: coords.y + 10, onExecute: dojo.hitch(this,this._closeMenu),orient: 'BL' });
			//register event handler that causes menu to close
			this._menuCloseOnBody = this.connect(this,"_onBlur",this._closeMenu);
			this._menu = menu;
			menu.focus();			
		},
		
		_closeMenu: function(e) {
			if (e) {
				dojo.stopEvent(e);
			}
			if (this._mouseOutOccurred) {
				this._dropDown.style.visibility = "hidden";
			}
			this.disconnect(this._menuCloseOnBody);
			dijit.popup.close(this._menu);
			this._menu = null;
		},
		
		_newFolder: function() {
			var args = {
				controller: this._controller,
				shared: this._shared
			};
			this.newFolderDialog = new com.ibm.team.reports.web.ui.internal.NewFolderDialog(args);
		},

		_runAction: function(action) {
			// Do nothing
		},		
		
		_trim: function(string) {
			return this._trimLeft(this._trimRight(string));
		},
		_trimLeft: function(string) {
			return string.replace(new RegExp("^[\\s]+", "g"), "");
		},
		_trimRight: function(string) {
			return string.replace(new RegExp("[\\s]+$", "g"), "");
		}
	}
);

})();