/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	function isNode(o){
		return o && o.nodeType && o.cloneNode;
	}
	
	com.ibm.team.process.web.ui.internal.admin.projects.UIHelper = {
	
		/*
		 * Retrieves the width of the longest string in the array of
		 * strings passed to it. Can be used to determine the correct size for buttons.
		 * Returns the width in EM.
		 */
		getMaxStringDisplayWidthInEms: function(stringArray, testNode, pxMinWidth){
			if (!isNode(testNode)) {
				return -1;
			}
			var tester = document.createElement("div"); //$NON-NLS-1$
			dojo.style(tester, "position", "absolute"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(tester, "left", "-10000px"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(tester, "pad", "0"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(tester, "border", "0"); //$NON-NLS-1$ //$NON-NLS-2$
			testNode.appendChild(tester);
			var width = pxMinWidth || -1;
			
			// Find the longest string
			dojo.forEach(stringArray, function(sample){
				var sampleDiv = document.createElement("div"); //$NON-NLS-1$
				dojo.style(sampleDiv, "float", "right"); //$NON-NLS-1$ //$NON-NLS-2$
				sampleDiv.innerHTML = sample;
				tester.appendChild(sampleDiv);
				width = sampleDiv.clientWidth > width ? sampleDiv.clientWidth : width;
				tester.removeChild(sampleDiv);
				sampleDiv = null;
			});
			
			// Compute pixels per EM
			tester.style.height = '100em'; //$NON-NLS-1$
			var emSize = tester.offsetHeight / 100;
			
			testNode.removeChild(tester);
			
			return width / emSize;
		},
		
		/*
		 *  Helper to style the button for I18N support.  Adds the desired pad
		 *  to the width and sets the button width in em.  Removes any left
		 *  or right pad on the button to ensure that it doesn't eat into the
		 *  content space
		 */
		styleI18NButton: function(button, emWidth, emPad){
			if (emPad === undefined) {
				emPad = 1; //  Set a default pad
			}
			var width = emWidth + emPad + "em"; //$NON-NLS-1$
			dojo.style(button, "width", width); //$NON-NLS-1$
			dojo.style(button, "paddingLeft", 0); //$NON-NLS-1$
			dojo.style(button, "paddingRight", 0); //$NON-NLS-1$
		},
		
		/*
		 * Resize the buttons in the array to accomodate the width of their labels.
		 *
		 * testNode - The node that can be used to calculate the required button width
		 * buttons - The array of buttons with their labels set
		 * pxMinWidth - The optional min width of the buttons in pixels
		 * emPad - The option left and right pad to give the button on each side of label
		 */
		resizeI18NButtons: function(testNode, buttons, pxMinWidth, emPad){
			var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
			var labels = [];
			for (var i = 0; i < buttons.length; i++) {
				labels[i] = buttons[i].innerHTML;
			}
			var newWidth = UIHelper.getMaxStringDisplayWidthInEms(labels, testNode, pxMinWidth);
			dojo.forEach(buttons, function(button){
				UIHelper.styleI18NButton(button, newWidth, emPad);
			});
			return newWidth;
		},
		
		/*
		 * Returns the namespace constant for the supplied link type.
		 *
		 * linkType - one of ProcessConstants.LinkTypes
		 */
		getNamespaceForLinkType: function(linkType){
			switch (linkType) {
				case ProcessConstants.LinkTypes.IMPLEMENTS:
					return ProcessConstants.NAMESPACE_OSLC_RM;
				case ProcessConstants.LinkTypes.IMPLEMENTED_BY:
					return ProcessConstants.NAMESPACE_OSLC_CM;
				case ProcessConstants.LinkTypes.VALIDATES:
					return ProcessConstants.NAMESPACE_OSLC_RM;
				case ProcessConstants.LinkTypes.VALIDATED_BY:
					return ProcessConstants.NAMESPACE_OSLC_QM;
				case ProcessConstants.LinkTypes.TESTS:
					return ProcessConstants.NAMESPACE_OSLC_CM;
				case ProcessConstants.LinkTypes.TESTED_BY:
					return ProcessConstants.NAMESPACE_OSLC_QM;
				case ProcessConstants.LinkTypes.RELATED_TO:
					return ProcessConstants.NAMESPACE_OSLC_CM;
			}
		},
		
		/*
		 * Returns the tag name for the supplied link type.
		 *
		 * linkType - one of ProcessConstants.LinkTypes
		 */
		getTagNameForLinkType: function(linkType){
			switch (linkType) {
				case ProcessConstants.LinkTypes.IMPLEMENTS:
					return ProcessConstants.ServiceProviderTypes.RM;
				case ProcessConstants.LinkTypes.IMPLEMENTED_BY:
					return ProcessConstants.ServiceProviderTypes.CM;
				case ProcessConstants.LinkTypes.VALIDATES:
					return ProcessConstants.ServiceProviderTypes.RM;
				case ProcessConstants.LinkTypes.VALIDATED_BY:
					return ProcessConstants.ServiceProviderTypes.QM;
				case ProcessConstants.LinkTypes.TESTS:
					return ProcessConstants.ServiceProviderTypes.CM;
				case ProcessConstants.LinkTypes.TESTED_BY:
					return ProcessConstants.ServiceProviderTypes.QM;
				case ProcessConstants.LinkTypes.RELATED_TO:
					return ProcessConstants.ServiceProviderTypes.CM;
			}
		},
		
		/*
		 * Returns the node from the collection that has the matching baseName and namespace URI.
		 * undefined is returned if no match is found.
		 *
		 * collection - the collection to search
		 * baseName - the name (local name) of the node to match
		 * namespaceURI - the namespace URI to match
		 */
		getNode: function(collection, baseName, namespaceURI){
			var matchingNode;
			dojo.forEach(collection, function(node){
				if ((node.baseName == baseName || node.localName == baseName) && (node.namespaceURI == namespaceURI || (node.namespaceURI === "" && namespaceURI === null))) { //$NON-NLS-1$
					matchingNode = node;
				}
			});
			return matchingNode;
		},
		
		/*
		 * Returns the nodes from the collection that have the matching baseName and namespace URI.
		 * An empty collection is returned if no matches are found.
		 *
		 * collection - the collection to search
		 * baseName - the name (local name) of the nodes to match
		 * namespaceURI - the namespace URI to match
		 */
		getNodes: function(collection, baseName, namespaceURI){
			var matchingNodes = [];
			dojo.forEach(collection, function(node){
				if ((node.baseName == baseName || node.localName == baseName) && (node.namespaceURI == namespaceURI || (node.namespaceURI === "" && namespaceURI === null))) { //$NON-NLS-1$
					matchingNodes.push(node);
				}
			});
			return matchingNodes;
		},
		
		/*
		 * Returns the attribute node of the node that has the matching baseName and namespace.
		 * undefined is returned if no match is found.
		 *
		 * node - the node whose attributes are to be searched
		 * name - the name of the attribute
		 * namespace - the namespace to match
		 */
		getAttributeNode: function(node, name, namespace){
			var attribute;
			if (dojo.isIE) {
				attribute = node.attributes.getQualifiedItem(name, namespace);
			} else {
				attribute = node.attributes.getNamedItemNS(namespace, name);
			}
			return attribute;
		},
		
		/*
		 * Sets the required attributes of the specified service provider tree
		 * node from the XML specification.
		 *
		 * serviceProviderTreeNode - the node whose attributes are to be set
		 * serviceProvider - the DOM representation of the service provider
		 */
		configureServiceProviderTreeNode: function(serviceProviderTreeNode, serviceProvider){
			var serviceProviderChildren = serviceProvider.childNodes;
			var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
			var titleNode = UIHelper.getNode(serviceProviderChildren, "title", ProcessConstants.NAMESPACE_DC); //$NON-NLS-1$
			var name = titleNode.firstChild.data;
			
			var detailsNode = UIHelper.getNode(serviceProviderChildren, "details", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
			var attributeNode = UIHelper.getAttributeNode(detailsNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var url = attributeNode.value;
			var presentationUrl = url;
			
			var servicesNode = UIHelper.getNode(serviceProviderChildren, "services", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
			attributeNode = UIHelper.getAttributeNode(servicesNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var servicesUrl = attributeNode.value;
			
			var consumerRegistryNode = UIHelper.getNode(serviceProviderChildren, "consumerRegistry", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
			var consumerRegistryUrl;
			if (consumerRegistryNode) {
				attributeNode = UIHelper.getAttributeNode(consumerRegistryNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				consumerRegistryUrl = attributeNode.value;
			}
			
			var date = UIHelper.getUniqueTimeStamp();
			
			serviceProviderTreeNode.name = [name];
			serviceProviderTreeNode.id = [name + date.getTime()];
			serviceProviderTreeNode.type = [ProcessConstants.SERVICE_PROVIDER_TYPE];
			serviceProviderTreeNode.url = [url];
			serviceProviderTreeNode.presentationUrl = [presentationUrl];
			serviceProviderTreeNode.consumerRegistryUrl = [consumerRegistryUrl];
			serviceProviderTreeNode.servicesUrl = [servicesUrl];
		},
		
		/*
		 * Sets the required attributes of the specified service provider
		 * catalog tree node from the XML specification. As well, the stub node
		 * is configure to be a placeholder for the potential children of the
		 * catalog that are fetched in a deferred manner.
		 *
		 * serviceProviderCatalogTreeNode - the node whose attributes are to be set
		 * stubTreeNode - the node to represent potential children of the catalog
		 * stubNodeName - the name to use for the stub node
		 * serviceProviderCatalog - the DOM representation of the service provider catalog
		 */
		configureServiceProviderCatalogTreeNode: function(serviceProviderCatalogTreeNode, stubNode, stubNodeName, serviceProviderCatalog){
			serviceProviderCatalogTreeNode.type = [ProcessConstants.SERVICE_PROVIDER_CATALOG_TYPE];
			serviceProviderCatalogTreeNode.children = [];
			
			var serviceProviderCatalogChildren = serviceProviderCatalog.childNodes;
			
			var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
			var date = UIHelper.getUniqueTimeStamp();
			var titleNode = UIHelper.getNode(serviceProviderCatalogChildren, "title", ProcessConstants.NAMESPACE_DC); //$NON-NLS-1$
			serviceProviderCatalogTreeNode.name = [titleNode.firstChild.data];
			serviceProviderCatalogTreeNode.id = [titleNode.firstChild.data + date.getTime()];
			var attributeNode = UIHelper.getAttributeNode(serviceProviderCatalog, "about", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			serviceProviderCatalogTreeNode.serviceProviderCatalogURL = [attributeNode.value];
			
			date = UIHelper.getUniqueTimeStamp();
			stubNode.name = [stubNodeName];
			stubNode.id = [stubNodeName + date.getTime()];
			stubNode.type = [ProcessConstants.SERVICE_PROVIDER_NOT_LOADED_TYPE];
			stubNode.serviceProviderCatalogId = serviceProviderCatalogTreeNode.id;
			
			serviceProviderCatalogTreeNode.children.push(stubNode);
		},
		
		getUniqueTimeStamp: function(){
			var millis = 2;
			var date = new Date();
			var curDate = null;
			do {
				curDate = new Date();
			} while (curDate - date < millis);
			return curDate;
		}
	};
	
})();
