/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.common.UI"); //$NON-NLS-1$

(function() {

function isDisabled(evt){
	return dojo.hasClass(evt.target, "disabled"); //$NON-NLS-1$
}
	
	
function exclude(elements){
	__forEachAndOne(elements, function(element){
		dojo.addClass(element, "exclude"); //$NON-NLS-1$
	});
}
	
function include(elements){
	__forEachAndOne(elements, function(element){
		dojo.removeClass(element, "exclude"); //$NON-NLS-1$
	});
}
	
function selectGetSelection(selectNode) {
	var result= [];
	dojo.query("option", selectNode).forEach(function(option){ //$NON-NLS-1$
		if(option.selected)
			result.push(option.value);
	});
	return result;
}
	
function selectGetAll(selectNode){
	var result= [];
	dojo.query("option", selectNode).forEach(function(option){ //$NON-NLS-1$
		result.push(option.value);
	});
	return result;
}	

function selectReveal(selectNode, value){
	dojo.query("option", selectNode).forEach(function(option){ //$NON-NLS-1$
		if(option.value == value){
			selectNode.selectedIndex= option.index;
			return;
		}
	});
}

function selectPopulate(selectNode, elements, functions, options) {
	var fnCompare= functions.fnCompare;
	var fnFilter= functions.fnFilter;
	
	if(!dojo.isFunction(fnCompare)){
		fnCompare= function(a, b){
			return functions.fnLabel(a) == null ? (functions.fnLabel(b) == null ? 0 : -1) : functions.fnLabel(a).localeCompare(functions.fnLabel(b));
		}
	}
	elements.sort(fnCompare);
	
	dojo.forEach(elements, function(element) {
		if(!dojo.isFunction(fnFilter) || fnFilter(element)) {
			var optionNode= document.createElement("option"); //$NON-NLS-1$
			if(options && options.name)
				optionNode.setAttribute("name", options.name); //$NON-NLS-1$
			optionNode.value= functions.fnValue(element);
			optionNode.appendChild(document.createTextNode(functions.fnLabel(element)));
			selectNode.appendChild(optionNode);
		}
	});
}

function selectPopulateTree(selectNode, element, functions, level){
	if(functions.fnFilter && !functions.fnFilter(element))
		return;
	level= level || 0;
	var optionNode= document.createElement("option"); //$NON-NLS-1$
	optionNode.value= functions.fnValue(element);
	var dots= ""; //$NON-NLS-1$
	var indentWithDots= !dojo.isIE && !dojo.isMozilla;
	if(indentWithDots){
		var i;
		for(i= 0; i < level; i+=0.5) {
			dots+= "."; //$NON-NLS-1$
		}
		if(level > 0){
			dots+= " "; //$NON-NLS-1$
		}
	} else {
		optionNode.style.paddingLeft= level + "em"; //$NON-NLS-1$
	}
	optionNode.appendChild(document.createTextNode(dots + functions.fnLabel(element)));
	selectNode.appendChild(optionNode);
	
	dojo.forEach(functions.fnChildren(element), function(child){
		selectPopulateTree(selectNode, child, functions, level + 0.5);
	}, this);
}

function selectClear(selectNode) {
	dojo.query("option", selectNode).forEach(function(element){ //$NON-NLS-1$
		element.parentNode.removeChild(element);
	});
}

// deprecated
function populateSelect(selectNode, elements, fnLabel, fnValue, fnComparator, options){
	if(!dojo.isFunction(fnComparator)){
		fnComparator= function(a, b){
			var lA= fnLabel(a);
			var lB= fnLabel(b);
			return lA == null ? (lB == null ? 0 : -1) : lA.localeCompare(lB);
		};
	}
	elements.sort(fnComparator);
	
	dojo.forEach(elements, function(element){
		var optionNode= document.createElement("option"); //$NON-NLS-1$
		if(options && options.name)
			optionNode.setAttribute("name", options.name); //$NON-NLS-1$
		optionNode.value= fnValue(element);
		optionNode.appendChild(document.createTextNode(fnLabel(element)));
		selectNode.appendChild(optionNode);
	});
}

function populateInput(inputNode, elements, fnLabel, fnValue, fnComparator, options){

	if(!dojo.isFunction(fnComparator)){
		fnComparator= function(a, b){
			var lA= fnLabel(a);
			var lB= fnLabel(b);
			return lA == null ? (lB == null ? 0 : -1) : lA.localeCompare(lB);
		};
	}
	elements.sort(fnComparator);
	
	dojo.forEach(elements, function(element){
		var container=document.createElement("div"); //$NON-NLS-1$
		dojo.addClass(container, "entry"); //$NON-NLS-1$
		
		var input=document.createElement("input"); //$NON-NLS-1$
		input.setAttribute("name", options.name); //$NON-NLS-1$
		input.setAttribute("type", options.type); //$NON-NLS-1$
		input.setAttribute("value", fnValue(element)); //$NON-NLS-1$
		container.appendChild(input);
		
		var label=document.createElement("div"); //$NON-NLS-1$
		label.appendChild(document.createTextNode(fnLabel(element)));
		container.appendChild(label);
				
		inputNode.appendChild(container);
	});
}

// ----- implementation -------------------------------------------------------------
	
function __forEachAndOne(elements, fn){
	if(!dojo.isArray(elements)){
		fn(elements);
	} else {
		dojo.forEach(elements, function(element){
			fn(element);
		});
	}
}

//publish methods
dojo.mixin(com.ibm.team.apt.web.ui.internal.common.UI,  {
	isDisabled: isDisabled,
	exclude: exclude,
	include: include,
	populateSelect: populateSelect,
	selectReveal: selectReveal,
	selectPopulate: selectPopulate,
	selectPopulateTree: selectPopulateTree,
	selectClear: selectClear,
	selectGetSelection: selectGetSelection,
	selectGetAll: selectGetAll,
	populateInput: populateInput
});

})();
