/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.query.CompilationRowElementViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.build.web.ui.internal.Constants");
dojo.require("com.ibm.team.build.web.ui.internal.utils.TypeUtil");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ErrorUtil");
dojo.require("com.ibm.team.build.web.client.internal.BuildClient");
dojo.require("com.ibm.team.build.web.ui.internal.query.RowElementViewer");
dojo.require("com.ibm.team.build.web.ui.internal.query.CompilationCellElementViewer");

dojo.require("jazz.util.html");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function() {

/*
 *  Summary: Inherit from the RowElementViewer row widget. This class render all the rows in the Compilation tree-table,
 *  		 including the parent-child relationship of the component-plugin-package-source, and showing the detailed 
 * 			 error and warning data.
 */
	
var Constants = com.ibm.team.build.web.ui.internal.Constants;
var TypeUtil = com.ibm.team.build.web.ui.internal.utils.TypeUtil;
var ErrorUtil = com.ibm.team.build.web.ui.internal.utils.ErrorUtil;
var BuildClient = com.ibm.team.build.web.client.internal.BuildClient;
var RowElementViewer = com.ibm.team.build.web.ui.internal.query.RowElementViewer;
var CompilationCellElementViewer = com.ibm.team.build.web.ui.internal.query.CompilationCellElementViewer; 
var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

dojo.declare("com.ibm.team.build.web.ui.internal.query.CompilationRowElementViewer",
 [RowElementViewer],
{     	
	initializeViewer: function(){
		if (this._rowType == Constants.COMPILE_COMPONENT_ROW){  //Show the root rows in tree which type is "component"
			dojo.removeClass(this.domNode, "hidden"); //$NON-NLS-1$
		}
		if (this._rowData.sourceName){ //Fake the source file's row type
			this._rowType = Constants.COMPILE_SOURCEFILE_ROW;
		}
		if (this._rowType === Constants.COMPILE_SOURCEFILE_ROW) {
			this._isLeafRow = true;
		}
	},
	
	createChildRows: function(){  
		var commonArgs = {  resultItemId: this._resultItemId,
						    callback: dojo.hitch(this, this._handleCallback),
						    errback: ErrorUtil.handleErrorResponseAlert };
		var args = null;
		var deferred;
		if (this._rowType == Constants.COMPILE_COMPONENT_ROW){
			if (this._rowData.packageCount) {
				args = dojo.mixin(commonArgs, { componentName: this._rowData.name } );
				deferred = BuildClient.getPackages(args);
			} else {
				this._handleCallback();
			}
		}
		if (this._rowType == Constants.COMPILE_PACKAGE_ROW){
			args = dojo.mixin(commonArgs, { packageItemId: this._rowData.itemId} );
			deferred = BuildClient.getChildrenOfPackage(args);
		}
		if (deferred) {
			deferred.addErrback(dojo.hitch(this,function(response) {
				if (this.domNode) {
					this.loadFailed();
				}
			}));
		}
	},
	
	_handleCallback: function(response){
		if (!this.domNode) {
			return;
		}
		if (this._rowType == Constants.COMPILE_COMPONENT_ROW){
			this._compilePackages = response;
			this._sourceFiles = this._rowData.simpleContributions;
		}
		if (this._rowType == Constants.COMPILE_PACKAGE_ROW){
			this._compilePackages = response.subPackages;
			this._sourceFiles = response.sources;			
		}
		this._populatePackages();
		return response;
	},
		
	_populatePackages: function(){
		var compilePackages = this._compilePackages;
		var sourceFiles = this._sourceFiles;
		var _columns = [];
		var rowElements = [];
		var rowType = "";
		var columnTypes = [];
		var i, j, self, rowArgs, cellArgs;
		if (compilePackages && compilePackages.length > 0){   //Create children which are Package(Plugin&Package) type
			var packageRowElements = new Array(compilePackages.length);
			for (i = 0; i < compilePackages.length; i++){
				self = this;
				rowArgs = { parentPage: self._parentPage,
						container: self._bodyContainer, 
						parentRowElement: self, 
						_resultItemId: self._resultItemId,
						_rowData: compilePackages[i]
				};
				packageRowElements[i] = new com.ibm.team.build.web.ui.internal.query.CompilationRowElementViewer(rowArgs);
				rowType = compilePackages[i][Constants.REPOSITORY_TYPE];
				columnTypes = TypeUtil.getColumnTypes(rowType);
				for (j = 0; j < columnTypes.length; j++) {
					cellArgs = {rowContainer: packageRowElements[i],
									cellType: columnTypes[j] };
					_columns[j] = new CompilationCellElementViewer(cellArgs);
					if (j === 0 && !packageRowElements[i].hasDetails()) {
						packageRowElements[i]._twistie = _columns[j].addTwistie();
					}
					packageRowElements[i].domNode.appendChild(_columns[j].domNode);
				}
			}
			rowElements = rowElements.concat(packageRowElements);
		}
		if (sourceFiles && sourceFiles.length > 0){   //Create children which are Source file type
			var sourceRowElements = new Array(sourceFiles.length);
			for (i = 0; i < sourceFiles.length; i++){
				self = this;
				rowArgs = { parentPage: self._parentPage,
						container: self._bodyContainer, 
						parentRowElement: self, 
						_resultItemId: self._resultItemId,
						_rowData: sourceFiles[i]
				};
				sourceRowElements[i] = new com.ibm.team.build.web.ui.internal.query.CompilationRowElementViewer(rowArgs);
				var sourceColumnTypes = TypeUtil.COMPILE_SOURCE_COLUMN_TYPES;
				for (j = 0; j < sourceColumnTypes.length; j++) {
					cellArgs = {rowContainer: sourceRowElements[i],
									cellType: sourceColumnTypes[j] };
					_columns[j] = new CompilationCellElementViewer(cellArgs);
					if (j === 0 && !sourceRowElements[i].hasDetails()) {
						sourceRowElements[i]._twistie = _columns[j].addTwistie();
					}
					sourceRowElements[i].domNode.appendChild(_columns[j].domNode);
				}
			}
			rowElements = rowElements.concat(sourceRowElements);
		}
		this.toggleExpand(rowElements);
	},
	
	// clear the Details Conent section before showing the new.
	prepareShowDetails: function(){
		this._parentPage.getCompileDetailsContainer().innerHTML = "";
	},
	
	doShowDetails: function(){
		this.prepareShowDetails();
		this._showDetails(this._rowData, this._parentPage.getCompileDetailsContainer());
	},
	
	_showDetails: function(rowData, compileDetailsContainer){ //Show errors and warning info for rows which type is "Source File"
		var args;
		if (rowData.errorCount > 0){
			if (rowData.errors) {
				this._populateDetails(rowData.errors, true, compileDetailsContainer, rowData.errorCount);
			} else {
				var errorsDiv = document.createElement("div");
				compileDetailsContainer.appendChild(errorsDiv);
				var handleErrors = dojo.hitch(this, function(response) {
					if (errorsDiv && errorsDiv.parentNode == compileDetailsContainer) {
						this._populateDetails(response, true, errorsDiv, rowData.errorCount);
					}
				});
				if (rowData.deferredErrors) {
					rowData.deferredErrors.addCallback(handleErrors);
				} else {
					args = {callback: handleErrors, contributionItemId: rowData.compileContribution.itemId};
					rowData.deferredErrors = BuildClient.getCompilationErrors(args);
				}
			}
		}	
		if (rowData.warningCount > 0){
			if (rowData.warnings) {
				this._populateDetails(rowData.warnings, false, compileDetailsContainer, rowData.warningCount);
			} else {
				var warningsDiv = document.createElement("div");
				compileDetailsContainer.appendChild(warningsDiv);
				var handleWarnings = dojo.hitch(this, function(response) {
					if (warningsDiv && warningsDiv.parentNode == compileDetailsContainer) {
						this._populateDetails(response, false, warningsDiv, rowData.warningCount);
					}
				});
				if (rowData.deferredWarnings) {
					rowData.deferredWarnings.addCallback(handleWarnings);
				} else {
					args = {callback: handleWarnings, contributionItemId: rowData.compileContribution.itemId};
					rowData.deferredWarnings = BuildClient.getCompilationWarnings(args);
				}
			}
		}		
	},
	
	_populateDetails: function(data, errors, parent, count) {
		var i, div, message;
		if (typeof data === "string") {
			data = data.split("\n");
		}
		if (data.length !== count) {
			for (i = 0; i < data.length; i++) {
				data[i] = jazz.util.html.escape(data[i]);
			}
			data = data.join("\n");
			div = document.createElement("div");
			div.className = "compile-message compile-" + (errors ? "error" : "warning") +"-status";
			div.innerHTML = "<pre>" + data + "</pre>";
			parent.appendChild(div);
		} else {
			for (i = 0; i < data.length; i++){
				div = document.createElement("div");
				div.className = "compile-message compile-" + (errors ? "error" : "warning") +"-status";
				message = typeof data[i] === "string" ? jazz.util.html.escape(data[i]) :
					dojo.string.substitute(messages.compilationDetails,[data[i].lineNumber, jazz.util.html.escape(data[i].messageText)]);
				div.innerHTML = "<div>" + message + "</div>";
				parent.appendChild(div);
			}
		}
	},
	
	destroy: function() {
		this.inherited(arguments);
	}
		
});
	
})();
