/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.Observable");

(function() {

/*
 * An object that has any number of properties that can change, and
 * whose changes can be listened to.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.util.Observable", null, {
	
	/*
	 * Adds the given callback as a listener to the given property of
	 * this observable.
	 */
	/* void */ addListener: function(/* function */ callback, /* string */ property) {
		if (!this._callbacksByProperty) {
			this._callbacksByProperty = new Object();
		}
		var callbacks = this._callbacksByProperty[property];
		if (!callbacks) {
			callbacks = new Array();
			this._callbacksByProperty[property] = callbacks;
		}
		callbacks.push(callback);
	},

	/*
	 * Removes the given callback as a listener to the given property
	 * of this observable.
	 */	
	/* void */ removeListener: function(/* function */ callback, /* string */ property) {
		if (this._callbacksByProperty) {
			var callbacks = this._callbacksByProperty[property];
			if (callbacks) {
				for (var i=0;i<callbacks.length;i++) {
					if (callbacks[i] === callback) {
						callbacks.splice(i, 1);
						break;
					}
				}
			}
		}
	},
	
	/*
	 * Notifies all listeners that the given property of this observable
	 * has changed. Listeners of the property will be notified.
	 */
	/* void */ propertyChanged: function(/* string */ property, /* any */ arg) {
		if (this._callbacksByProperty) {
			var callbacks = this._callbacksByProperty[property];
			if (callbacks) {
				for (var i=0;i<callbacks.length;i++) {
					callbacks[i](property, arg);
				}
			}
		}
	}
});

})();
