/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

(function() {

var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

/*
 * Provides access to viewlet definitions, with caching.
 */
com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry = {
	
	_localDeferredsByDefId: new Object(),
	_deferredsByDefIdByRoot: new Object(),
	
	/*
	 * Retrieves the viewlet definition object from the specified origin.
	 */
	/* void */ getDefinition: function(/* string */ definitionId, /* string */ dataServerRoot, /* boolean */ passive, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		// get map
		var deferredsByDefId;
		if (dataServerRoot) {
			var deferredsByDefId = this._deferredsByDefIdByRoot[dataServerRoot];
			if (!deferredsByDefId) {
				deferredsByDefId = this._deferredsByDefIdByRoot[dataServerRoot] = new Object();
			}
		}
		else {
			deferredsByDefId = this._localDeferredsByDefId;
		}	
		
		// check cache
		var deferred = deferredsByDefId[definitionId];
		if (!deferred || (!passive && deferred.passive && !deferred.success)) {
			deferred = this._fetchDefinition(definitionId, dataServerRoot, passive, tracker);
			deferredsByDefId[definitionId] = deferred;
			deferred.passive = passive;
		}
		deferred.addCallback(dojo.hitch(this, function(/* ServiceResponseHandler */ srh, /* object[] */ definitions) {
			deferred.success = true;
			srh.doCallback(definitions);
	    	return definitions;
		}, srh));
		deferred.addErrback(dojo.hitch(this, function(/* ServiceResponseHandler */ srh, /* Error */ error) {
			srh.doErrorCallback(error);
	    	return error;
		}, srh));
	},
	
	/* Deferred */ _fetchDefinition: function(/* string */ definitionId, /* string */ dataServerRoot, /* boolean */ passive, /* ServiceTracker */ tracker) {
		var deferred = new dojo.Deferred();
		var serviceObj = {
			_deferred: deferred,
			success: function(/* object[] */ definitions) {
				this._deferred.callback(definitions);
			},
			failure: function(/* Error */ error) {
				if (!(error instanceof Error)) {
					var orig = error;
					error = new Error();
					dojo.mixin(error, orig);
				}
				this._deferred.errback(error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getViewletDefinitions([definitionId], dataServerRoot, passive, srh, tracker);
		return deferred;
	}
};

})();
