/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.viewlets.web.ui.internal.utils.MetagroupsDialog");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Dialog");

dojo.requireLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "StatisticsMessages");

// TODO: MP: Set limit on number of meta-groups
// TODO: MP: Validate patterns by try catch creation of regex and inform user
// TODO: MP: Check for instances of separators: ~~, @@ 
// TODO: MP: Add ability to specify associated colour?

/*
 * Aliases 
 */
var Dialog = jazz.ui.Dialog;


(function() {

/*
 * Class 
 */
dojo.declare("com.ibm.team.workitem.viewlets.web.ui.internal.utils.MetagroupsDialog",
	[dijit._Widget, dijit._Templated], {
	
	/*
	 * Properties
	 */		
	metagroups: [],
	onOK: null,
	onCancel: null,	
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "ui/internal/templates/utils/MetagroupsDialog.html"),

	/*
 	 * @param Object args
 	 * 
 	 * @return void
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "StatisticsMessages");
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		
		this.inherited(arguments);
		
		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->

		this._dialog = null;
		
		this._doIt = false;
		
		this._DOM_ID_PREFIX = "wis-mgd_";
		this._domIDCount = 0;
		
		this._metagroupModel = {};
		
		// Initialization //--------------------------------------------------->

		// Create the UI
		this._createDialogContents();
		this._connectDialogHandlers();
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		this._disconnectDialogHandlers();
		
		// Dialog
		if (this._dialog) {
			this._dialog.destroy();
			delete this._dialog;
		}

		this.inherited(arguments);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	show: function() {
		// Create the dialog
		this._dialog = new Dialog({
			contentNode: this.domNode,
			primaryTitle: this.messages.msgMetagroups,
			width: "452px",
			onClose: dojo.hitch(this, this._handleEventDialogClosed) 
		});		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	hide: function() {
		if (this._dialog) {
			this._dialog.close();
		}		
	},

	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventDialogClosed: function() {

		if (this._doIt && 
			this.onOK) {
			// OK button clicked
			var metagroups = this._getMetagroups();
			this.onOK(metagroups);
		} else if (this.onCancel) {
			// Cancel button clicked
			this.onCancel();
		}
		
		var self = this;
		setTimeout(function() {
			self.destroy();
		}, 1000);
		
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Array
 	 */	
	_getMetagroups: function() {
		
		var metagroups = [];
		
		for (var id in this._metagroupModel) {
			
			var row = this._metagroupModel[id];
			
			if (!row) {
				// TODO: MP: Error message?
				continue;
			} else if (row.hasChildNodes() === false) {
				// TODO: MP: Error message?
				continue;
			}
			
			var columns = row.childNodes;
			
			if (columns.length < 3) {
				// TODO: MP: Error message?
				continue;
			} else if (!columns[0]) {
				// TODO: MP: Error message?
				continue;
			} else if (!columns[1]) {
				// TODO: MP: Error message?
				continue;
			}

			var labelField = columns[0].firstChild; 
			var patternField = columns[1].firstChild;
			
			if (!labelField) {
				// TODO: MP: Error message?
				continue;
			} else if (!patternField) {
				// TODO: MP: Error message?
				continue;
			} else if (!labelField.value) {
				// TODO: MP: Error message?
				continue;
			} else if (!patternField.value) {
				// TODO: MP: Error message?
				continue;
			}

			try {
				var re = new RegExp(patternField.value, "i");
				re.test("_");
			} catch (errorObject) {
				// TODO: MP: Error message?
				continue;
			}

			metagroups.push({
				label: labelField.value,
				pattern: patternField.value
			});
		}
		
		return metagroups;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_createDialogContents: function() {
		
		if (this.metagroups.length === 0) {
			this._createRow({
				label: "",
				pattern: ""
			});
		} else {
			for (var index = 0; index < this.metagroups.length; index++) {
				this._createRow(this.metagroups[index]);
			}
		}
	},

	/*
 	 * @param Object metagroup
 	 * 
 	 * @return void
 	 */	
	_createRow: function(metagroup) {

		this._domIDCount++;
		var metagroupID = this._DOM_ID_PREFIX + this._domIDCount;

		// Label
		var inputLabel = document.createElement("input");
		inputLabel.type = "text";
		inputLabel.size = 30;
		inputLabel.maxlength = 50;
		inputLabel.value = metagroup.label;
		dojo.addClass(inputLabel, "labelField");
		
		// Pattern
		var inputPattern = document.createElement("input");
		inputPattern.type = "text";
		inputPattern.size = 30;
		inputPattern.maxlength = 50;
		inputPattern.value = metagroup.pattern;
		dojo.addClass(inputPattern, "patternField");
		
		// Remove
		var removeAction = document.createElement("a");
		removeAction.href = "javascript:void(0);";
		removeAction.id = metagroupID;
		removeAction.innerHTML = this.messages.msgRemove;
		// TODO: MP: Connect handlers in separate function?
		// TODO: MP: Track handlers to disconnect?
		this.connect(removeAction, "onclick", this._handleEventRemoveClicked);
		
		// Columns
		var td1 = document.createElement("td");
		td1.appendChild(inputLabel);
		var td2 = document.createElement("td");
		td2.appendChild(inputPattern);
		var td3 = document.createElement("td");
		td3.appendChild(removeAction);
		
		// Row
		var tr = document.createElement("tr");
		tr.appendChild(td1);
		tr.appendChild(td2);
		tr.appendChild(td3);
		
		this._contentSite.appendChild(tr);
		
		// Model
		this._metagroupModel[metagroupID] = tr;
		
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return void
 	 */	
	_handleEventRemoveClicked: function(event) {
		if (event) {
			dojo.stopEvent(event);
		} else {
			// TODO: MP: Error message?
			return true;
		}
		
		if (!event.target) {
			// TODO: MP: Error message?
			return true;
		}
		
		var metagroupID = event.target.id;
		if (!metagroupID) {
			// TODO: MP: Error message?
			return true;
		}		
		
		var row = this._metagroupModel[metagroupID];
		if (!row) {
			// TODO: MP: Error message?
			return true;
		}

		if (this._contentSite.hasChildNodes() &&
			this._contentSite.childNodes.length === 1) {
			// Do not delete the last row
			// Clear the fields
			this._clearRow(row);
		} else {
			// Remove from DOM
			this._contentSite.removeChild(row);
			// Remove from Model
			delete this._metagroupModel[metagroupID];
		}

		// TODO: MP: Disconnect handler to be safe

		return true;
	},

	/*
 	 * @param Element row
 	 * 
 	 * @return void
 	 */	
	_clearRow: function(row) {
		
		if (!row) {
			// TODO: MP: Error message?
			return;
		} else if (row.hasChildNodes() === false) {
			// TODO: MP: Error message?
			return;
		}		
		
		var columns = row.childNodes;
		
		if (columns.length < 3) {
			// TODO: MP: Error message?
			return;
		} else if (!columns[0]) {
			// TODO: MP: Error message?
			return;
		} else if (!columns[1]) {
			// TODO: MP: Error message?
			return;
		}

		var labelField = columns[0].firstChild; 
		var patternField = columns[1].firstChild;
			
		if (!labelField) {
			// TODO: MP: Error message?
			return;
		} else if (!patternField) {
			// TODO: MP: Error message?
			return;
		}
		
		labelField.value = "";
		patternField.value = "";
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_connectDialogHandlers: function() {
		// Add link
		this._addLinkHandler = this.connect(this._addLink, "onclick", this._handleEventAddLinkClicked);
		// OK button
		this._clickOKHandler = this.connect(this._okButton, "onclick", this._handleEventOKButtonClicked);
		// Cancel button
		this._clickCancelHandler = this.connect(this._cancelButton, "onclick", this._handleEventCancelButtonClicked);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_disconnectDialogHandlers: function() {
		// Add link
		if (this._addLinkHandler) {
			this.disconnect(this._addLinkHandler);
			delete this._addLinkHandler;
		}
		// OK button
		if (this._clickOKHandler) {
			this.disconnect(this._clickOKHandler);
			delete this._clickOKHandler;
		}
		// Cancel button
		if (this._clickCancelHandler) {
			this.disconnect(this._clickCancelHandler);
			delete this._clickCancelHandler;
		}
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventAddLinkClicked: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}

		this._createRow({
			label: "",
			pattern: ""
		});
		
		return true;
	},
	
	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventOKButtonClicked: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		
		this._doIt = true;
		this.hide();
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return void
 	 */	
	_handleEventCancelButtonClicked: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		
		this._doIt = false;
		this.hide();
		
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;
		
	}
	
});

})();
