/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.viewlets.web.ui.internal.utils.ReportsPreferenceProvider");

/*
 * Imports 
 */
dojo.require("dojo.i18n");  
dojo.require("dojo.string");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");

dojo.require("com.ibm.team.reports.web.ui.internal.ReportParamFactory");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParamGroup");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParametersPanel");

dojo.requireLocalization("com.ibm.team.reports.viewlets.web.ui.internal", "TrendReportMessages");


(function() {

/*
 * Aliases 
 */
var CustomPreferenceProvider = com.ibm.team.dashboard.web.ui.CustomPreferenceProvider;
var ReportParamFactory = com.ibm.team.reports.web.ui.internal.ReportParamFactory;
var ReportParametersPanel = com.ibm.team.reports.web.ui.internal.ReportParametersPanel;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var REPORTS_REST_SERVICE_URI = "com.ibm.team.reports.common.internal.service.IReportRestService";

/*
 * Class 
 */
dojo.declare("com.ibm.team.reports.viewlets.web.ui.internal.utils.ReportsPreferenceProvider",
	[dijit._Widget, dijit._Templated, CustomPreferenceProvider], {

	/*
	 * Properties
	 */	
	context: null,
	projectAreaUUID: null,	
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return ReportsPreferenceProvider
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.viewlets.web.ui.internal", "TrendReportMessages");
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		
		this.inherited(arguments);
		
		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		
		this._reportParams = [];
		this._reportQueryLabelMap = [];
		this._fetchLabelInProgress = false;
		
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;

		this._static._GROUP_PARAMETER = "com.ibm.team.reports.dto:ReportParameterGroupDTO";
		this._static._SINGLE_PARAMETER = "com.ibm.team.reports.dto:ReportParameterDTO";

		this._static._STYLE_LOADING = "color:#BBBBBB;";

	},

	/*
 	 * @param String value
 	 * 
 	 * @return String
 	 * @overrides super
 	 */	
	getLabel: function(value) {
		var queryLabel = this._getReportQueryLabel(value); 
		if (!queryLabel) {
			// Return null to initiate polling of this method by the framework 
			return null;
		}
		return this._buildLabel(queryLabel, null);
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 * @overrides super
 	 */	
	getLoadingLabel: function() {
		return this._buildLabel(this.messages.msgLoading, this._static._STYLE_LOADING);
	},
	
	_getParameterSelections: function() {
		var selections = {
			parameterNames: [],
			parameterValues: []
		};
		
		for (var i = 0, len = this._reportParams.length; i < len; ++i) {
			var reportParam = this._reportParams[i];
			selections.parameterNames[i] = reportParam.parameterDTO.name;
			var vals = reportParam.getSelectedValues();
			
			var noSelection;
			if (((vals instanceof Array) === false) ||
				(vals.length === 0) ||
				!vals[0]) {
				noSelection = true;
			} else {
				noSelection = false;
			}
			
			if (reportParam.parameterDTO.required && noSelection) {
				var errMessage = dojo.string.substitute(this.messages.msgReportParamMissing, [reportParam.parameterDTO.name]);
				this.context._alertErrorMessage(errMessage);
				return null;
			}
			if (noSelection) {
				selections.parameterValues[i] = "";
			} else {
				selections.parameterValues[i] = vals.join(","); 
			}
		}
		
		return selections;
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	getSelectedParameterValues: function() {

		var selections = {};
		
		for (var index = 0; index < this._reportParams.length; index++) {
			var reportParam = this._reportParams[index];
			if (reportParam.parameterDTO.concealed === false) {
				selections[reportParam.parameterDTO.name] = reportParam.getSelectedValues();
			}
		}
		
		return selections;
	},	
	
	_cacheReportQueryLabel: function(queryUUID, queryName) {
		this._reportQueryLabelMap[queryUUID] = queryName;
	},
	
	/*
 	 * @param String queryUUID
 	 * 
 	 * @return String
 	 */	
	getCachedQueryLabel: function(queryUUID) {
		if (queryUUID in this._reportQueryLabelMap) {
			return this._reportQueryLabelMap[queryUUID];
		}
		return null;
	},	
	
	/*
 	 * @param String queryUUID
 	 * 
 	 * @return String
 	 */	
	_getReportQueryLabel: function(queryUUID) {
		
		if (this.context._isValidUUID(queryUUID) === false) {
			// No query has been selected yet
			return this.messages.msgNoQuerySpecified;
        } else if (queryUUID in this._reportQueryLabelMap) {
			// Query label found in cache
			return this._reportQueryLabelMap[queryUUID];
		} else if (this._fetchLabelInProgress === false) {
			// Fetch query label
			this._fetchReportQueryLabel(queryUUID);
		}
		return null;
	},
	
	/*
 	 * @param String queryUUID
 	 * 
 	 * @return void
 	 */	
	_fetchReportQueryLabel: function(queryUUID) {
		
		this._fetchLabelInProgress = true;
		
		var self = this;
		var serviceObject = {
			success: function(queryDescriptorDTO) {
				self._fetchLabelInProgress = false;
				self._cacheReportQueryLabel(queryDescriptorDTO.queryUUID, queryDescriptorDTO.name);
			},
			failure: function(errorObject) {
				self._fetchLabelInProgress = false;
				self._cacheReportQueryLabel(queryUUID, self.messages.msgPreferenceLoadingFailure);
				self.context._logError(errorObject);
				self.context._displayErrorMessage(self.messages.msgReportParameterLoadingFailed, errorObject);
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

		this._callRESTGetQueryDescriptor(handler, queryUUID);
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */
	save: function() {
		// Viewlet settings area:  "Save" button pressed
		// NO-OP
	},
		
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_saveQuery: function() {
		
		this._okBtn.disabled = true;
		
		var selections = this._getParameterSelections();
		if (!selections) {
			var errorObject = new Error(this.messages.msgNoParametersSelected);
			this.context._logError(errorObject);
			this.context._alertErrorMessage(this.messages.msgQuerySaveFailed, errorObject);
			this._okBtn.disabled = false;
			return;
		} else if (!this.projectAreaUUID) {
			this.context._logError(new Error("Project area is not defined"));
			this.context._alertErrorMessage(this.messages.msgQuerySaveFailed);
			this._okBtn.disabled = false;
			return;
		}

		var self = this;
		var serviceObject = {
			success: function(queryUUID){
				if (self.context._isValidUUID(queryUUID)) {
					self._cacheReportQueryLabel(queryUUID, self._nameField.value);
					self._onPrefOk(queryUUID, self.context._static._PREFERENCE_ID_QUERY_UUID);
					self._reportParametersPanel.closeDialog();
				} else {
					var errorObject = new Error(self.messages.msgInvalidQueryUUID);
					self.context._logError(errorObject);
					self.context._alertErrorMessage(self.messages.msgQuerySaveFailed, errorObject);
					self._okBtn.disabled = false;
				}
			},
			failure: function(errorObject){
				self.context._logError(errorObject);
				self.context._alertErrorMessage(self.messages.msgQuerySaveFailed, errorObject);
				self._okBtn.disabled = false;
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

		this._callRESTPostSaveQueryDescriptor(
			handler, 
			this.context._reportUUID,
			this.projectAreaUUID,
			"", // Create a new query item ID
			selections.parameterNames, 
			selections.parameterValues, 
			this._nameField.value,
			true /* keepInTempStorage */);
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_createParametersPanel: function() {
		
		this._reportParametersPanel = new ReportParametersPanel({
				nestedStyledBox: true,
				padding: 0
			});
		
		// Preview Button
		this._previewBtn = document.createElement("button");
		this._previewBtn.innerHTML = this.messages.msgPreview;
		this.connect(this._previewBtn, "onclick", this.handleEventPreviewButtonClicked);
		
		// OK Button
		this._okBtn = document.createElement("button");
		this._okBtn.innerHTML = this.messages.msgOK;
		this.connect(this._okBtn, "onclick", this._handleEventOKButtonClicked);
		
		// Cancel Button		
		var cancelBtn = document.createElement("button");
		cancelBtn.innerHTML = this.messages.msgCancel;
		this.connect(cancelBtn, "onclick", this._handleEventCancelButtonClicked);
		
		// Button Site
		var buttonSite = document.createElement("div");
		buttonSite.style.margin = "8px 8px 6px 0";
		buttonSite.appendChild(this._previewBtn);
		buttonSite.appendChild(this._okBtn);
		buttonSite.appendChild(cancelBtn);

		var bottomDiv = document.createElement("div");
		bottomDiv.style.textAlign = "right";
		bottomDiv.style.margin = "14px 0 0 0";
		bottomDiv.style.borderTop = "solid #DDDDDD 1px";
		bottomDiv.appendChild(buttonSite);

		this._reportParametersPanel.setBottomButtonsSection(bottomDiv);
		
		// Create right preview section
		var previewDiv = document.createElement("div");
		previewDiv.style.position = "relative";
		var previewLabelDiv = document.createElement("div");
		this._previewContentDiv = document.createElement("div");
		this._previewContentDiv.className = "sectionContent";
		previewLabelDiv.className = "sectionTitle";
		previewLabelDiv.innerHTML = this.messages.msgPreviewLabel;
		previewDiv.appendChild(previewLabelDiv);
		previewDiv.appendChild(this._previewContentDiv);
		previewDiv.appendChild(this._createClosePreviewButton());
		this._reportParametersPanel.setMainSectionRight(previewDiv);
		
		// Create top name field
		this._nameField = document.createElement("input");
		this._nameField.type = "text";
		this._nameField.size = 40;
		this._nameField.value = this.context.getSite().getTitle();
		this._nameField.style.margin = "10px 0 1px 7px";
		this.connect(this._nameField, "onblur", this.handleEventNameFieldBlurred);
		
		this._reportParametersPanel.setTopInputFieldSection(this._nameField);
		this._reportParametersPanel.focusOnOpen(this._nameField);
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	handleEventPreviewButtonClicked: function(event) {
		dojo.stopEvent(event);
		
		// Show preview pane
		this.showPreviewPane(true);
		// Stop subsequent clicks
		this._previewBtn.disabled = true;
		// Clear previous 'preview'
		this._previewContentDiv.innerHTML = "";
			
		var loadingDiv = document.createElement("div");
		loadingDiv.className = "progress-icon";
		this._previewContentDiv.appendChild(loadingDiv);
			
		var selections = this._getParameterSelections();
		if (!selections) {
			// Missing required parameter
			this._previewBtn.disabled = false;
			this._previewContentDiv.innerHTML = "";
			return true;
		} else if (!this.projectAreaUUID) {
			this.context._logError(new Error("Project area is not defined"));
			this._previewBtn.disabled = false;
			this._previewContentDiv.innerHTML = "";
			return true;
		}
			
		var self = this;
		var serviceObject = {
			success: function(chartHTML){
				// TODO if cancelled ...
				self._previewBtn.disabled = false;
				self._previewContentDiv.innerHTML = chartHTML;
			},
			failure: function(errorObject){
				self._previewBtn.disabled = false;
				self._previewContentDiv.innerHTML = "";
				self.context._logError(errorObject);
				self.context._displayErrorMessage(self.messages.msgReportParameterLoadingFailed, errorObject);
			}
		};
		
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		this._callRESTPostRenderReport(
			handler, 
			this.context._reportUUID, 
			this.projectAreaUUID,
			selections.parameterNames, 
			selections.parameterValues);
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	handleEventNameFieldBlurred: function(event) {
		dojo.stopEvent(event);
		
		if (!this._nameField.value) {
			this._nameField.value = this.context.getSite().getTitle();
		}
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventOKButtonClicked: function(event) {
		dojo.stopEvent(event);
		
		this._saveQuery();
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventCancelButtonClicked: function(event) {
		dojo.stopEvent(event);

		this._reportParametersPanel.closeDialog();
		
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventDialogClosed: function() {
		// Events:
		// OK button clicked
		// Cancel button clicked
		// X button clicked
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */
	_createClosePreviewButton: function() {
		
		this._closeButton = document.createElement("img");
		this._closeButton.alt = "X";
		this._closeButton.src = dojo.moduleUrl("com.ibm.team.reports.viewlets.web", "graphics/shared/images/close.gif");
		this._closeButton.style.cursor = "pointer";
		
		this.connect(this._closeButton, "onclick", this.handleEventClosePreviewClicked);		
		this.connect(this._closeButton, "onmouseover", this.handleEventClosePreviewMouseOver);
		this.connect(this._closeButton, "onmouseout", this.handleEventClosePreviewMouseOut);
		
		var closeButtonSite = document.createElement("div");
		closeButtonSite.style.position = "absolute";
		closeButtonSite.style.right = "5px";
		closeButtonSite.style.top = 0;

		closeButtonSite.appendChild(this._closeButton);
		
		return closeButtonSite;
	},	

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	handleEventClosePreviewClicked: function(event) {
		dojo.stopEvent(event);
		
		// Hide preview pane
		this.showPreviewPane(false);
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	handleEventClosePreviewMouseOver: function(event) {
		dojo.stopEvent(event);
		
		// Highlight on
		this._closeButton.src = dojo.moduleUrl("com.ibm.team.reports.viewlets.web", "graphics/shared/images/close-hov.gif");		
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	handleEventClosePreviewMouseOut: function(event) {
		dojo.stopEvent(event);
		
		// Highlight off
		this._closeButton.src = dojo.moduleUrl("com.ibm.team.reports.viewlets.web", "graphics/shared/images/close.gif");		
		
		return true;
	},

	/*
	 * @param Boolean show
	 * 
	 * @return void
	 */			
	showPreviewPane: function(show) {
		this._reportParametersPanel.showPreviewPane(show);
	},

	/*
	 * @param Function onOk(String value, String prefId)
	 * @param Object settings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
	 * @param Object viewlet
	 * 
	 * @return void
 	 * @overrides super
 	 */		
	createChooser: function(onOk, settings, viewlet) {
		
		var self = this;
		this._onPrefOk = onOk;
		this.projectAreaUUID = settings.scopeItemId;
		this._reportParams = [];
		
		// TODO: MP: No access to project area name
		
		if (!this.projectAreaUUID) {
			alert(this.messages.msgSelectProject);
			return;
		} else if (this._isReportDeployed() === false) {
			var message = dojo.string.substitute(this.messages.msgReportNotDeployed2, [this.context._reportUUID]);
			alert(message);
			return;
		}
		
		self._createParametersPanel();
		self._reportParametersPanel.showAsDialog(dojo.hitch(this, this._handleEventDialogClosed));
		
		// Create the service object
		var paramsServiceObject = {
			// Service success
			success: function(baseParamsArray){
				dojo.forEach(baseParamsArray, dojo.hitch(this, function(baseParam){
				
					if (baseParam._eQualifiedClassName == "com.ibm.team.reports.dto:ReportParameterGroupDTO") {
						var reportParamGroup = new com.ibm.team.reports.web.ui.internal.ReportParamGroup({
							parameterGroupDTO: baseParam,
							parameterName: self.parameterName,
							parameterValue: self.parameterValue,
							urlPrefix: "../../",
							nestedStyledBox: true
						});
						self._reportParametersPanel.appendParamGroup(reportParamGroup);
						reportParamGroup.appendReportParameters(self._reportParams);
						
						// Create the service object
						var serviceObject = {
							// Service success
							success: function(reportParameterGroupDTO){
								reportParamGroup.initGroupValues(reportParameterGroupDTO);
							},
							// Service failure
							failure: function(errorObject){
								self.context._logError(errorObject);
								self.context._displayErrorMessage(self.messages.msgReportParameterLoadingFailed, errorObject);
							}
						};
						
						var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
						self._callRESTGetParameterGroupValues(handler, self.context._reportUUID, self.projectAreaUUID, baseParam.name);
					}
					else 
						if (baseParam._eQualifiedClassName == "com.ibm.team.reports.dto:ReportParameterDTO") {
							var reportParam = ReportParamFactory.create(baseParam, "../../");
							if (!baseParam.concealed) {
								self._reportParametersPanel.appendParam(reportParam);
							}
							self._reportParams.push(reportParam);
							
							// Create the service object
							var serviceObject = {
								// Service success
								success: function(reportParameterDTO){
									reportParam.init(reportParameterDTO);
									if (self.parameterName && self.parameterName.length > 0) {
										for (var i = 0; i < self.parameterName.length; ++i) {
											if (self.parameterName[i] == reportParam.parameterDTO.name) {
												reportParam.setValues(self.parameterValue[i]);
												break;
											}
										}
									}
									reportParam.postInit();
								},
								// Service failure
								failure: function(errorObject){
									self.context._logError(errorObject);
									self.context._displayErrorMessage(self.messages.msgReportParameterLoadingFailed, errorObject);
								}
							};
							
							var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
							self._callRESTGetParameterValues(handler, self.context._reportUUID, self.projectAreaUUID, baseParam.name);
						}
				}));
				// Reposition the dialog to be vertically centered on the screen 
				// after the parameter boxes have loaded
				self._reportParametersPanel._dialog.updatePosition();
			},
			// Service failure
			failure: function(errorObject){
				self.context._logError(errorObject);
				/* TODO update loading message */
				self.context._displayErrorMessage(self.messages.msgReportLoadingFailed, errorObject);
				self.context.getSite().doneRefresh();
			}
		};
		
		var curQueryUUID = this.context._getReportQueryUUID();
		if (this.context._isValidUUID(curQueryUUID)) {
			// We have an existing query uuid so we must fetch its associated parameters to
			// select them in the dialog
			var serviceObject = {
				success: function(queryDescriptor) {
					self.parameterName = queryDescriptor.parameterNames;
					self.parameterValue = queryDescriptor.parameterValues;
					// Create the handler
					var handler = new ServiceResponseHandler(paramsServiceObject, "success", "failure");
					// Invoke the service
					self._callRESTGetParameters(handler, self.context._reportUUID, self.projectAreaUUID);
					
				},
				failure: function(errorObject) {
					self.context._logError(errorObject);
					self.context._displayErrorMessage(self.messages.msgReportParameterLoadingFailed, errorObject);
				}
			};
			var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
			this._callRESTGetQueryDescriptor(handler, curQueryUUID);
		} else {
			// Create the handler
			var handler = new ServiceResponseHandler(paramsServiceObject, "success", "failure");
			// Invoke the service
			this._callRESTGetParameters(handler, this.context._reportUUID, this.projectAreaUUID);
		}
	},	

	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */		
	_isReportDeployed: function() {
		// Note: There is a chance that the user opens the settings area before
		// the REST service request that populates the project area white list 
		// returns. In this case, the report will be classified as undeployed.
		if (!this.projectAreaUUID) {
			return false;
		} else if ((this.projectAreaUUID in this.context._projectAreaWhitelist.entries) === false) {
			return false;
		}
		return true;
	},

	/*
 	 * @param String projectAreaUUID
 	 * @param String queryUUID
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	getParametersMetaData: function(projectAreaUUID, queryUUID, callback) {
		
		var self = this;
		var serviceObject = {
			success: function(queryDescriptor) {
				self.getNamesMetaData(queryDescriptor, projectAreaUUID, callback);
			},
			failure: function(errorObject) {
				self.context._logError(errorObject);
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		this._callRESTGetQueryDescriptor(handler, queryUUID);		
	},

	/*
 	 * @param Object data
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	getPrimedValuesQuery: function(data, callback) {
		this._savePrimedValuesQuery(data, callback);
	},

	/*
 	 * @param Array data 
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	_savePrimedValuesQuery: function(data, callback) {

		// TEST
		//console.debug(data);

		if (!this.projectAreaUUID) {
			this.context._logError(new Error("Project area is not defined"));
			return;
		}		
		
		var normalizedData = this._normalizePrimedValuesData(data);
		
		var self = this;
		var serviceObject = {
			success: function(queryUUID) {
				callback(queryUUID);
			},
			failure: function(errorObject) {
				self.context._logError(errorObject);
				callback(null);
			}
		};
		
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		var queryUUID = this.context._getReportQueryUUID();
		
		this._callRESTPostSaveQueryDescriptor(
			handler, 
			this.context._reportUUID,
			this.projectAreaUUID,
			queryUUID,
			normalizedData.names, 
			normalizedData.values, 
			this.context.getSite().getTitle(),
			true); // keepInTempStorage		

	},

	/*
	 * @param Array data
	 * 
	 * @return Object
	 */
	_normalizePrimedValuesData: function(data) {
		
		var normalizedData = {
			names: [],
			values: []
		};
		
		for (var index = 0; index < data.length; index++) {
			var parameter = data[index]; 
			normalizedData.names.push(parameter.name);
			normalizedData.values.push(parameter.value);
		}
		
		return normalizedData;
	},	

	/*
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	getDefaultValuesQuery: function(callback) {
		var self = this;
		this.getDefaultValues(function(data) {
			self._saveDefaultValuesQuery(data, callback);
		});
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_debugDefaultValues: function() {
		
		// e.g.
		//
		//<preference id="report" value="workitems.MicroBurndown"/>
		//<memento>
		//    <property name="parameters" type="xsd:string">
		//        <memento>
		//            <property name="name" type="xsd:string">ProjectAreaName</property>
		//            <property name="value" type="xsd:string">'{Current Project Area}'</property>
		//        </memento>
		//        <memento>
		//            <property name="name" type="xsd:string">TeamAreaName</property>
		//            <property name="value" type="xsd:string">'{My Team Areas}'</property>
		//        </memento>
		//        <memento>
		//            <property name="name" type="xsd:string">ProjectAreaName1</property>
		//            <property name="value" type="xsd:string">'{Current Project Area}'</property>
		//        </memento>
		//        <memento>
		//            <property name="name" type="xsd:string">Interval</property>
		//            <property name="value" type="xsd:string">'{Current Iteration}'</property>
		//        </memento>
		//        <memento>
		//            <property name="name" type="xsd:string">Zoom</property>
		//            <property name="value" type="xsd:string">false</property>
		//        </memento>
		//    </property>
		//</memento>		
		
		var self = this;
		this.getDefaultValues(function(data) {
			console.log("= START ================================>");
			
			console.log('<preference id="report" value="' + self.context._reportUUID + '"/>');
			console.log('<memento>');
			
			console.log("\t" + '<property name="parameters" type="xsd:string">');
			
			for (var index = 0; index < data.length; index++) {
				var name = data[index].name;
				var value = self._normalizeValue(data[index].defaultValue);
				
				console.log("\t\t" + '<memento>');
				console.log("\t\t\t" + '<property name="name" type="xsd:string">' + name + '</property>');
				console.log("\t\t\t" + '<property name="value" type="xsd:string">' + value + '</property>');
				console.log("\t\t" + '</memento>');
			}

			console.log("\t" + '</property>');
			console.log('</memento>');
			console.log("= END ==================================>");
		});
	},

	/*
	 * @param Array data
	 * 
	 * @return Object
	 * @throws Error
	 */
	_normalizeDefaultValuesData: function(data) {
		
		var normalizedData = {
			names: [],
			values: []
		};
		
		for (var index = 0; index < data.length; index++) {
			var parameter = data[index]; 
		
			if (parameter.required &&
				!parameter.defaultValue) {
				throw new Error("Required parameter has no default value: " + parameter.name);
			} 
			
			normalizedData.names.push(parameter.name);
			normalizedData.values.push(this._normalizeValue(parameter.defaultValue));
		}
		
		return normalizedData;
	},		
	
	/*
 	 * @param String value
 	 * 
 	 * @return String
 	 */	
	_normalizeValue: function(value) {
		
		if (!value) {
			// All
			return "";
		} else if (value === "true") {
			// Boolean
			return value;
		} else if (value === "false") {
			// Boolean
			return value;
		} else if (value.charAt(0) === "'") {
			// Quoted
			return value;
		}
		// TODO: MP: Worry about special variables? Strip {}
		
		return "'" + value + "'";
	},	

	/*
 	 * @param Array data 
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	_saveDefaultValuesQuery: function(data, callback) {

		// TEST
		//console.debug(data);

		if (!this.projectAreaUUID) {
			this.context._logError(new Error("Project area is not defined"));
			callback(null);
			return;
		}

		var normalizedData;
		try {
			normalizedData = this._normalizeDefaultValuesData(data);
		} catch (errorObject) {
			this.context._logInfo(errorObject);
			callback(null);
			return;
		}
		
		var self = this;
		var serviceObject = {
			success: function(queryUUID) {
				callback(queryUUID);
			},
			failure: function(errorObject) {
				self.context._logError(errorObject);
				callback(null);
			}
		};
		
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		var queryUUID = this.context._getReportQueryUUID();		
		
		this._callRESTPostSaveQueryDescriptor(
			handler, 
			this.context._reportUUID,
			this.projectAreaUUID,
			queryUUID,
			normalizedData.names, 
			normalizedData.values, 
			this.context.getSite().getTitle(),
			true); // keepInTempStorage		

	},

	/*
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	getDefaultValues: function(callback) {
		
		if (!this.projectAreaUUID) {
			this.context._logError(new Error("Project area is not defined"));
			return;
		}		
		
		var self = this;
		var serviceObject = {
			success: function(baseParamsArray) {
				var data = self._processDefaultBaseParams(baseParamsArray);
				callback(data);
			},
			failure: function(errorObject) {
				self.context._logError(errorObject);
				callback(null);
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		
		this._callRESTGetParameters(handler, this.context._reportUUID, this.projectAreaUUID);
	},

	/*
 	 * @param Array baseParamsArray
 	 * 
 	 * @return Array
 	 */	
	_processDefaultBaseParams: function(baseParamsArray) {
		// TEST
		//console.debug(baseParamsArray);
		
		var data = [];
		
		if ((baseParamsArray instanceof Array) === false) {
			return data;
		}
		for (var index = 0; index < baseParamsArray.length; index++) {
			var baseParameter = baseParamsArray[index];
			
			if (!baseParameter) {
				this.context._logWarning(new Error("Base parameter not defined"));
			} else if (baseParameter._eQualifiedClassName === this._static._GROUP_PARAMETER) {
				this._processDefaultGroupBaseParam(baseParameter, data);
			} else if (baseParameter._eQualifiedClassName === this._static._SINGLE_PARAMETER) {
				this._processDefaultSingleBaseParam(baseParameter, data);
			} else {
				this.context._logWarning(new Error("Unrecognized qualified class name: " + baseParameter._eQualifiedClassName));
			}
		}
		
		return data;
	},

	/*
 	 * @param Object baseParameter
 	 * @param Array data
 	 * 
 	 * @return void
 	 */	
	_processDefaultSingleBaseParam: function(baseParameter, data) {
		
		if (!baseParameter.name) {
			this.context._logWarning(new Error("Base parameter name not defined"));
			return;
		}
		
		// TEST
		//console.debug(baseParameter);
		
		// Add parameter
		data.push({
			name: baseParameter.name,
			required: baseParameter.required,
			defaultValue: baseParameter.defaultValue
		});
	},

	/*
 	 * @param Object baseParameter
 	 * @param Array data
 	 * 
 	 * @return void
 	 */	
	_processDefaultGroupBaseParam: function(baseParameter, data) {
		
		if ((baseParameter.parameters instanceof Array) === false) {
			this.context._logWarning(new Error("Group has no parameters"));
			return;
		}
		
		for (var index = 0; index < baseParameter.parameters.length; index++) {
			var parameter = baseParameter.parameters[index];
			this._processDefaultSingleBaseParam(parameter, data);
		}
	},

	/*
 	 * @param Object queryDescriptor
 	 * @param String projectAreaUUID
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	getNamesMetaData: function(queryDescriptor, projectAreaUUID, callback) {
		// TEST
		//console.debug(queryDescriptor);		
		
		if (!queryDescriptor) {
			this.context._logError(new Error("Query descriptor not defined"));
			return;
		} else if (!queryDescriptor.reportUUID) {
			this.context._logError(new Error("Report UUID not defined"));
			return;
		} else if (!projectAreaUUID) {
			this.context._logError(new Error("Project area is not defined"));
			return;
		}
		
		var self = this;
		var serviceObject = {
			success: function(baseParamsArray) {
				var data = self.processBaseParams(queryDescriptor, baseParamsArray);
				callback(data);
			},
			failure: function(errorObject) {
				self.context._logError(errorObject);
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		
		this._callRESTGetParameters(handler, queryDescriptor.reportUUID, projectAreaUUID);
	},

	/*
 	 * @param Object queryDescriptor
 	 * @param Array baseParamsArray
 	 * 
 	 * @return Object
 	 */	
	processBaseParams: function(queryDescriptor, baseParamsArray) {
		// TEST
		//console.debug(baseParamsArray);
		
		var data = {};
		
		if ((baseParamsArray instanceof Array) === false) {
			return data;
		}
		for (var index = 0; index < baseParamsArray.length; index++) {
			var baseParameter = baseParamsArray[index];
			
			if (!baseParameter) {
				this.context._logWarning(new Error("Base parameter not defined"));
			} else if (baseParameter._eQualifiedClassName === this._static._GROUP_PARAMETER) {
				this._processGroupBaseParam(queryDescriptor, baseParameter, data);
			} else if (baseParameter._eQualifiedClassName === this._static._SINGLE_PARAMETER) {
				this._processSingleBaseParam(queryDescriptor, baseParameter, data);
			} else {
				this.context._logWarning(new Error("Unrecognized qualified class name: " + baseParameter._eQualifiedClassName));
			}
		}
		
		return data;
	},

	/*
 	 * @param Object queryDescriptor
 	 * @param Object baseParameter
 	 * @param Object data
 	 * 
 	 * @return void
 	 */	
	_processSingleBaseParam: function(queryDescriptor, baseParameter, data) {
		
		if (!baseParameter.name) {
			this.context._logWarning(new Error("Base parameter name not defined"));
			return;
		} else if (baseParameter.concealed) {
			return;
		} else if ((queryDescriptor.parameterNames instanceof Array) === false) {
			this.context._logWarning(new Error("Parameter names not defined"));
			return;
		} else if ((queryDescriptor.parameterValues instanceof Array) === false) {
			this.context._logWarning(new Error("Parameter values not defined"));
			return;
		} else if (queryDescriptor.parameterNames.length !== queryDescriptor.parameterValues.length) {
			this.context._logWarning(new Error("Parameter name count does not equal parameter value count"));
			return;
		}
		
		var index = 0;
		
		// Get values
		var rawValues = null;
		for (index = 0; index < queryDescriptor.parameterNames.length; index++) {
			// e.g.
			// TeamAreaName
			// WhichBuilds
			var parameterName = queryDescriptor.parameterNames[index];
			if (parameterName === baseParameter.name) {
				// e.g.
				// "'/Jazz Development/Reports','/Jazz Development/Repository','/Jazz Development/Work Item'"
				// "'1 Last 10 Builds'"
				// ""
				// "false"
				rawValues = queryDescriptor.parameterValues[index];
				break;
			}
		}
		
		if (typeof(rawValues) !== "string") {
			this.context._logWarning(new Error(
				"Report template parameter '" + 
				baseParameter.name + 
				"' not found in the report query. The report template (new) is out of sync with the report query (old)."));
			return;
		}
		
		var values;
		if (rawValues.length === 0) {
			// Empty string "" (All)
			values = [""];
		} else {
			// Split values (if more than one value)
			values = rawValues.split(",");
			// Remove single quotes (if any)
			var pattern = /'(.*)'/; 
			for (index = 0; index < values.length; index++) {
				var matches = values[index].match(pattern);
				if ((matches instanceof Array) &&
					(matches.length > 1)) {
					values[index] = matches[1];
				}
			}
		}
		
		var displayName;
		if (baseParameter.displayName) {
			displayName = baseParameter.displayName;
		} else {
			this.context._logInfo(new Error("No display name: " + baseParameter.name));
			displayName = baseParameter.name;
		}
		
		// Add parameter
		data[baseParameter.name] = {
			displayName: displayName,
			values: values
		};
		
	},

	/*
 	 * @param Object queryDescriptor
 	 * @param Object baseParameter
 	 * @param Array data
 	 * 
 	 * @return void
 	 */	
	_processGroupBaseParam: function(queryDescriptor, baseParameter, data) {
		
		if ((baseParameter.parameters instanceof Array) === false) {
			this.context._logWarning(new Error("Group has no parameters"));
			return;
		}
		
		for (var index = 0; index < baseParameter.parameters.length; index++) {
			var parameter = baseParameter.parameters[index];
			this._processSingleBaseParam(queryDescriptor, parameter, data);
		}
	},

	/*
 	 * @param String text
 	 * @param String style
 	 * 
 	 * @return String
 	 */	
	_buildLabel: function (text, style) {
		var html = "<input type='text' readonly='readonly' value='" + 
					text.replace("'","&#39;") + 
					"'";
		if (style) {
			html = html + 
					" style='" +
					style +
					"'"; 
		}
		html = html + "/>"; 
		
		return html;
	},

	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String queryUUID
 	 * 
 	 * @return void
 	 */	
	_callRESTPostDeleteQueryDescriptor: function(handler, queryUUID) {
		var arguments = {
			queryUUID: queryUUID,
			onlyIfInTempStorage: true
		};
		this._callRESTReportsService(handler, "postDeleteQueryDescriptor", arguments);
	},

	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String projectAreaUUID
 	 * @param String queryUUID
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * @param String queryName
 	 * @param Boolean keepInTempStorage
 	 * 
 	 * @return void
 	 */	
	_callRESTPostSaveQueryDescriptor: function(handler, reportUUID, projectAreaUUID, queryUUID, parameterNames, parameterValues, queryName, keepInTempStorage) {
		var arguments = {
			projectAreaUUID: projectAreaUUID,
			reportUUID: reportUUID,
			queryUUID: queryUUID,
			parameterName: parameterNames,
			parameterValue: parameterValues,
			name: queryName,
			keepInTempStorage: keepInTempStorage,
			sharedIfNullFolder: true
		};
		this._callRESTReportsService(handler, "postSaveQueryDescriptor", arguments);
	},
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String projectAreaUUID
 	 * 
 	 * @return void
 	 */	
	_callRESTGetParameters: function(handler, reportUUID, projectAreaUUID) {
		var arguments = {
			projectAreaUUID: projectAreaUUID,
			reportUUID: reportUUID
		};
		this._callRESTReportsService(handler, "getParameters", arguments);
	},
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String queryUUID
 	 * 
 	 * @return void
 	 */	
	_callRESTGetQueryDescriptor: function(handler, queryUUID) {
		var arguments = {
			queryUUID: queryUUID
		};
		this._callRESTReportsService(handler, "getQueryDescriptor", arguments);
	},
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String projectAreaUUID
 	 * @param String parameterName
 	 *  
 	 * @return void
 	 */	
	_callRESTGetParameterValues: function(handler, reportUUID, projectAreaUUID, parameterName) {
		var arguments = {
			reportUUID: reportUUID,
			parameter: parameterName,
			showArchived: false,
			projectAreaUUID: projectAreaUUID
		};
		this._callRESTReportsService(handler, "getParameterValues", arguments);
	},		
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String projectAreaUUID
 	 * @param String groupName
 	 *  
 	 * @return void
 	 */	
	_callRESTGetParameterGroupValues: function(handler, reportUUID, projectAreaUUID, groupName) {
		var arguments = {
			reportUUID: reportUUID,
			groupName: groupName,
			showArchived: false,
			projectAreaUUID: projectAreaUUID
		};
		this._callRESTReportsService(handler, "getParameterGroupValues", arguments);
	},
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String projectAreaUUID
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * 
 	 * @return void
 	 */	
 	_callRESTPostRenderReport: function(handler, reportUUID, projectAreaUUID, parameterNames, parameterValues) {
		var arguments = {
			projectAreaUUID: projectAreaUUID,
			reportUUID: reportUUID,
			parameterName: parameterNames,
			parameterValue: parameterValues,
			showArchived: false,
			urlPrefix: this.context._getImageServerRoot(),
			serverURL: this.context._getDrillInServerRoot()
		};
		// TODO: MP: Disable drill-in link for preview images 
		// BUG: Drill-in link for remote viewlets can get out of sync.
		// e.g. User selects a new project area, opens the report parameter
		// dialog and previews (old project area is used for the project area
		// name)
		this._callRESTReportsService(handler, "postRenderReport", arguments);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param ServiceResponseHandler handler
 	 * @param String method
 	 * @param Object arguments
 	 * @return void
 	 */	
	_callRESTReportsService: function(handler, method, arguments) {
		// Create the service request
		var request = new ServiceRequest(
					REPORTS_REST_SERVICE_URI,
					method,
					arguments);
		this.context.getServiceTracker().register(request, handler);
		// Invoke the service
		TeamServerClient.invokeService(request, handler);
	}
	
});

})();
