/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.scm.web.ui.internal.view.browse.UserLocks");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojox.data.dom");
dojo.require("jazz.ui.HoverPopup");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.scm.web.client.util");
dojo.require("com.ibm.team.scm.web.client.internal.ScmClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function(){
    var ScmUtil = com.ibm.team.scm.web.client.util;
    var ScmClient = com.ibm.team.scm.web.client.internal.ScmClient;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    /**
     * Widget to display "User Locks" link.
     */
    dojo.declare("com.ibm.team.scm.web.ui.internal.view.browse.UserLocks", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/view/browse/templates/UserLocks.html"),
        
        constructor: function(){
            this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
            this.controller = null;
            this._userLocksData = {
				// Number of locks requested from server
				maxLocksRequested: 512,
				locksDTO: null,
				// Number of known locks 
				versionableDTOsCount: 0,
				// Number of locks to display initially
				maxShowPartial: 10,
				showPartial: true
            };
        },
        
        postCreate: function(){
			this.connect(this.controller, "onUpdateLocks", this.refresh);
			this.connect(this.controller, "onUpdateStream", this.refresh);
        },
		setNavigationSection: function (navigationSection) {
			this._navigationSection = navigationSection; 
		},
        /**
         * Recompute the locks and update the UI
         */
        refresh: function(){
			this.updateLocksDTO()
			.addCallback(dojo.hitch(this, this.redraw))
			.addCallback(dojo.hitch(this._navigationSection, this._navigationSection.update));
        },
		updateLocksDTO: function () {
            var serviceResponseHandler, serviceRequest, args, that, d;
            d = new dojo.Deferred();
            that = this;
            serviceResponseHandler = new ServiceResponseHandler({
                success: function(result){
					// First call
					if (!that._userLocksData.locksDTO) {
	                    that._userLocksData.locksDTO = result;
					}
					// Update if lock time is different
					if (that._userLocksData.locksDTO.contributorLockTime !== result.contributorLockTime) {
						that._userLocksData.locksDTO = result;
					}
                    d.callback(true);
                },
                failure: function(errorObject){
                    d.errback(new Error(""));
                }
            }, "success", "failure");
            serviceRequest = new com.ibm.team.repository.web.transport.ServiceRequest(ScmClient.SERVICE_URI_2, "getLocks", {
                lockedByContributorId: ScmUtil.getLoggedInContributor().itemId,
				lockTime: that._userLocksData.locksDTO ? that._userLocksData.locksDTO.contributorLockTime : 0,
                length: that._userLocksData.maxLocksRequested
            });
            com.ibm.team.repository.web.transport.TeamServerClient.invokeService(serviceRequest, serviceResponseHandler);
            return d;			
		},
		redraw: function() {
			var _userLocksData, wIndex, workspaceLocksDTO, cIndex, componentLocksDTO, clIndex, contributorLocksDTO, contributorDTO, vIndex, versionableDTO;
			dojox.data.dom.removeChildren(this._userLocks);
			if (!this._userLocksData.locksDTO) {
				// No information available yet
				return;
			}
			if (!this._userLocksData.locksDTO.workspaceLocksDTO) {
				// User has no locks
				return;
			}
			this._userLocksData.versionableDTOsCount = 0;
			for (wIndex = 0; wIndex < this._userLocksData.locksDTO.workspaceLocksDTO.length; wIndex += 1) {
				workspaceLocksDTO = this._userLocksData.locksDTO.workspaceLocksDTO[wIndex];
				for (cIndex = 0; cIndex < workspaceLocksDTO.componentLocksDTO.length; cIndex += 1) {
					componentLocksDTO = workspaceLocksDTO.componentLocksDTO[cIndex];
					for (clIndex = 0; clIndex < componentLocksDTO.contributorLocksDTO.length; clIndex += 1) {
						contributorLocksDTO = componentLocksDTO.contributorLocksDTO[clIndex];
						contributorDTO = contributorLocksDTO.contributorDTO;
						for (vIndex = 0; vIndex < contributorLocksDTO.versionableDTOs.length; vIndex += 1) {
							versionableDTO = contributorLocksDTO.versionableDTOs[vIndex];
							this._userLocksData.versionableDTOsCount += 1;
							if (!this._userLocksData.showPartial || this._userLocksData.versionableDTOsCount <= this._userLocksData.maxShowPartial) {
								this._addLockedResourceLink({
									parentNode: this._userLocks,
									versionableDTO: versionableDTO,
									componentDTO: componentLocksDTO.componentDTO,
									workspaceDTO: workspaceLocksDTO.workspaceDTO
								});								
							}
						}
					}
				}
			}
		},
		hasShowPartial: function() {
			return this._userLocksData.versionableDTOsCount > this._userLocksData.maxShowPartial;
		},
		
		onShowPartial: function() {
			this._userLocksData.showPartial = true;
			this.redraw();
		},
		
		onMaximize: function() {
			this._userLocksData.showPartial = false;
			this.redraw();
		},
		onMinimize: function() {
		},
        /**
         * Args: parentNode, versionableDTO, componentDTO, workspaceDTO (stream)
         */
        _addLockedResourceLink: function(args){
            var node, anchor, img, versionableDTO, hover, that;
            versionableDTO = args.versionableDTO;
            node = document.createElement("div");
            node.className = "com-ibm-team-scm-userLocks-lockedEntry";
			img = ScmUtil.getDecoratedResourceIcon(
			{
				fileName : versionableDTO.versionable.name,
				isFolder : versionableDTO.versionable["com.ibm.team.repository.typeName"] === "com.ibm.team.scm.Folder",
				isSymbolicLink : versionableDTO.versionable["com.ibm.team.repository.typeName"] === "com.ibm.team.filesystem.SymbolicLink",
				isLockedByMe : true
			}
			);
            anchor = document.createElement("a");
            anchor.href = ScmUtil.getBrowseElementURI(versionableDTO.workspace.itemId, versionableDTO.component.itemId, versionableDTO.path);
            anchor.appendChild(document.createTextNode(versionableDTO.versionable.name));
            node.appendChild(img);
            node.appendChild(anchor);
            args.parentNode.appendChild(node);
			that = this;
			hover = new jazz.ui.HoverPopup({ 
				around: [ node ],
				createContents: function () {
					var result;
					result = document.createElement("div");
					result.appendChild(document.createTextNode(args.workspaceDTO.workspace.name));
					result.appendChild(document.createTextNode(" > "));
					result.appendChild(document.createTextNode(args.componentDTO.component.name));
					return result;
				},
				orientation: "R"}); // tooltip to show locked by info.
			node.hover = hover;
        }
    });
    
})();
