/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalWidget");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dojo.fx");

dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionListWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.TimestampWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget");
dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.LabelWidget");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
/*
 * This is a wrapper widget that shows the Approvals Section
 */
(function() {

var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var WorkItemUtil			= com.ibm.team.workitem.web.client.util;
var ImagePool = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var SelectContributorDialog = com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog;

var stateMap = {
	com_ibm_team_workitem_approvalstate_pending: "Pending",
	com_ibm_team_workitem_approvalstate_approved: "Approved",
	com_ibm_team_workitem_approvalstate_rejected: "Rejected"
};
var headingColor = "#666668";
var columnLayout;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalWidget",
	[dijit._Widget, dijit._Templated],
	{
		
		templateString: '<DIV class="ApprovalWidget">' +
						'<span class="ApprovalList" dojoAttachPoint="_listNode"></span>' +
						'</DIV>',

		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
	 		columnLayout = {		
				approvalType: {
						headingLabel: this.messages.labelType,
						headingStyle: { },
						attributes: { },
						style: { width: "9.2em" }
				}, 
				name: {
						headingLabel: "Name",
						headingStyle: { },
						style: { width: "60%" }
				},
				state:{
						headingLabel: this.messages.labelState,
						headingStyle: { },
						style: { width: "40%" },
						headingAttributes: { },
						attributes: { }
				},
				dueDate: {
						headingLabel: this.messages.labelDue,
						headingStyle: { },
						style: { width: "15em" }					
				}
			}
		},
		postCreate: function() {
			// Model:
			// this.approval
			//       .name (250 chars, required)
			//       .cumulativeState
			//       .dueDate
			//       .approvals
			//           .approver.name
			//           .state
			// this.approvalTypes
			// this.approvalStates

			this.refresh();		
		},	
		destroyWidget: function(){
			this.resetWidget();
			this.destroy();
		},
		resetWidget: function(){
			if(this.sectionListWidget) {
				this.sectionListWidget.destroyWidget();
				this.sectionListWidget = null;
			}
			if(this.nameWidget) this.nameWidget.destroy();		
		},
		
		refresh: function() {
			this.resetWidget();
			
			dojox.data.dom.removeChildren(this._listNode);
						
			var creationArgs;
			if (this.isHeading) {
				if (this.isFooter) {
					creationArgs = {
						parentWidget: this,
						parentController: this.parentController, 
						noBorder: true,
						heading: {
							showDelete: false
						}
					};
				}
				else {
					// Add an action to show a heading line
					var headingWidgets = this._getColumnHeadingWidgets();
					creationArgs = {
						parentWidget: this,
						parentController: this.parentController,
						isBox: true,
						isHeading: true,
						isFirst: true,
						heading: {
							columnWidgets: headingWidgets,
							showDelete: false
						}
					};
				}

				this.sectionListWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.SectionListWidget(creationArgs);
				this._listNode.appendChild(this.sectionListWidget.domNode);
				return;
			}
			var itemList = [];
			
			
			if (!this._isDeleted) {
				if (this.approval.approvals) {
					var sortedApprovals = this.approval.approvals.sort( function(a, b ) {
						if(!a || !a.approver || !a.approver.name) return -1;
						if(!b || !b.approver || !b.approver.name) return 1;
						if(a.approver.name < b.approver.name) return -1;
						return 1;
						
					})
					for (var i = 0; i < sortedApprovals.length; i++) {
						var isLast = i == sortedApprovals.length;
						if (sortedApprovals[i] && (sortedApprovals[i].approver)) {
							itemList.push(this._getApproverWidgets(sortedApprovals[i], /* isFirst */ i == 0, isLast));
						}
					}
				}
				
				// Add an action to add more approvers
				if (!this.workingCopy.isGuest()) {
					itemList.push({
						columnWidgets: this._getAddApproverActionWidgets(),
						showDelete: false
					});
				}
				
			}
			creationArgs = {
						parentWidget: this,
						parentController: this.parentController,
						heading: { columnWidgets: this._getApprovalWidgets(),
						           value: this.approval,
								   deleteAction: {
										selector: "deleteApproval",
										title: this.messages.titleApprovalDelete
										},
								   restoreAction: {
										selector: "restoreApproval",
										title: this.messages.titleApprovalRestore
										}
								},
						rows: { 	itemList: itemList,
									deleteAction: {
											selector: "deleteApprover",
											title: this.messages.titleApproverDelete
										},
								   restoreAction: {
										selector: "restoreApprover",
										title: this.messages.titleApproverRestore
										}
								}
					};
			this.sectionListWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.SectionListWidget(creationArgs);
			this.sectionListWidget.domNode.actionWidget = this;
			this._listNode.appendChild(this.sectionListWidget.domNode);
			this.domNode.widget = this;
			this.domNode.__addApproverAnchor = this.addApproverAnchor; 

// The following is used to optimize many approvals.
//			this.parentController.pageController.addMouseListener(this, this.sectionListWidget.domNode,  { } );

		},
//		handlePageMouseMove: function(e, listener, isTargetChildOfDomNode) {
//			this._setAnchorVisible(isTargetChildOfDomNode);
//		},
//		_setAnchorVisible: function( isVisible) {
//
//			if(this.isApprovalVisible == isVisible) return;
//
//			if(!this.addApproverAnchor) return;
//			this.addApproval_fadeOut.node = this.addApproverAnchor;
//			this.addApproval_fadeIn.node = this.addApproverAnchor;
//
//			if(!isVisible) {
//				this.isApprovalVisible = false;
//				dojo.animateProperty(this.addApproval_fadeOut).play();
////				this.addApproverAnchor.style.color = "#EEE";
//			} else {
//				this.isApprovalVisible = true;	
////				this.addApproverAnchor.style.color = "";
//				dojo.animateProperty(this.addApproval_fadeIn).play();
//			}
//			
//		},
		_setWorkingCopyValue: function() {
			this.workingCopy.setValue({
				forceNotify: true,
				path: ["approvals", this.approval._approvalId],
				caller: this.parentWidget, // changed on behalf of
				value: 	this.approval // Current values
			});
		},
		_getColumnHeadingWidgets: function() {
				var result = [];
				for(var attr in columnLayout ) {
					result.push(this._getHeadingLabelWidget(columnLayout[attr]) )
				}
				result.push(this._getEmptyLabelWidget());
				return result;
		},
		_getHeadingLabelWidget: function(layout) {
			var widget = this._getFakeLabelWidget(layout.headingLabel, layout.headingStyle);
			widget.columnParms= layout;
					
			return widget;
		},
		_getFakeLabelWidget: function(label, style) {
			var span = document.createElement("SPAN");
			span.appendChild(document.createTextNode(label));
			
			for(var attr in style ) { 
					span.style[attr] = style[attr];
				}
			return { domNode: span }
		},
		_getEmptyLabelWidget: function() {
			var span = document.createElement("span");
			span.innerHTML = "X";
			span.style.color = "#f2f2f2";
			return { domNode: span, columnParms: {headingLabel: "", headingStyle: {}, style:{width: "23px"}} };
		},
		_getAddApproverActionWidgets: function() {

			var spacerWidget = { domNode: document.createElement("SPAN") };
			
			var anchor = document.createElement("A");
			anchor.appendChild(document.createTextNode(this.messages.buttonAddApprovers));
			anchor.parentWidget = this;
			anchor.href = window.location.href;
			anchor.className = "AddApproversAnchor";
			this.connect(anchor, "onclick", dojo.hitch(this, this.openAddApproversDialog));
						
			this.addApproverAnchor = anchor;
//			this._setAnchorVisible(false);
			
			var anchorWidget = { domNode: anchor };
						
			return [ spacerWidget, anchorWidget ]
		},
		
		_getApprovalWidgets: function() {
// Type \/  Name     Due date *                     |  cumulativeStatus
						
			var headingWidgets = [];
			
			var setTypeActions = [];
			var defaultAction = null;
			for(var i=0;i<this.approvalTypes.length;i++){
				var type = this.approvalTypes[i];
				var action = {
					isEnabled: true,
					receiverObject: this,
					receiverSelector: "setApprovalType",
					label: type.label,
					iconSrc: type.iconUrl,
					value: type
					
				}
				setTypeActions.push(action);
				if(action.value.id == this.approval.approvalType) {
					defaultAction = action;
				}
			}

			var creationArgs = {
						parentWidget: this,
						parentController: this.parentController,
						actions: setTypeActions,
						defaultAction: defaultAction,
						isBold: false,
						align: "right",
						label: "",
						columnParms: columnLayout.approvalType
					};
			var setTypeWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget(creationArgs);

			creationArgs = {
						parentWidget: this,
						parentController: this.parentController,
						presentation: {
							attribute: {
								attributeTypeId: "ShortString"
							}
						},
						selection:  [ {
							id: this.approval.name
							} 
						 ] ,
						 converter: this.nameConverter,
						 converterParms: {
						 		toolTip: this.messages.msgApprovalNameValidation,
								approval: this.approval
						 },
						columnParms: columnLayout.name
					};
					
			this.nameWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget(creationArgs);
			this.nameWidget.domNode.style.paddingLeft = "4px";
			this.nameWidget._textInput.__jazzApproval = this;
			this.connect(this.nameWidget._textInput, "onkeyup", this.setName); 
			this.connect(this.nameWidget._textInput, "onblur", this.checkName);
			
			var date = this.approval.dueDate ? this.approval.dueDate : "";
			creationArgs = {
						parentWidget: this,
						parentController: this.parentController,
						values: [ date ],
						_modifiers: {
							onValueChanged: "setDueDate"
						},
						columnParms: columnLayout.dueDate
					};
			var dateWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.TimestampWidget(creationArgs);
			dateWidget.domNode.style.marginTop="-10px";
			dateWidget.domNode.style.marginBottom="-1px";
			
			var stateKey = this.approval.cumulativeState.toLowerCase().replace(/\./g,"_");
			var state = stateMap[stateKey];
			state = state ? state : this.approval.cumulativeState;			
			creationArgs = {
				parentWidget: this,
				parentController: this.parentController,
				values: [ state ],
				align:"right",
				columnParms: columnLayout.state
			};
			var statusWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.LabelWidget(creationArgs);
			statusWidget.domNode.style.marginRight = "18px";
			
			//focus listeners
			this.connect(setTypeWidget._actionContainerNode, "onfocus", this._handleRowFocus);
			this.connect(setTypeWidget._actionContainerNode, "onblur", this._handleRowBlur);
			this.connect(this.nameWidget._textInput, "onfocus", this._handleRowFocus);
			this.connect(this.nameWidget._textInput, "onblur", this._handleRowBlur);
			this.connect(dateWidget.dateWidget.focusNode, "onfocus", this._handleRowFocus);
			this.connect(dateWidget.dateWidget.focusNode, "onblur", this._handleRowBlur);
			
 			return [ setTypeWidget, this.nameWidget, statusWidget, dateWidget]

		},
		_handleRowFocus: function(e) {
			var target = e.target;
			while(target.tagName != "TR" && target.parentNode) {
				target = target.parentNode;
			}
			if (this.sectionListWidget && target.tagName == "TR") {
				this.sectionListWidget.setRowBackground(target, true);
			}
		},
		_handleRowBlur: function(e) {
			var target = e.target;
			while(target.tagName != "TR" && target.parentNode) {
				target = target.parentNode;
			}
			if (this.sectionListWidget && target.tagName == "TR") {
				this.sectionListWidget.setRowBackground(target, false);
			}
		},
		//* This runs in the context of TextWidget
		//
		nameConverter: function (value, parms, isGettingValidValue) {
			var self=this.parentWidget;
			if(isGettingValidValue) {
				if(!value || value.length == 0) {
					return self.parentWidget._getDefaultName(parms.approval);
				}
				if(self.parentWidget._hasApprovalWithName(value, parms.approval)) {
					return self.parentWidget._getDefaultName(parms.approval);
				}
				
				var newValue = dojo.trim(value);
				if(value.length > 250) {
					return newValue.substring(0,250);
				}
				return newValue;
			}
			var valid = value &&  value.length > 0 && value.length <=250;
			if(valid) {
				valid = !self.parentWidget._hasApprovalWithName(value, parms.approval);
			}
			
			return {isCustom: true, isValid: valid };
		},

		_getApproverWidgets: function(approval, isFirst, isLast) {
			// TODO: add list arrow graphcs
			//           .approver.name
			//           .state

			var creationArgs = {
						parentWidget: this,
						parentController: this.parentController,
						values: [ " " ],
						columnParms: {
							style: { }
						}
					};
			var blankWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.LabelWidget(creationArgs);
			
			
			
			
			creationArgs = {
						parentWidget: this,
						parentController: this.parentController,
						values: [ approval.approver.name ],
						columnParms: {
							style: {  }
						}
					};
			var nameWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.LabelWidget(creationArgs);
			
			creationArgs = {
						parentWidget: this,
						parentController: this.parentController,
						values: [ " " ],
						columnParms: {
							style: { }
						}
					};
			var blankWidget2 = new com.ibm.team.workitem.web.ui.internal.view.layout.LabelWidget(creationArgs);
			
			//* If its the current user, can change the status, else not
			var statusWidget;
			if (getAuthenticatedContributor().itemId == approval.approver.itemId) {
			
				var setStateActions = [];
				var defaultAction = null;
				for (var i = 0; i < this.approvalStates.length; i++) {
					var state = this.approvalStates[i].id;
					var stateKey = state.toLowerCase().replace(/\./g, "_");
					var stateLabel = stateMap[stateKey];
					stateLabel = stateLabel ? stateLabel : approval.state;
					
					var action = {
						isEnabled: true,
						receiverObject: this,
						receiverSelector: "setApproverState",
						label: stateLabel,
						value: state,
						approver: approval
					}
					setStateActions.push(action);
					if (action.value == approval.state) {
						defaultAction = action;
					}
				}
				
				
				var creationArgs = {
					parentWidget: this,
					parentController: this.parentController,
					actions: setStateActions,
					defaultAction: defaultAction,
					isBold: false,
					align: "right",
					label: "",
					columnParms: columnLayout.state
				};
				statusWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget(creationArgs);
				
				//focus listeners
				this.connect(statusWidget._actionContainerNode, "onfocus", this._handleRowFocus);
				this.connect(statusWidget._actionContainerNode, "onblur", this._handleRowBlur);
				
			} else {
				var state = approval.state;
				var stateKey = state.toLowerCase().replace(/\./g, "_");
				var stateLabel = stateMap[stateKey];
				stateLabel = stateLabel ? stateLabel : approval.state;
				creationArgs = {
					parentWidget: this,
					parentController: this.parentController,
					values: [stateLabel],
					align: "left",
					columnParms: columnLayout.state
				};
				statusWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.LabelWidget(creationArgs);
				statusWidget.domNode.style.marginRight = "18px";
			}					

			return { columnWidgets: [ blankWidget, nameWidget, statusWidget, blankWidget2 ],
					 value: approval,
					_isNew: (approval._isNew || false)				 
					}		
			
		},

		/*
		 * Actions
		 */
		// var command= { id: approvalId, cmd: "removeApprover", param: { user: contributorItemId } };
		deleteApprover: function(e) {
			var self=e.target.parentWidget;
			var target = e.target;
			if(!self) {
				self = e.target.firstChild.parentWidget;
				target = e.target.firstChild;
			}
			if(!self) return;
			target.value.approver._isDeleted = true;
			target.value._isDeleted = true;
			if(target.value._isNew) {
				for(var i=0;i<self.approval.approvals.length;i++) {
					if(self.approval.approvals[i].approver.itemId == target.value.approver.itemId) {
						self.approval.approvals.splice(i, 1);
						break;
					}
				}
			} else {
				target.value.approver._deleteCommandIndex = self.addApprovalCommand("user", target.value.approver.itemId, null, null, "removeApprover");
			}
			self._handleDirtyingEvent();						
			self.refresh();
		},
		restoreApprover: function(e) {
			var self=e.target.parentWidget;
			var target = e.target;
			if(!self) {
				self = e.target.firstChild.parentWidget;
				target = e.target.firstChild;
			}
			if(!self) return;
			target.value.approver._isDeleted = false;
			target.value._isDeleted = false;
			self.removeApprovalCommand(target.value.approver._deleteCommandIndex);
			self._handleDirtyingEvent();
			self.refresh();
		},		
		deleteApproval: function(e) {
			var self=e.target.parentWidget;		
			if(!self) {
				self = e.target.firstChild.parentWidget;
			}
			if(!self) return;
			self.approval._isDeleted = true;
			if(self.approval._isNew) {
				self.parentWidget.removeApproval(self.approval);
			} else {
				self.approval._deleteCommandIndex = self.addApprovalCommand(null,null, null, null, "removeApproval");				
			}
			self._handleDirtyingEvent();
			self.parentWidget._refreshApprovals();
		},
		restoreApproval: function(e) {
			var self=e.target.parentWidget;
			if(!self) {
				self = e.target.firstChild.parentWidget;
			}
			if(!self) return;
			self.approval._isDeleted = false;
			if(self.approval._isNew) {
				self.parentWidget.removeApproval(self.approval);
			} else {
				self.removeApprovalCommand(self.approval._deleteCommandIndex);				
			}
			self._handleDirtyingEvent();
			self.parentWidget._refreshApprovals();
		},

		openAddApproversDialog: function(e) {
			if(!EditorUtil.checkLogin()) return;
			var self= this;
			dojo.stopEvent(e);
			var exclusionList = [];	
			if(self.approval.approvals) {
				for(var i=0; i<self.approval.approvals.length;i++) {
					exclusionList.push({
						itemId: self.approval.approvals[i].approver.itemId
					} )
				}				
			}

			var params= {
				title: this.messages.titleSelectApprovers,  // change to dialogTitle:
				disableUsers: exclusionList
			}
			this.ownerListWidget= new SelectContributorDialog(params);
			this.connect(this.ownerListWidget, "onClose", dojo.hitch(this, function() { this.parentController.focus();}));
			this.connect(this.ownerListWidget, "onOK", dojo.hitch(this, function(selectedUserList) { this._addApproversFromSelectionDialog(selectedUserList);}));
			this.ownerListWidget.searchText.focus();
			return false;	
		},
		_addApproversFromSelectionDialog:function (selectedUserList){
			if(selectedUserList) {
				for(var i=0;i<selectedUserList.length;i++) {
					this._addApprover(selectedUserList[i]);
				}
			}
			this._handleDirtyingEvent();
			this.refresh();
		},
		_addApprover:function (user){
			var approval = {
				state: "com.ibm.team.workitem.approvalState.pending",
				_isNew: true
			}
			
			var approver = {
				itemId: user.itemId,
				name: user.name
			}
			approval.approver = approver;
			if (!this.approval.approvals) {
				this.approval.approvals = [];
			}
			approval._approvalId = this.approval.approvals.length;
			this.approval.approvals.push(approval);
//			var command= { id: approvalId, cmd: "updateApprover", param: { user: contributorItemId } };
			this.setApproverCommand(approver, "user", user.itemId, null, null);
			this._handleDirtyingEvent();			
		},
		setDueDate: function(widget, date) {
			// dates are local time
			// set to noon local time
			if(date) {
				date.setHours(12);
				date.setMinutes(0);
				date.setSeconds(0);
				date.setMilliseconds(0);
			}
			
			var time  = !date ? null : date.getTime(); 
			this.addApprovalCommand("dueDate", time || 0); 

			// This changes it to a date from the ISO String
			this.approval.dueDate = time;
			this.approval.dueDateAsDate = date;
			this._handleDirtyingEvent();
		},
		checkName: function(e) {
			var self = e.target.__jazzApproval;
			if(!self) { // OK, came from somewhere else
				return;
			}
			var name = e.target.value;
			if (name == "") {
				self.approval.name = ""; // changed in updateName
				self._updateName();
			}
		},
		setName: function(e) {
			var self = e.target.__jazzApproval;
			if(!self) { // OK, came from somewhere else
				return;
			}
			var name = e.target.value;
			if (name == "") {
				self.approval.name = ""; // changed in updateName
				return;
			}
			self._addSetNameCommand(name);
		},
		_addSetNameCommand: function(approvalName){
//			var approvalName = dojo.trim(p1);
			if(approvalName == this.approval.name) return;
			this.addApprovalCommand("name", approvalName);
			this.approval.name = approvalName;
			if(this.nameWidget._textInput.value != approvalName) {
				this.nameWidget._textInput.value = approvalName;
			}
			this._handleDirtyingEvent();
		},
		setApprovalType: function(action) {
			if(action.value.id == this.approval.approvalType) return;
			this.addApprovalCommand("type", action.value.id);
			this.approval.approvalType = action.value.id;
			this._updateName();
			this._handleDirtyingEvent();
		}, 
		/*
		 * When the approval type changes, 
		 * Update the name so if it is a default name
		 *   target_Review(2)
		 * Change it to the new type
		 *   target_Verify(2)
		 */
		_updateName: function() {
			var newName = this.parentWidget._getUpdatedDefaultName(this.approval);
			if(newName && newName != this.approval.name) {
				this._addSetNameCommand(newName);	
			}
		},		
		// var command= { id: approvalId, cmd: "updateApprover", param: { user: contributorItemId, state: state } };
		setApproverState: function(action) {
			this.setApproverCommand(action.approver.approver, "user", getAuthenticatedContributor().itemId, "state", action.value);
			action.approver.state = action.value;
			action.approver._isDirty = true;
			this._handleDirtyingEvent();			
		}, 
		
		_handleDirtyingEvent:function() {
			this._setWorkingCopyValue();
			if(this.parentWidget && this.parentWidget.markDirty) {
					this.parentWidget.markDirty();
				} 

			if(this.parentWidget && this.parentWidget.parentController && this.parentWidget.parentController.markDirty) {
						this.parentWidget.parentController.markDirty();						
					}
		},
		// id: approvalId, cmd: "updateApprovals", param: { name: "New name", dueDate: date.getTime() } };
		addApprovalCommand: function(paramKey, value, paramKey2, value2, cmdName) {
			if(!cmdName) {
				cmdName = "updateApproval"
			} 
			if(!this.approval._commands) {
				this.approval._commands = [];
			}
			var command = {
					id: this.approval.id,
					cmd: cmdName,
					param: {}
				}
			if(paramKey!=null && value!=null) {
				command.param[paramKey] = value;
			}
			if(paramKey2) {
				command.param[paramKey2] = value2;
			}
			var savedCommand, found=false;
			for(var i=0;i<this.approval._commands.length;i++) {
				savedCommand = this.approval._commands[i];
				// If the command applies to this approver (user) and this approval, update the command
				if(savedCommand.param && savedCommand.param.user && command.param && command.param.user && savedCommand.param.user == command.param.user &&
					savedCommand.cmd == command.cmd && savedCommand.id == command.id ) {
					found=true;
					savedCommand.param[paramKey] = value;
					if(paramKey2) {
						savedCommand.param[paramKey2] = value2;
					}
					break;
				}
			}
			
			if(!found)	this.approval._commands.push(command);
			
			return this.approval._commands.length -1;
			
		},
		removeApprovalCommand: function(index) {
			this.approval._commands[index] = false;
		},
		setApproverCommand: function(approver, paramKey, value, paramKey2, value2, cmdName) {
			if(!cmdName) {
				cmdName = "updateApprover"
			} 
			if(!approver._command){
				approver._command = {
					id: this.approval.id,
					cmd: cmdName,
					param: {}
				} 
			}
			approver._command.param[paramKey] = value;
			if(paramKey2) {
				approver._command.param[paramKey2] = value2;
			}
		}
	}
);

})();
