/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
	
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var ImagePool 				= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var SEVERITY_INFO = 0;
var SEVERITY_WARNING = 4;
var SEVERITY_ERROR = 8;

var checkNotNull=	net.jazz.ajax.toolbox.lang.checkNotNull;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout",
	[dijit._Widget, dijit._Templated], 
	{
	templateString: '<div class="com-ibm-team-workitem-view-layout-NameValueLayout">'
		+ '<table CELLPADDING="0" CELLSPACING=0 dojoAttachPoint="_nameValueTable" class="com-ibm-team-workitem-namevalue-table">'
		+ '</table></div>',
	postCreate: function() {
		// hasRequiredMarker
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		this.labelSpans = [];
		this._childWidgets = [];
		this.inherited(arguments);
	},
	destroy: function() {
		for(var i=0;i< this._childWidgets.length;i++) {
			if(this._childWidgets[i] && this._childWidgets[i].destroy) {
				this._childWidgets[i].destroy();			
			}
			this._childWidgets[i] = null;
		}
		this._childWidgets = [];
		this.inherited(arguments);
	},
	setAttributeId: function(id) {
		if(this._lastRequiredMark) {
			this._lastRequiredMark.setAttributeId(id);
		}
	},
	addRow: function(parms /* labelText: labelText, valueNode, isWide, isOneLine, widget */ ) {
		
		var row = this._nameValueTable.insertRow(-1);
		row.className = "com-ibm-team-workitem-namevalue-tr";
		if(parms.labelText && parms.labelText.indexOf(":") == -1) parms.labelText = dojo.string.substitute(this.messages.labelValue, [parms.labelText]);

		if(!parms.isWide) {
			var labelCol = row.insertCell(-1);
			labelCol.className = "com-ibm-team-workitem-namevalue-label-td";
			var labelSpan = document.createElement("div");
			labelSpan.className = "com-ibm-team-workitem-namevalue-label-node";
			labelSpan.appendChild(document.createTextNode(parms.labelText || "" ));
			this.labelSpans.push(labelSpan);
			if(this.hasRequiredMarker && (this.attributeId || (parms.widget && parms.widget.attributeId))) {
//				var reqCol = row.insertCell(-1);
//				reqCol.className = "com-ibm-team-workitem-namevalue-required-td";
				var attributeId = (parms.widget && parms.widget.attributeId) ? parms.widget.attributeId : this.attributeId;
				labelSpan.appendChild(this._getRequiredMarkerElement(attributeId));
			} else {
				this._lastRequiredMark = null;
			}
			labelCol.appendChild(labelSpan);
			
			var valueCol = row.insertCell(-1);
			valueCol.className = "com-ibm-team-workitem-namevalue-value-td";
			valueCol.appendChild(parms.valueNode);
		} else {
			var sectionCol = row.insertCell(-1);
			sectionCol.colSpan = 3;
			var nameDiv = document.createElement("DIV");
			nameDiv.appendChild(document.createTextNode(parms.labelText))
			if(parms.isOneLine) {
				dojo.addClass(nameDiv, "com-ibm-team-workitem-namevalue-label-oneline");
				dojo.addClass(parms.valueNode, "com-ibm-team-workitem-namevalue-value-oneline");
			}
			sectionCol.appendChild(nameDiv);
			sectionCol.appendChild(parms.valueNode);
		}
	},
	addIconMessageRow: function(severity, message, isSmallImage) {
		var row = this._nameValueTable.insertRow(-1);
		row.className = "com-ibm-team-workitem-namevalue-tr";
		
		var labelCol = row.insertCell(-1);
		labelCol.className = "com-ibm-team-workitem-namevalue-img-td";
		var labelSpan = document.createElement("div");
		labelSpan.className = "com-ibm-team-workitem-namevalue-label-node";
		
		var icon = document.createElement("IMG");
		switch(severity) {
			case SEVERITY_WARNING:
			icon.src = ImagePool.WARNING;
			break;			
			case SEVERITY_ERROR:
			icon.src = ImagePool.INFO;
			break;
			default:
			icon.src = ImagePool.INFO;
		}
		if(isSmallImage) {
			icon.className = "com-ibm-team-workitem-namevalue-img-small";
		} else {
			icon.className = "com-ibm-team-workitem-namevalue-img-large";		
		}
		labelSpan.appendChild(icon);
		labelCol.appendChild(labelSpan);
		
		var valueCol = row.insertCell(-1);
		valueCol.className = "com-ibm-team-workitem-namevalue-value-td";
		valueCol.appendChild(document.createTextNode(message || ""));
	},
	addMessageRow: function(severity, message) {
		var row = this._nameValueTable.insertRow(-1);
		row.className = "com-ibm-team-workitem-namevalue-tr";
		
		var valueCol = row.insertCell(-1);
		valueCol.COLSPAN = 2;
		valueCol.className = "com-ibm-team-workitem-namevalue-value-td";
		valueCol.appendChild(document.createTextNode(message || ""));
	},
	addSeparatorRow: function() {
		var row = this._nameValueTable.insertRow(-1);
		row.className = "com-ibm-team-workitem-namevalue-tr-sep";
		var labelCol = row.insertCell(-1);
		labelCol.appendChild(this._getSeparatorElement());
		if(this.hasRequiredMarker) {
			var reqCol = row.insertCell(-1);
			reqCol.appendChild(this._getSeparatorElement());
		}
		var valueCol = row.insertCell(-1);
		valueCol.appendChild(this._getSeparatorElement());
	},
	setLabel: function(parms) {
		if(parms.index == null) parms.index = 0;
		var labelSpan = this.labelSpans[parms.index];
		dojox.data.dom.removeChildren(labelSpan);
		labelSpan.appendChild(document.createTextNode(parms.labelText || "" ));
	},
	setVisible: function(isVisible) {
		this._nameValueTable.style.visibility = isVisible ? "visible" : "hidden";
	},
	isVisible: function() {
		return this._nameValueTable.style.visibility != "hidden";
	},
	_getSeparatorElement: function() {
		var span = document.createElement("span");
		span.className = "com-ibm-team-workitem-namevalue-value-td-sep";
		return span;
	},
	_getRequiredMarkerElement: function(attributeId) {
		var reqMark = new com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget(
			{workingCopy: this.workingCopy,
			 attributeId: attributeId}
		);
		this._childWidgets.push(reqMark);
		return reqMark.domNode;	
	}
});
	
})();