/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.actions.AddWorkItemAction"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.PlanningClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.WorkItemSaveManager"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.ImageClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.projectarea"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {
	
var SelectWorkItemDialog= com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog
var ImageClient= com.ibm.team.apt.web.client.internal.ImageClient;

var Messages= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var IconUrl= dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/images/search-workItemIcon.gif"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.actions.AddWorkItemAction", //$NON-NLS-1$
			 [com.ibm.team.apt.web.ui.internal.view.plan.actions.Action], {
	
	_viewModel: null,
	_movePolicy: null,
	_viewerItem: null,
	_location: null,
	
	constructor: function(viewModel, itemMovePolicy, viewerItem, location) {
		this._viewModel= viewModel;
		this._movePolicy= itemMovePolicy;
		this._viewerItem= viewerItem;
		this._location= location;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getId: function() {
		return "add.existing.workItem"; //$NON-NLS-1$
	},
	
	getName: function() {
		return Messages.ACTION_ADD_WORK_ITEM;
	},
	
	getIconUrl: function() {
		return IconUrl;
	},
	
	canHandle: function() {
		return this.__getAllowedWorkItemTypes().length > 0;
	},
	
	handle: function() {
		var plan= this._viewModel.getInput();
		var $this= this;
		
		var serviceCallback= {
			_success: function(workItems) {
				com.ibm.team.apt.web.client.internal.WorkItemSaveManager.compoundSave(function() {
					
					$this._viewModel.updateModel(function(updateAccessor){
						var planItems= [];
						plan.compoundChange(function() {
							var prop;
							for(prop in workItems){
								planItems.push(plan.adoptPlanItem(workItems[prop]));
							}
						}, $this);
						
						if(this._viewerItem){
							var targetEntry= this._viewerItem.getViewEntry();
							var navigator= updateAccessor.getEntryNavigator(false);
							for(var i= 0; i < planItems.length; i++) {
								var entry= navigator.findPrimaryEntry(planItems[i]);
								if(entry != null && this._movePolicy.canMove(entry, targetEntry, this._location)){
									this._movePolicy.move(entry, targetEntry, this._location);
								}
							}
						}
						
					}, $this);
					
				}, $this._viewModel, {showErrorMessage: function() {}}, $this);
			},
			_error: function(e) {
				
			}					
		};
		var responseHandler= new com.ibm.team.repository.web.transport.ServiceResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$
		
		var fnOk= function(workItems){
			var planId= plan.getPlanId();
			var itemIds= dojo.map(workItems, function(workItem){
				return workItem.workItemItemId;
			});
			com.ibm.team.apt.web.client.internal.PlanningClient.loadPlanItems(responseHandler, planId, itemIds, plan.__referencedItems);
		};
		
		var workItemIds= [];
		plan.accept(function(planElement) {
			var attribute= plan.findAttribute(com.ibm.team.apt.client.PlanItem.ID);
			if(planElement instanceof com.ibm.team.apt.client.PlanItem){
				var value= attribute.getValue(planElement);
				workItemIds.push(String(value));
			}
			return true;
		});
		
		var params= {
			allowsMultiple: true,
			projectAreaItemId: com.ibm.team.process.web.client.projectarea.getCurrentProjectArea().itemId,
			exclusionList: workItemIds,
			workItemTypeIdList: dojo.map(this.__getAllowedWorkItemTypes(), function(type) {
				return type.itemId;
			}),
			onOk: fnOk
		};
		new SelectWorkItemDialog(params);
	},
		
	__getAllowedWorkItemTypes: function() {
		var plan= this._viewModel.getInput();
		var types= plan.getCreatableWorkItemTypes();

		if (this._viewerItem) {
			var targetEntry= this._viewerItem.getViewEntry();
			types= dojo.filter(types, function(type) {
				return this._movePolicy.canCreate(targetEntry, type, this._location);
			}, this);
		}
			
		return types;
	},
	
	__sentinel: null // terminates this class definition
});

})();
