/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.actions.NewWorkItemMenuAction"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.MenuAction"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.Separator"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.NewWorkItemAction"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.calm.WorkItemsFromRequirementsAction"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.AddWorkItemAction"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$=

(function() {
	
var Assert							= com.ibm.team.apt.web.ui.internal.common.Assert;
var NewWorkItemAction				= com.ibm.team.apt.web.ui.internal.view.plan.actions.NewWorkItemAction;
var WorkItemsFromRequirementsAction = com.ibm.team.apt.web.ui.internal.calm.WorkItemsFromRequirementsAction;
var AddWorkItemAction				= com.ibm.team.apt.web.ui.internal.view.plan.actions.AddWorkItemAction;
var Separator						= com.ibm.team.apt.web.ui.internal.view.plan.actions.Separator;

var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var AddIconUrl= dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/images/create_workitem.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var AddAboveIconUrl= dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/images/add_item_above.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var AddBelowIconUrl= dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/images/add_item_below.gif"); //$NON-NLS-1$ //$NON-NLS-2$
	
dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.actions.NewWorkItemMenuAction", //$NON-NLS-1$
		[com.ibm.team.apt.web.ui.internal.view.plan.actions.MenuAction], {
	
	_viewModel: null,
	_itemMovePolicy: null,
	_tab: null,
	
	constructor: function(viewModel, itemMovePolicy, errorReporter, viewerItem, tab) {
		this._viewModel= viewModel;
		this._itemMovePolicy= itemMovePolicy;
		this._viewerItem= viewerItem;
		this._errorReporter= errorReporter;
		this._location= com.ibm.team.apt.ui.Move.Child;
		this._tab= tab;
	},

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getName: function() {
		return MSGS.ACTION_NEW_WORK_ITEM;
	},
	
	getId: function() {
		return "new.planitem." + this._location; //$NON-NLS-1$
	},
	
	getIconUrl: function() {
		return AddIconUrl;
	},
	
	createActions: function() {
		var result= [];
		
		result.push(new AddWorkItemAction(this._viewModel, this._itemMovePolicy, this._viewerItem, this._location));
		result.push(new Separator());
		var plan= this._viewModel.getInput();
		var types= plan.getCreatableWorkItemTypes();
		
		for ( var i = 0; i < types.length; i++) {
			var type= types[i];
			result.push(this._createNewWorkItemAction(type));	
		}
		
		if (this._tab) {
			var serviceDescriptors= plan.getRequirementServiceDescriptors();
			if (serviceDescriptors.length > 0) {
				result.push(new com.ibm.team.apt.web.ui.internal.view.plan.actions.Separator());
	
				for ( var i = 0; i < serviceDescriptors.length; i++) {
					result.push(this._createNewWorkItemFromRequirementsAction(serviceDescriptors[i]));	
				}
			}
		}
		return result;
	},
	
	canHandle: function() {
		if (!this._viewerItem)
			return true;
		
		var targetEntry= this._viewerItem.getViewEntry();
		var selection= this._viewModel.getSelectedEntries();
		
		if (selection.length == 0 || (selection.length == 1 && selection[0] == targetEntry)) {
			return dojo.some(this.createActions(),  function(action) {
				return !(action instanceof Separator) && action.canHandle();
			});
		}
		return false;
	},
	
	// ---- SPI ------------------------------------
	
	_createNewWorkItemAction: function(type) {
		return new NewWorkItemAction(this._viewModel, type, this._itemMovePolicy, this._viewerItem, this._location);
	},
	
	_createNewWorkItemFromRequirementsAction: function(serviceDescriptor) {
		return new WorkItemsFromRequirementsAction(this._viewModel, serviceDescriptor, this._tab);
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.actions.NewWorkItemMenuAction.Above", //$NON-NLS-1$
		[com.ibm.team.apt.web.ui.internal.view.plan.actions.NewWorkItemMenuAction], {
	

	constructor: function(viewModel, itemMovePolicy) {
		this._location= com.ibm.team.apt.ui.Move.Before;
	},

	getName: function() {
		return MSGS.action_newWorkItemAbove;
	},
	
	getIconUrl: function() {
		return AddAboveIconUrl;
	}
});

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.actions.NewWorkItemMenuAction.Below", //$NON-NLS-1$
		[com.ibm.team.apt.web.ui.internal.view.plan.actions.NewWorkItemMenuAction], {
	

	constructor: function(viewModel, itemMovePolicy) {
		this._location= com.ibm.team.apt.ui.Move.After;
	},

	getName: function() {
		return MSGS.action_newWorkItemBelow;
	},
	
	getIconUrl: function() {
		return AddBelowIconUrl;
	}
});

})();
