/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.dnd.PlanItemDnDSource"); //$NON-NLS-1$

dojo.require("dojo.dnd.Source"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.ImageClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.WorkItemSaveManager"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.dialogs.ErrorDialog"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
	
var ImageClient			= com.ibm.team.apt.web.client.internal.ImageClient;
var WorkItemSaveManager	= com.ibm.team.apt.web.client.internal.WorkItemSaveManager;
var ErrorDialog			= com.ibm.team.apt.web.ui.internal.view.plan.dialogs.ErrorDialog;

var doc= document;
var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var TypeDecorationUrl= dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/images/complete_ovr.gif"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.dnd.PlanItemDnDSource", //$NON-NLS-1$
		 dojo.dnd.Source, {
	
	_viewerItem: null,
	_itemMovePolicy: null,
	_errorReporter: null,
	_lastLocation: null,
	__isDestroyed: false,
	
	constructor: function(node, params) {
		this._lastLocation= com.ibm.team.apt.ui.Move.Child;
		
		this.__isDestroyed= false;
	},
	
	getViewerItem: function() {
		return this._viewerItem;
	},
	
	checkAcceptance: function(source, nodes) {
		if (nodes.length != 1 || !nodes[0])
			return false;
		
		var item= source.getItem(nodes[0].id);
		var type= item.type[0];
		if (!(type in this.accept))
			return false;
		
		var target= this.__calculateTarget();
		if (!target)
			return false;
		
		var sourceEntry= item.data.getViewEntry();
		var targetEntry= target.getViewEntry();
		
		if (!sourceEntry.getModel().getInput().equals(targetEntry.getModel().getInput()))
			return false; // can not move from one plan to another, yet.

		return this._itemMovePolicy.canMove(sourceEntry, targetEntry, this._lastLocation);
	},
	
	onDndCancel: function(){
		var localDoc= doc;

		this.inherited(arguments);
		
		this._lastLocation= null;
		this.targetBox= null;
		
		dojo.removeClass(localDoc.body, "com-ibm-team-apt-jazz-ui-dropCursor"); //$NON-NLS-1$
		dojo.removeClass(localDoc.body, "com-ibm-team-apt-jazz-ui-noDropCursor"); //$NON-NLS-1$
	},
	
	onDndStart: function(source, nodes, copy){
		this.inherited(arguments);
		
		dojo.addClass(doc.body, "com-ibm-team-apt-jazz-ui-dropCursor"); //$NON-NLS-1$
	},
	
	onMouseMove: function(e){
		if (this.isDragging) {
			var potentialTarget= this._calculatePotentialTarget();
			if (!potentialTarget) {
				this.__disableDropTarget();
				return;
			}
			
			if(!this.targetBox || this.targetAnchor != potentialTarget){
				this.targetBox = {
					xy: dojo.coords(potentialTarget, true),
					w: potentialTarget.offsetWidth,
					h: potentialTarget.offsetHeight
				};
			}
			
			if (e.pageX < this.targetBox.xy.x || e.pageX > this.targetBox.xy.x + this.targetBox.w) {
				this.__disableDropTarget();
				return;
			}
			
			if (e.pageY < this.targetBox.xy.y || e.pageY > this.targetBox.xy.y + this.targetBox.h) {
				this.__disableDropTarget();
				return;
			}
			
			dojo.dnd.Source.superclass.onMouseMove.call(this, e);
			
			var location= this.__calculateLocation(e, this.targetBox);
			if (potentialTarget != this.targetAnchor || location != this._lastLocation) {
				this._lastLocation= location;
				
				var m = dojo.dnd.manager();
				var accepted = this.accept && this.checkAcceptance(m.source, m.nodes);
				
				this.current= potentialTarget;
				
				this._markTargetAnchor(location === com.ibm.team.apt.ui.Move.Before, location === com.ibm.team.apt.ui.Move.After);
				m.target= this;
				m.canDrop(accepted);
				
				var localDoc= doc;

				dojo.toggleClass(localDoc.body, "com-ibm-team-apt-jazz-ui-dropCursor", accepted); //$NON-NLS-1$
				dojo.toggleClass(localDoc.body, "com-ibm-team-apt-jazz-ui-noDropCursor", !accepted); //$NON-NLS-1$
			}
		} else {
			this.inherited(arguments);
		}
	},
	
	__disableDropTarget: function() {
		var m = dojo.dnd.manager();
		if (m.target !== this)
			return;
				
		this._markTargetAnchor(false, false);
		m.target= null;
		m.canDrop(false);
	},
	
	onDndDrop: function(source, nodes, copy, target){
		if(this == target){
			var viewerItem= source.getItem(nodes[0].id).data;
			viewerItem.getViewModel().updateModel(function(updateAccessor) {
				this.onDrop(source, nodes, copy);
				this.onDndCancel();
			}, this);
		} else {
			this.onDndCancel();
		}
	},
	
	onDrop: function(source, nodes, copy){
		try {
			dojo.forEach(nodes, dojo.hitch(this, function(node) {
				var planItemData= source.getItem(node.id).data;
				this._moved(planItemData.getViewEntry());
			}));
		} catch (e) {
			console.log(e);
			
			new ErrorDialog({
				title: MSGS.DROP_ITEM_ERROR_TITLE, 
				message: bind(MSGS.DROP_ITEM_ERROR_MESSAGE, [e.message])
			});
		}
	},
	
	_drop: function() {
		var manager= dojo.dnd.manager();
		if (manager.target !== this){
		    return;
		}
		
		var nodes= manager.nodes;
		dojo.forEach(nodes, dojo.hitch(this, function(node) {
			var source= manager.source;
			var planItemData= source.getItem(node.id).data;
			this._moved(planItemData.getViewEntry());
		}));
	},

	_moved: function(viewEntry) {
		var target= this.__calculateTarget();
		WorkItemSaveManager.compoundSave(dojo.hitch(this, function() {
			this._itemMovePolicy.move(viewEntry, target.getViewEntry(), this._lastLocation);
		}), target.getViewModel(), this._errorReporter, this);
	},
	
	destroy: function(){
		this.__isDestroyed= true;
		
		this.inherited(arguments);
	},
	
	_changeState: function(type, newState){
		if (this.__isDestroyed)
			//This can happen if onDrop on another target results in destroying this target once the
			//view model updates and onDnDCancel is called on this target after it has been destroyed 
			//-> Because this is destroyed anyway we ignore the state change
			return;
		
		var state  = type.toLowerCase() + "State"; //$NON-NLS-1$
		this[state] = newState;
		return;
	},
	
	_addItemClass: function(node, type){
		return;
	},
	
	_removeItemClass: function(node, type){
		return;
	},
	
	_markTargetAnchor: function(before, after){
		if (this.targetAnchor) {
			dojo.removeClass(this.targetAnchor, "dndBefore"); //$NON-NLS-1$
			dojo.removeClass(this.targetAnchor, "dndAfter"); //$NON-NLS-1$
		}
		
		this.inherited(arguments);
		
		if (this.targetAnchor) {
			dojo.toggleClass(this.targetAnchor, "dndBefore", before); //$NON-NLS-1$
			dojo.toggleClass(this.targetAnchor, "dndAfter", after); //$NON-NLS-1$
		}
	},
	
	_unmarkTargetAnchor: function(){
		if (this.targetAnchor) {
			dojo.removeClass(this.targetAnchor, "dndBefore"); //$NON-NLS-1$
			dojo.removeClass(this.targetAnchor, "dndAfter"); //$NON-NLS-1$
		}
		
		this.inherited(arguments);
	},
	
	_calculatePotentialTarget: function() {
		return this.current;
	},
	
	__calculateTarget: function() {
		if (this.current) {
			var result= this.getItem(this.current.id);
			if (!result)
				return null;
			
			return result.data;
		} else {
			return this.getViewerItem();
		}
	},
	
	__calculateLocation: function(e, targetBox) {
		if (e.pageY < targetBox.xy.y + 5) {
			return com.ibm.team.apt.ui.Move.Before;
		} else if (e.pageY > targetBox.xy.y + targetBox.h - 5) {
			return com.ibm.team.apt.ui.Move.After;
		} else {
			return com.ibm.team.apt.ui.Move.Child;
		}
	},

	
	__sentinel: null
});

com.ibm.team.apt.web.ui.internal.view.plan.dnd.PlanItemDnDSource.constructorArgs= function(viewerItem, itemMovePolicy, errorReporter, parent) {
	return {
		accept: ["workItem"], //$NON-NLS-1$
		parent: parent,
		withHandles: true,
		skipForm: true,
		_itemMovePolicy: itemMovePolicy,
		_viewerItem: viewerItem,
		_errorReporter: errorReporter,

		creator: function(item, hint) {
			if (hint != "avatar") { //$NON-NLS-1$
				return {node: item.domNode, data: item, type: ["workItem"]}; //$NON-NLS-1$
			}else{
				var node = this._createAvatar(item.getPlanItem());
				return {node: node, data: item, type: ["workItem"]}; //$NON-NLS-1$
			}
		},
		
		_createAvatar: function(planItem) {
			var localDoc= doc;
			var result= localDoc.createElement("div"); //$NON-NLS-1$
			dojo.addClass(result, "attribute"); //$NON-NLS-1$
			dojo.addClass(result, "summary"); //$NON-NLS-1$
			
			var iconContainer= localDoc.createElement("div"); //$NON-NLS-1$
			dojo.addClass(iconContainer, "icon"); //$NON-NLS-1$
			
			var iconElement= localDoc.createElement("img"); //$NON-NLS-1$
			dojo.addClass(iconElement, "typeIcon"); //$NON-NLS-1$
			iconElement.src= ImageClient.getImageUrl(planItem.getItemType().iconURL);
			iconContainer.appendChild(iconElement);
	
			var decorationElement= localDoc.createElement("img"); //$NON-NLS-1$
			dojo.addClass(decorationElement, "typeDecoration"); //$NON-NLS-1$
			decorationElement.src= TypeDecorationUrl;
			iconContainer.appendChild(decorationElement);
	
			result.appendChild(iconContainer);
		
			var idLink= localDoc.createElement("a"); //$NON-NLS-1$
			dojo.addClass(idLink, "label"); //$NON-NLS-1$
			var summary= planItem.getSummary();
			if (summary.length > 40)
				summary= summary.substring(0, 40) + "..."; //$NON-NLS-1$
			idLink.appendChild(localDoc.createTextNode(summary));
			result.appendChild(idLink);
			
			var workItemWrap= localDoc.createElement("div"); //$NON-NLS-1$
			dojo.addClass(workItemWrap, "workItem"); //$NON-NLS-1$
			workItemWrap.appendChild(result);
			
			var plannedItemsWrap= localDoc.createElement("div"); //$NON-NLS-1$
			plannedItemsWrap.appendChild(workItemWrap);
			dojo.addClass(plannedItemsWrap, "tabPlannedItems"); //$NON-NLS-1$
			
			var aptWrap= localDoc.createElement("div"); //$NON-NLS-1$
			aptWrap.appendChild(plannedItemsWrap);
			dojo.addClass(aptWrap, "com-ibm-team-apt"); //$NON-NLS-1$
			
			return aptWrap;
		}
	};
};

})();
