/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.query.QueryResults");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.fx");
dojo.require("dijit.form.CheckBox");
dojo.require("jazz.util.ResizeNotifier"); //$NON-NLS-1$
dojo.require("jazz.app.ResourceLink");
dojo.require("jazz.ui.ImagePreview");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.InlineWorkItemEditor");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
(function() { // module container

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var WorkItemUtil			= com.ibm.team.workitem.web.client.util;

var ImageClient				= com.ibm.team.workitem.web.client.internal.ImageClient;
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var commonImagePath = "plugin/com.ibm.team.workitem.web/resources/ui/internal/view/common/images/";
var ROW_WORK_ITEM_ID_PROP	= "_jazzRowWorkItemId";
var ROW_WORK_ITEM_ITEMID_PROP	= "_jazzRowWorkItemItemId";
var ROW_QUERY_PROP			 = "_jazzRowQuery";
var ROW_HEADERS_PROP			 = "_jazzHeaders";
var ROW_QUERY_ROW_PROP = "_jazzRowQueryRow";
var ROW_LINKS_PROP			 = "_jazzRowLinks";

// constant for stripping hyperlinks
var STRIP_HYPERLINK_REGEX	= /(<a.*?>)(.*?)(<\/a>)/gi;

function getRow(e) {
	var element= e.target || e.srcElement;
	if(element._isClickDisabled) return null;
	// walk up the DOM tree until we find a row element
	while (element && element.tagName !== "TR") {
		element= element.parentNode;
	}
	return element || null;
}

/**
 * Provides a UI to iterate through the work items in a query result.
 */
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.query.QueryResults",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/query/templates/QueryResults.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/query/templates/Query.css"),
		
		// object initializer
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
			this._rowCache= [];
			this._childWidgets = [];
			this._inlineEditors= {};
			this._twistieImages= {};
			this._filterInProgress= false;
			this._queuedFilter= null;
			
			this.textNoWorkItemsFound = this.messages.labelNoWorkItemsFound;
			this.textNoWorkItemsMatchFilter = this.messages.labelNoWorkItemsMatchFilter;
			
			this._workItemChangedSubscribeHandle = dojo.subscribe(EventTopics.WORKITEM_CHANGED, this, this._handleWorkItemChanged);
			this._viewChangedSubscribeHandle = dojo.subscribe(EventTopics.WORKITEM_VIEW_CHANGED, this, this._handleWorkItemViewChanged);
			this._itemProxyDestroyedHandle = dojo.subscribe(EventTopics.ITEMPROXY_DESTROYED, this, this._handleWorkItemDestroyed);

			// Initialize the configurations for known data types
			this._columnConfigs = {
				internalState: {
						showIcon: true	
					},				
				internalSeverity: {
						titlePrefix: this.messages.labelSeverity,
						titleSelector: "_getTypeTitle"	
					},
				internalPriority: {
						titlePrefix: this.messages.labelPriority,
						titleSelector: "_getTypeTitle"	
					},
				workItemType: {
						titlePrefix: this.messages.labelWorkItemType,
						titleSelector: "_getTypeTitle"	
					},
				id: {
						isLinked: true
					},
				summary: {
						isLinked: true,
						isHtml: true,
						formatterSelector: "_getSummaryLabel"
					},
				modified: {
						formatterSelector: "_getModifiedLabel",
						conversionSelector: "_getDateFromValue"
				},
				creationDate: {
						formatterSelector: "_getDateLabel",
						conversionSelector: "_getDateFromValue" 
				},
				resolutionDate: {
						formatterSelector: "_getDateLabel",
						conversionSelector: "_getDateFromValue" 
				},
				dueDate: {
						formatterSelector: "_getDateLabel",
						conversionSelector: "_getDateFromValue" 
				}				
			}
		},
		postCreate: function() {
			this.pageController = this.parentController;
		},
		destroy: function() {
			if(this.isDestroyed) return;
			this._resetTableBody();
			if(this._selectAllCheckbox) this._selectAllCheckbox.destroy();
			for(var key in this._inlineEditors) {
				if(this._inlineEditors[key] && dojo.isFunction(this._inlineEditors[key].destroy)){
					this._inlineEditors[key].destroy();
				}
			}

			if(this._workItemChangedSubscribeHandle) {
				dojo.unsubscribe(this._workItemChangedSubscribeHandle);
			}
			if(this._itemProxyDestroyedHandle) {
				dojo.unsubscribe(this._itemProxyDestroyedHandle);
			}
			if(this._viewChangedSubscribeHandle) {
				dojo.unsubscribe(this._viewChangedSubscribeHandle);
			}
			for(var i=0;i< this._childWidgets.length;i++) {
				if(this._childWidgets[i] && this._childWidgets[i].destroy) {
					this._childWidgets[i].destroy();			
				}
				this._childWidgets[i] = null;
			}
			this._childWidgets = [];
			this.isDestroyed = true;
			this.inherited(arguments);
		},
		/*
		 *  Return the results of the last query
		 */ 
		getResults: function() {
			return this._rowCache;
		},
		refreshLastQuery: function(query) {
			this.parentController.refreshLastQuery(query); //see WorkItemPageController 
		},
		/*
		 * Methods for table population and clearing.
		 */
		setQueryResultsTable: function(query, queryResultsSummaryPage, orderByAttribute, direction, itemsPerPage) {
			checkNotNull(query, "query");
			checkNotNull(queryResultsSummaryPage, "queryResultsSummaryPage");
			
			// if no matches, provide an empty arr for easier programming later
			if (!queryResultsSummaryPage.workItemSummaryDTOs) {
				queryResultsSummaryPage.workItemSummaryDTOs= [];
			}
			
			// reset the UI
			this._resetTableHeader();
			this._resetTableBody();
			
			// populate the UI
			this._populateTableHeader(queryResultsSummaryPage, orderByAttribute, direction);
			this._populateTableBody(query, queryResultsSummaryPage);

			// show the bottom page control
			dojox.data.dom.removeChildren(this._bottomPageControlSection);
			this.parentView.showQueryBottomPageControl(query,queryResultsSummaryPage, orderByAttribute, direction, itemsPerPage, this._bottomPageControlSection );
			
			this._updateRowHighlighting();			

			if(	this.parentView._queryResultPageControl.query._isTableEditModeSelected) {
				this.showCheckboxes(true, this.parentView._queryResultPageControl._bulkOperationsMomento.attributeTree);
			}
			jazz.util.ResizeNotifier.poke();
		},
		scrollResultsIntoView: function() {
			// scroll the first row into view
			setTimeout(dojo.hitch(this, function() {
				if(this._rowCache[0] && this._rowCache[0].row)
					dijit.scrollIntoView(this._rowCache[0].row);
			}), 0);			
		},
		/*
		 * methods to do with resetting the UI
		 */
		// clear header rows
		_resetTableHeader: function() {
			dojox.data.dom.removeChildren(this._headerRowLine);
			dojox.data.dom.removeChildren(this._headerRow);
		},
		
		// clear body rows
		_resetTableBody: function() {
			dojox.data.dom.removeChildren(this._tableBody);
			for (var i=0; i<this._rowCache.length; i++) {
				if(this._rowCache[i].row.checkbox) {
					this._rowCache[i].row.checkbox.destroy();
					this._rowCache[i].row.checkbox = null;
// broken in IE		delete this._rowCache[i].row.checkbox;
				}
				this._rowCache[i]= null;
			}
			this._rowCache= [];
		},
		
		/*
		 * methods to do with populating the UI
		 */
		
		_populateTableHeader: function(queryResultsSummaryPage, orderByAttribute, direction) {
			
			// if there are zero or one query results, it makes no sense to
			// allow users to reorder them
			var preventOrdering= queryResultsSummaryPage.rows == null || queryResultsSummaryPage.rows.length < 2;
			
			if (!preventOrdering) {
				var uriInfo= this._getUriInfo();
			}

			var self= this;
			function addCol(typeId, displayText, isOrderable, alignment, width, color, isIconOnly, preventOrdering, isTwistie, isSelectAll) {
				
				// If nothing specified, assume it is em
				if( width && 
					width.toLowerCase().indexOf("em") == -1 &&
					width.toLowerCase().indexOf("px") == -1) {
					width = width + "em"
				}
				
				if(typeId && typeId.indexOf("linktype") != -1) {
					var typeClassName = "linktype";
				} else {
					var typeClassName = typeId || "";
				}
				// Add top white line trim
				var headerColumnLine = document.createElement("th");
				dojo.addClass(headerColumnLine, "headerColumnLine");
				dojo.addClass(headerColumnLine, typeClassName);
				headerColumnLine.innerHTML = "&nbsp;";
				self._headerRowLine.appendChild(headerColumnLine);
				
				var th= document.createElement("th");
				if(width) th.style.width = width;
				
				th.className= typeClassName;
				
				if (isOrderable && !preventOrdering) {
					var link= document.createElement("a");
					link.appendChild(document.createTextNode(displayText));
					// IE Bug: Required because CSS whitespace nowrap does not work
					var nobr = document.createElement("nobr");
					nobr.appendChild(link);
					th.appendChild(nobr);
					var newDirection = "descending";					
					if (typeId === orderByAttribute) {
						// Add sorting direction
						var orderDirectionImage = document.createElement("img");
						var imagePath = "plugin/com.ibm.team.workitem.web/resources/ui/internal/view/query/images/";
						dojo.addClass(orderDirectionImage, "sortDirection");
						if (direction == "descending") {
							orderDirectionImage.alt = self.messages.labelDescending;
							orderDirectionImage.src = ImageClient.getImageUrl(imagePath + "q-result-down-arrow.gif");
							newDirection = "ascending";
						} else {
							orderDirectionImage.alt = self.messages.labelAscending;
							orderDirectionImage.src = ImageClient.getImageUrl(imagePath + "q-result-up-arrow.gif");
							newDirection = "descending";
						}
						link.appendChild(orderDirectionImage);
					}
					link.href= uriInfo.baseUri + (!uriInfo.isFirst ? "&" : "") + "orderBy=" + typeId + "&direction=" + newDirection;		
					link.title= dojo.string.substitute(self.messages.titleOrderResultsBy,  [ displayText , newDirection ]);
					// Style link
					dojo.addClass(link, "headerColumnSortedLink");
					// Style column
					dojo.addClass(th, "headerColumnSorted");
					self._addBulkOperationImage(link, typeId);
				} else {
					if(isSelectAll) {
						self._selectAllCheckbox = new dijit.form.CheckBox({value: -1, name: displayText});
						self._selectAllCheckbox.domNode.style.display = "none";
						th.title = self.messages.titleSelectDeselectAll;
						self._selectAllCheckbox.th = th;
						self._selectAllCheckbox.th2 = headerColumnLine;
						self._selectAllCheckbox.th2.style.display = "none";
						th.style.display = "none";					
						th.appendChild(self._selectAllCheckbox.domNode);
						th._isSelected = false;
	
						self.connect(self._selectAllCheckbox, "onClick", function(e) {
							dojo.stopEvent(e);
							self._selectAll();
						});
					} else {
						th.appendChild(document.createTextNode(displayText));
						if(!isTwistie) {
							self._addBulkOperationImage(th, typeId);
						}
					}

					// Style column
					dojo.addClass(th, "headerColumnUnsorted");

				}
				self._headerRow.appendChild(th);
			}
			addCol(/* id */ "selection", /* label */ this.messages.titleSelectDeselectAll, /* isOrderable */ false , /* alignment */ "right", /* width */ "0.8em" 
					, /* color */ "cyan", /* iconOnly */ true, /* preventOrdering */ true, /* isTwistie */ false, /* isSelectAll */ true);

			
			dojo.forEach(queryResultsSummaryPage.headers,
				function(col) {
					addCol(col.attributeId, col.label, col.isOrderable, col.alignment, col.width, col.color, col.iconOnly, preventOrdering );
				});
			if(!dojo.isSafari) {
				addCol(/* id */ "twistie", /* label */ this.messages.labelActions, /* isOrderable */ false , /* alignment */ "right", /* width */ "0.5em" 
					, /* color */ "cyan", /* iconOnly */ true, /* preventOrdering */ true, /* isTwistie */ true);
			} else {
				addCol(/* id */ "twistie", /* label */ this.messages.labelActions, /* isOrderable */ false , /* alignment */ "right", /* width */ "4em" 
					, /* color */ "cyan", /* iconOnly */ true, /* preventOrdering */ true, /* isTwistie */ true);				
			}

		},
		_addBulkOperationImage: function(node, typeId) {
			this._bulkOperationImages = this._bulkOperationImages || [];
			var bulkAttrImage = document.createElement("img");
			var imagePath = "plugin/com.ibm.team.workitem.web/resources/ui/internal/view/queryeditor/images/";
			bulkAttrImage.src = ImageClient.getImageUrl(imagePath + "add-condition-th.gif");
			bulkAttrImage.src_normal = ImageClient.getImageUrl(imagePath + "add-condition-th.gif");
			bulkAttrImage.src_hover = ImageClient.getImageUrl(imagePath + "add-condition-th-hov.gif");
			bulkAttrImage.style.display = "none";
			bulkAttrImage.className = "header-add-bulk-operation-img";
			bulkAttrImage.attributeId = typeId;
			bulkAttrImage.title = this.messages.titleAddOperation;
			this.connect(bulkAttrImage, "onmouseover",  function(e) {
				var img = e.currentTarget;
				if(img) {
					img.src = img.src_hover;
				}
			});
			this.connect(bulkAttrImage, "onmouseout",  function(e) {
				var img = e.currentTarget;
				if(img) {
					img.src = img.src_normal;
				}
			});
			this.connect(bulkAttrImage, "onclick", dojo.hitch(this, this._handleHeaderBulkAdd));
			node.appendChild(bulkAttrImage);
			this._bulkOperationImages.push(bulkAttrImage);
		},
		clearMessages: function() {
			this.parentView._queryResultPageControl.logClearMessages();
		},		
		_handleHeaderBulkAdd: function(e) {
			dojo.stopEvent(e);
			var attributeId = e.currentTarget.attributeId;
			if(this.parentView._queryResultPageControl) {
				this.parentView._queryResultPageControl.addBulkOperationAttribute(attributeId);
			}
		},
		showCheckboxes: function(isShown, attributeTree) {
			if(!attributeTree) return; // not yet initialized;
			this._attributeTree = attributeTree;
			this._isShowingCheckboxes = isShown;
			var displayValue = isShown ? "" : "none"; 
			this._selectAllCheckbox.domNode.style.display = displayValue;
		    this._selectAllCheckbox.th.style.display = displayValue;
		    this._selectAllCheckbox.th2.style.display = displayValue;
			for (var i=0; i<this._rowCache.length; i++) {
				this._rowCache[i].row.checkbox.domNode.style.display = displayValue;
				var td = this._rowCache[i].row.checkbox.domNode.parentNode;
				td.style.display = displayValue;
			}
						
			for(var i=0;i<this._bulkOperationImages.length;i++) {
				var img = this._bulkOperationImages[i];
				var link = this._bulkOperationImages[i].parentNode;
				
				if (isShown && !img._attribute && attributeTree && attributeTree.children) {
					// FInd the attribute in the tree. It should be 
					var treeAttribute = null;
					for (var j = 0; j < attributeTree.children.length; j++) {
						if (attributeTree.children[j].item.id == img.attributeId) {
							treeAttribute = attributeTree.children[j];
							break;
						}
					}	
					img._attribute = treeAttribute || {};
				}
				if (isShown && img._attribute && img._attribute.item && img._attribute.item.isBulkEditable &&  !img._attribute.item.isMultiValued) {
					img.style.display = displayValue;
				} else {
					img.style.display = "none";
				}
				// Disable the sort links
				if(isShown) {
					if(link && link.nodeName && link.nodeName.toLowerCase() == "a") {
						img.parentNode_handle = this.connect(link, "onclick", dojo.hitch(this, this._ignoreClick));
						img.parentNode_title = link.title;
						link.title = this.messages.titleSortingIsDisabled;	
					}
				} else {
					if(img.parentNode_handle) {
						this.disconnect(img.parentNode_handle);
					}
					if(img.parentNode_title) {
						link.title = img.parentNode_title;
					}
				}
			}
			this.parentView._queryResultPageControl.setRunButtonEnabled();
		},
		_ignoreClick: function(e){
			dojo.stopEvent(e);
		},
		_selectAll: function() {
			this._isRowSelectionDisabled = true;
			var state = this._selectAllCheckbox.checked;
			for(var i=0;i<this._rowCache.length;i++) {
				if(this._rowCache[i].row.checkbox) {
					this._rowCache[i].row.checkbox.attr('checked',state);
				}
			}	
			this._isRowSelectionDisabled = false;
			this.parentView._queryResultPageControl.handleRowSelectionChanged();
		},
		_rowSelected: function(e) {
			if(this._isRowSelectionDisabled) return;
			var checkbox = e.currentTarget;
			var isShiftClick = e.shiftKey;
			if(isShiftClick && this._lastSelectedCheckboxId) {
				this._selectMultipleRows(this._lastSelectedCheckboxId, checkbox.value);
			}
			this._lastSelectedCheckboxId = checkbox.value;
			this.parentView._queryResultPageControl.handleRowSelectionChanged();
		},
		_selectMultipleRows: function(fromId, toId) {
			this._isRowSelectionDisabled = true;
			var startRow = this._findRow(fromId);
			var startState = startRow.row.checkbox.checked;
			var startIndex = startRow.index;
			var endIndex = this._findRow(toId).index;
			if(startIndex <= endIndex ) {
				var start = startIndex;
				var end = endIndex;
			} else {
				var start = endIndex;
				var end = startIndex;				
			}

			for(var i=start;i<end;i++) {
				if(this._rowCache[i].row.checkbox) {
					this._rowCache[i].row.checkbox.attr("checked", startState);
				}
			}
			this._isRowSelectionDisabled = false;
		},
		getSelectedRows: function() {
			var selectedRows = [];
			for(var i=0;i<this._rowCache.length;i++) {
				if(this._rowCache[i].row.checkbox && this._rowCache[i].row.checkbox.checked) {
					selectedRows.push(this._rowCache[i]);
				}
			}
			return selectedRows;
		},
		// get the current URI fragment, sans 'orderBy' and 'direction' attributes
		_getUriInfo: function() {
			var uriInfo= {
				baseUri: "#",
				isFirst: true
			};
						
			var oldFrag= WorkItemUtil.getFragment(document.location.href);
			if (oldFrag) {
				var nvps= oldFrag.split("&");
				for (var i=0; i<nvps.length; i++) {
					var nvp= nvps[i].split("=");
					// strip off orderBy and direction (we'll deal with them
					// later); also strip off 'index' since we want to go back
					// to page 1 on a re-order
					if (nvp[0] != "orderBy" && nvp[0] != "direction" && nvp[0] != "index") {
						if (uriInfo.isFirst) {
							uriInfo.isFirst= false;
						} else {
							uriInfo.baseUri += "&";
						}
						uriInfo.baseUri += nvp[0] + "=" + nvp[1]; 
					}
				}
			}
			
			return uriInfo;
		},
		_getLabelAttributeFromRow: function(row, headers, attributeId) {
			for(var i=0;i<headers.length;i++) {
				if(headers[i].attributeId == attributeId) {
					return row.labels[i];
				}
			}
			return null;
		},
		_populateTableBody: function(query, queryResultsSummaryPage) {
			// if we don't have any work item summaries display the "no results" message
			if (queryResultsSummaryPage.rows == null || !queryResultsSummaryPage.rows.length || queryResultsSummaryPage.rows.length === 0) {
				this._setNoResultsVisibility(/* isVisible */ true);
				this._bottomPageControlSection.style.display = "none";
			} else {
				this._setNoResultsVisibility(/* isVisible */ false);
				this._bottomPageControlSection.style.display = "block";
				
				// Each row has two values... icons and labels
				// icons is an array for each column, what icon to display
				// labels is an array for each colum, what lable to display
				// if the column header says "icononly", then only show the icon
				// if the column header says "color", then use that color
				// use the width and alignment from the header
				var rows= queryResultsSummaryPage.rows;
				var headers= queryResultsSummaryPage.headers;
				for (var i=0; i<rows.length; i++) {
					var row= document.createElement("tr");
					this._tableBody.appendChild(row);
					this._populateRow(row, rows[i], headers, query);
					// find the owner and summary, if any
					var summary = this._getLabelAttributeFromRow(rows[i], headers, "summary");
					var ownerName = this._getLabelAttributeFromRow(rows[i], headers, "ownerName");
					this._rowCache[i]= {
						row: row,
						indices: [
							rows[i].id,
							summary == null ? "" : summary.toLowerCase(),
							ownerName == null ? "" : ownerName.toLowerCase(),
							summary == null ? "" : summary
							]
					};
				}
			}
		},
		_findRow: function(id) {
			for (var i=0; i<this._rowCache.length; i++) {
				if(this._rowCache[i].indices[0] == id) {
					return {
						row: this._rowCache[i].row,
						index: i
					} ;
				}
			}
			return null;
		}, 
		_updateQueryRowFromWorkingCopy: function(queryRow, headers) {
			var workingCopy = com.ibm.team.workitem.web.cache.internal.Cache.getItem(queryRow.id);
			if(!workingCopy) {
				return queryRow;
			}
			var i = 0;
			var self = this;
			dojo.forEach(headers, function(header) {
					if (header && header.attributeId) {
						var value = workingCopy.getValue({
							path: ["attributes", header.attributeId]
						});
						if (value) { // TODO: support link attributes;
							if (value.iconUrl) {
								queryRow.iconUrls[i] = value.iconUrl;
							}
							else {
								queryRow.iconUrls[i] = "";
							}
							var valueLabel = value.label || value.content || value.name;
							if (valueLabel) {
								if (self._columnConfigs[header.attributeId] && self._columnConfigs[header.attributeId].conversionSelector) {
									valueLabel = self[self._columnConfigs[header.attributeId].conversionSelector](valueLabel, value.id);
								}
								else {
									if (header.attributeType == 'timestamp') {
										valueLabel = self._getDateFromValue(valueLabel, value.id);
									}
								}
							}
						  queryRow.labels[i] = valueLabel || "";
						};
											}
					i++;
				}
			)
			return queryRow;
		},
		_populateRow: function(tableRow, queryRowFromQuery, headers, query) {
			checkNotNull(tableRow, "tableRow");
			checkNotNull(queryRowFromQuery, "queryRow");
			var columnElements= [];
			var queryRow = this._updateQueryRowFromWorkingCopy(queryRowFromQuery, headers);

			var viewWorkItemUrl= WorkItemUtil.getViewWorkItemUri(queryRow.id);
			
			tableRow[ROW_WORK_ITEM_ID_PROP]= queryRow.id;
			tableRow[ROW_WORK_ITEM_ITEMID_PROP]= queryRow.itemId;
			tableRow[ROW_QUERY_ROW_PROP]= queryRowFromQuery;
			tableRow[ROW_QUERY_PROP]= query;
			tableRow[ROW_HEADERS_PROP]= headers;
			
			var i = 0;
			var self = this;
			dojo.forEach(headers, function(header) {
				if(header && header.attributeId && header.attributeId.indexOf("linktype") != -1) {
					var typeClassName = "linktype";
				} else {
					var typeClassName = header.attributeId || "";
				}
				columnElements.push(self._doColumnElementHelper(
						self._createColumnElement(tableRow, queryRow, header, i++, viewWorkItemUrl ),
						typeClassName));
				}
			);
			
	    // Add a Checkbox
			var colNum = 0;
			var col= tableRow.insertCell(colNum++);
			if(col == null) {
				col = document.createElement("TD");
				tableRow.appendChild(col);
			}
			col._isControl = true;
			tableRow.checkbox = new dijit.form.CheckBox({value: queryRow.id, name: queryRow.id +""});
			tableRow.checkbox.domNode.style.display = "none";
//			tableRow.checkbox.td = col;
			col.style.display = "none";
			col.style.cursor = "default";
			col.appendChild(tableRow.checkbox.domNode);

			this.connect(tableRow.checkbox, "onClick", function(e) {
					dojo.stopEvent(e);
					this._rowSelected(e);
				});

			// disable the row onclick for this box since it causes accidental drill down missing the checkbox
			col._isClickDisabled = true;
			
			
			// Add the columns
			for (var i=0; i<columnElements.length; i++) {
				var col= tableRow.insertCell(colNum++);
				if(col == null) {
					col = document.createElement("TD");
					tableRow.appendChild(col);
				}
				if(headers[i].width && headers[i].width > 0) { col.style.width = headers[i].width; }
				if(headers[i].alignment) { col.style.align = headers[i].alignment; }
				if(headers[i].color) { col.style.color = headers[i].color; }
				col.appendChild(columnElements[i].element);
				col._resultIndex = i;
				if (columnElements[i].className) {
					col.className= columnElements[i].className;
				}
			}
			
	    // Add a Twistie
			var col= tableRow.insertCell(colNum++);
			if(col == null) {
				col = document.createElement("TD");
				tableRow.appendChild(col);
			}
			col._isControl = true;
			col._workItemId = queryRow.id;
			dojo.addClass(col, "twistie");;
			var twistieImage = document.createElement("img");

			twistieImage.src = ImageClient.getImageUrl(commonImagePath + "row-edit.gif");
			twistieImage.style.visibility = "hidden";
			col._editorToggle = twistieImage;
			this._twistieImages["twistie-"+ queryRow.id] = twistieImage;
			
			col.appendChild(twistieImage);
			col._isExpanded = false;
			this.connect(col, "onclick", function(e) {
					dojo.stopEvent(e);
				    if(!twistieImage._isExpanded) {
						self._showEditorTwistie(twistieImage, /* open */ true )
						self._openPreviewEditor(col);						
					} else {
//						self._showEditorTwistie(twistieImage, /* close */ false );
//						var widget = this._inlineEditors["InlineWorkItemEditor-" + col._workItemId];
//						if(widget) {
//							self.hidePreviewEditor(widget.parentParms);
//						}
					}
				});
			this.connect(col.parentNode, "onmouseover", function(e) {
					twistieImage.style.visibility = "";
				});
			this.connect(col.parentNode, "onmouseout", function(e) {
				    if(!twistieImage._isExpanded) {
						twistieImage.style.visibility = "hidden";
					}
				});

			return tableRow;
		},
		_hideAllEditors: function() {
			for(var i=0; i< this._rowCache.length; i++){
				var row = this._rowCache[i].row;
				this._hideEditorInRow(row);
			}
		},
		_showEditorTwistie: function(twistieImage, isOpen) {
 				if(isOpen) {
						twistieImage.src = ImageClient.getImageUrl(commonImagePath + "row-edit-open.gif");
						twistieImage._isExpanded = true;
						twistieImage.style.visibility = "";
					} else {
						twistieImage.src = ImageClient.getImageUrl(commonImagePath + "row-edit.gif");
						twistieImage._isExpanded = false;
						twistieImage.style.visibility = "hidden";
					}			
		},
		_openPreviewEditor: function(col) {
			var workItemId = col._workItemId;
			var resultIndex = col._resultIndex;
			var row = col.parentNode;
			dojo.addClass(row, "double-row");
			
			var widget = this._inlineEditors["InlineWorkItemEditor-" + col._workItemId];
			if(widget && !widget.isDestroyed) {
				if(!widget._isDirty) widget.refresh(/* isForced */ false, /* isInit */ true);
				widget.parentParms.row.style.display="";
				return;					
			} 
			var newRow = row.parentNode.insertRow(row.rowIndex-1);
			var newCol = document.createElement("td");
			newCol.colSpan = row.childNodes.length -1;

			var widget = new com.ibm.team.workitem.web.ui2.internal.widget.InlineWorkItemEditor({ workItemId: workItemId, parentWidget: this, parentParms: { workItemId: workItemId, parentRow: row, row: newRow, actionColumn: col }});
			this._inlineEditors["InlineWorkItemEditor-" + workItemId] = widget;
			
			newCol.appendChild(widget.domNode);

			newRow.appendChild(newCol); 
		},
		_createColumnElement: function(tableRow, queryRow, header, i, workItemUrl) {
				// Header has:
				//  attributeId: "workItemType"
				//  attributeType: type, isOrderable
				//  label, width, alignment, color
				//  iconOnly
				var config = this._columnConfigs[header.attributeId];
				if(!config) { config = {}};
				var span = document.createElement("SPAN");
				span._workItemId = queryRow.id;
				if(config.className) {
					span.className = config.className;
				}
				
				if (!dojo.isIE || dojo.isIE > 6) {
					if (queryRow.iconUrls[i] && this._columnConfigs[header.attributeId] && this._columnConfigs[header.attributeId].showIcon) {
						var img = document.createElement("IMG");
						img.className = "queryResultIcon";
						img.src = queryRow.iconUrls[i];
						span.appendChild(img);
						dojo.addClass(span, "iconBackground");
					}
				}

				var parentElement = span;
				if(config.isLinked) {
					var link= document.createElement("a");
					link.href= workItemUrl;
					span.appendChild(link);
					parentElement = link;

					if(tableRow[ROW_LINKS_PROP] == null) {
						tableRow[ROW_LINKS_PROP] = [];
					}
					tableRow[ROW_LINKS_PROP].push(span);
				}
				// If there is an icon URL then add an icon
				if(header.iconOnly && queryRow.iconUrls[i]) {
					var titlePrefix = header.attributeType;
					if(config.titleSelector) {titlePrefix = this[config.titleSelector](config, header) };
					parentElement.appendChild(this._createAttributeImage(titlePrefix, queryRow.labels[i], queryRow.iconUrls[i]));				
				}
				// If it isn't icon only then add the text too
				if(!header.iconOnly) {
					var label = queryRow.labels[i];
					if(config.formatterSelector) {
						label = this[config.formatterSelector](label);
					} else {
						if(header.attributeType == 'timestamp') {
							label = this._getDateLabel(label);
						}
					}
					
					var domElement
					if (com.ibm.team.workitem.web.client.util.isStringDefined(label) == false) {
						if( queryRow.info[i] && queryRow.info[i].item && queryRow.info[i].item.length > 0 ) {
							try{
								domElement = this._getLinkItemUL(queryRow.info[i].item);
							} catch(e) {
								console.debug(e.message);
								domElement = document.createTextNode("+");
							}
							isText = false;
						} else {
							// IE: Border is not rendered unless there is content
							dojo.addClass(parentElement, "emptyContent");
							domElement = document.createTextNode("-");
						}
					} else {
						domElement = document.createTextNode(label);
					} 
					
					parentElement.appendChild(domElement);
				}
				
				return span;

		},
		_getLinkItemUL: function(itemList) {
			/*
			 * First go through the item list and resolve shortName, stateGroup, safeHref and safeComment for each item
			 */
			for (var k=0; k<itemList.length; k++) {
					try {
					if(itemList[k].id == null && itemList[k].target && itemList[k].target.attributes) {
						for(var j=0;j<itemList[k].target.attributes.length;j++) {
							if(itemList[k].target.attributes[j].key == "id" && itemList[k].target.attributes[j].value && itemList[k].target.attributes[j].value.id) {
								itemList[k].id = itemList[k].target.attributes[j].value.id;
								break;
							}
						}
						for(var j=0;j<itemList[k].target.attributes.length;j++) {
							if(itemList[k].target.attributes[j].key == "stateGroup" && itemList[k].target.attributes[j].value && itemList[k].target.attributes[j].value.id) {
								itemList[k].stateGroup = itemList[k].target.attributes[j].value.id;
								break;
							}
						}
					}
					} catch(e) {}; // ignore  problem here
					// Prefer the weburi if there is one
					itemList[k].newWindow = true;

					if (itemList[k].weburi) {
						itemList[k].newWindow = false;
					}
					if(itemList[k].url) {
						itemList[k].safeHref = EditorUtil.removeHrefScripting(itemList[k].url); // remove scripts
					}
					if (itemList[k].comment) {
						itemList[k].safeComment = WorkItemUtil.plainToHTML(itemList[k].comment); // remove scripts
					} else {
					    // there can be no comment in related artifacts
						if(itemList[k].safeHref) {
							itemList[k].safeComment = itemList[k].safeHref;
						} else { // may never happen
							if(itemList[k].id) {
								itemList[k].safeComment =itemList[k].id;
							} 
						}
					}
					itemList[k].shortName = itemList[k].safeComment;
					if(("" + itemList[k].shortName).length > 500) {
						itemList[k].shortName = itemList[k].shortName.substring(0,500) + "...";
					}
				}
			// itemList[k].shortName, itemList[k].stateGroup, itemList[k].safeHref and itemList[k].safeComment should now exist for each item where valid.
			var anchor, itemCount =0;
			var iconNode;

			var table = document.createElement("TABLE");
			var tableBody= document.createElement("TBODY");
			table.appendChild(tableBody);
			
			var itemsToRender= Math.min(itemList.length, 10);

 			for (var i = 0; i < itemsToRender; i++) {
				var item = itemList[i];
				
				var imageTD = document.createElement("TD");
				dojo.addClass(imageTD, "linkColumnIcon");
				
				var linkTD = document.createElement("TD");
				dojo.addClass(linkTD, "linkColumn");
				
				if (i == itemsToRender-1) {
					dojo.addClass(imageTD, "lastLinkColumn");
					dojo.addClass(linkTD, "lastLinkColumn");
				}
				
				var tr = document.createElement("TR");
				tr.appendChild(imageTD);
				tr.appendChild(linkTD);
				
				tableBody.appendChild(tr);
				
				if(item.safeHref) {
					if(item.shortName) {
						
						anchor = document.createElement("A");
						anchor.innerHTML = item.shortName; // innerHTML OK, shortName is safe
						anchor.href = item.safeHref;
						
						if(item.stateGroup == 2) {
							dojo.addClass(anchor, "Closed");
						}
					
						if(item.newWindow) {
							anchor.target = "_blank";
						}
						
						linkTD.appendChild(anchor);
						
						var isBinary= (item.target && item.target.contentType);
						
						if (isBinary) {
							var isImage= isBinary && (item.target.contentType.indexOf("image/") == 0);
							if (isImage && item.weburi) {
								new jazz.ui.ImagePreview({ around: [anchor], src: item.weburi, filename: item.shortName }, document.createElement("div"));
							}
						} else {
							var resourceLinkArgs = { retainLinkText: false, uri: item.safeHref, iconNode: imageTD };
							if (item.weburi) {
								resourceLinkArgs.presentationUri = item.weburi;
							}
							this._childWidgets.push(new jazz.app.ResourceLink(resourceLinkArgs, anchor));
						}
						
						// TODO: Open local links in the current window
						this.connect(anchor,"onclick",function(e) {
							dojo.stopEvent(e);
							if(e.target) {
								if (e.target.href) {
								    if(e.target.target == "_blank") {
									// open the same work item in the same new window
									try {
									if (e.target.text) { // Firefox
										window.open(e.target.href, "_wi" + e.target.text.replace(/,/,""));
									}
									else 
										if (e.target.innerText) { // IE 
											window.open(e.target.href, "_wi" + e.target.innerText.replace(/,/,""));
										}
										else {
											window.open(e.target.href);
										}
									} catch (e) {
									// Some bad text might blow up open
										window.open(e.target.href);
									}
									} else {
									   window.location.href = e.target.href;
									}
								}
							} 
							});
					}
				} else {
					if(item.shortName) {
						var div = document.createElement("DIV");
						div.innerHTML = item.shortName; // innerHTML OK, shortName is safe
						linkTD.appendChild(div);
					}
				}
			}
			// Limit to 10 items in the cell
			if(itemList.length > itemsToRender) {
				
				var formatted= dojo.string.substitute(this.messages.labelMoreLinks, [ itemList.length - itemsToRender ]);
				
				var td = document.createElement("TD");
				td.setAttribute("colspan", "2");
				td.appendChild(document.createTextNode(formatted));
				dojo.addClass(td, "lastLinkColumn");
				
				var tr = document.createElement("TR");
				tr.appendChild(td);
				
				tableBody.appendChild(tr);
			}
 			return table;
				
		},
		_getTypeTitle: function(config, header) {
			var prefix = config.titlePrefix == null ? header.label : config.titlePrefix;
			return prefix;
		},
		
		/*
		 * Looks for long unbroken strings ( > 40 chars ) and breaks them up so
		 * that the table doesn't get pushed beyond off the right side of the
		 * page.
		 *
		 * An example long unbroken string is:
		 * com.ibm.team.filesystem.tests.api.IFileSystemViewTest.test_deleteSubtreeLcom_ibm_team_scm_common_IChangeSetHandleLcom_ibm_team_scm_common_IComponentHandleLcom_ibm_team_repository_common_IVersionableHandle
		 */
		_getSummaryLabel: function(summary) {
			summary= this._removeHyperlinks(summary);
			summary = EditorUtil.restoreTags(summary);
			return WorkItemUtil._breakupAllLongWords(summary);
		},
		
		// if any occurence of a hyperlink, strip out hyperlink tags
		// (nested links are illegal as per HTML 4, section 12.2.2)
		// http://www.w3.org/TR/html4/struct/links.html#idx-link-6
		_removeHyperlinks: function(summary) {
			return summary.indexOf("<a") === -1 ?
					summary :
					summary.replace(STRIP_HYPERLINK_REGEX, "$2");
		},
		
		_doColumnElementHelper: function(element, className) {
			return {
				element: element,
				className: className ? className : null
			};
		},
		
		_setNoResultsVisibility: function(isVisible) {
			this._noResultsMsg.className=
				isVisible ? "visible" : "hidden";
		},
		
		_createAttributeImage: function(attributeName, attributeTextValue, iconUrl) {
			var img= document.createElement("img");
			img.title= attributeName + ": " + (attributeTextValue || this.messages.labelNotSet);
			img.alt= (attributeTextValue || this.messages.labelNotSet);
			img.src= iconUrl ?
					ImageClient.getImageUrl(iconUrl) :
					ImageClient.getImageUrl("/com.ibm.team.workitem.common/icons/obj16/type_none.gif");
			return img;
		},
		_getDateFromValue: function(timeLong, isoString) {
			if(isNaN(timeLong) && isoString) {
				var isoDate = WorkItemUtil.fromISOString(isoString);
				return isoDate.getTime();
			}
			return timeLong;
		},
		_getModifiedLabel: function(modified) {
			if(isNaN(modified)) return modified;  // It's a string, probably "Unassigned"
			var date = new Date()
			date.setTime(modified);
			return WorkItemUtil.getModifiedLabel(date);
		},
		_getDateLabel: function(timeLong) {
			if(!timeLong || isNaN(timeLong)) {
				return this.messages.labelNone;
			}
			var date = new Date()
			date.setTime(timeLong);
			return WorkItemUtil.getDateLabel(date);
		},
	
		/*
		 * Methods related to filtering
		 */
		
		filter: function(filterStr) {
			if (!dojo.isString(filterStr)) {
				throw new Error("QueryResults#filter: input must be of type string");
			}

			// if a filter is in progress, queue the incoming request and return
			if (this._filterInProgress) {
//				console.debug("queueing filter: " + filterStr);
				this._queuedFilter= filterStr;
				return;
			}
			
			//
			// BEGIN GUARDED SECTION
			//
			this.parentController._lastFilter=filterStr;
			this._filterInProgress= true;
			filterStr= dojo.trim(filterStr).toLowerCase();
			if (filterStr === "") {
				this._doUnfilterAll();
				this._setNoFilterMatchesVisibility(/* isVisible */ false);
			} else {
				this._doFilter(filterStr);
			}
			this._filterInProgress= false;
			//
			// END GUARDED SECTION
			//

			// if we queued a filter during processing, clear it and call
			// ourselves with the queued filter
			if (this._queuedFilter !== null) {
//				console.debug("Applying queued filter: " + this._queuedFilter);
				var newStr= this._queuedFilter;
				this._queuedFilter= null;
				this.filter(newStr);
			}
		},
		
		_doFilter: function(filterStr) {
			var noMatches= true;
			for (var i=0; i<this._rowCache.length; i++) {
				var row= this._rowCache[i].row;
				var indices= this._rowCache[i].indices;
				if (this._isMatch(row, indices, filterStr)) {
					this._showRow(row);
					noMatches= false;
				} else {
					this._hideRow(row);
				}
			}
			this._setNoFilterMatchesVisibility(/* isVisible */ noMatches);
		},
		
		_doUnfilterAll: function() {
			for (var i=0; i<this._rowCache.length; i++) {
				this._showRow(this._rowCache[i].row);
			}
		},
		
		_isMatch: function(row, indices, filterStr) {
			for (var i=0; i<indices.length; i++) {
				//alert("indices[i]: " + indices[i] + ", filterStr: " + filterStr + ", index: " + indices[i].indexOf(filterStr));
				if ((indices[i]+ "").indexOf(filterStr) !== -1) {
					return true;
				}
			}
			return false;
		},
		
		_showRow: function(row) {
			row.className= "visibleRow";
			var id = row[ROW_WORK_ITEM_ID_PROP];
			var widget = this._inlineEditors["InlineWorkItemEditor-" + id];
			var image = this._twistieImages["twistie-"+ id] || {};
			if(widget && image._isExpanded) {
				this.unhidePreviewEditor(widget.parentParms);
			}				
			var workItemProxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem(id);
			if (workItemProxy && workItemProxy.isChanged()) {
					dojo.addClass(row, "save-pending");
				} else {
					dojo.removeClass(row, "save-pending");
				}
		},
		
		_hideRow: function(row) {
			row.className= "hidden";
			this._hideEditorInRow(row, true);
		},
		_hideEditorInRow: function(row, keepTwistie) {
			var id = row[ROW_WORK_ITEM_ID_PROP];
			var widget = this._inlineEditors["InlineWorkItemEditor-" + id]; 
			if(widget) {
				this.hidePreviewEditor(widget.parentParms, /* keep twistie state*/ keepTwistie);
			}			
		},
		_setNoFilterMatchesVisibility: function(isVisible) {
			this._noFilterMatches.className=
				isVisible ? "visible" : "hidden";
		},
		
		/*
		 * Event handlers
		 */
		
		_handleTbodyMouseOver: function(e) {
			var row= getRow(e);
			if (!row) { return; }
			if(row.isDisabled) return; // TODO: TEMPORARY FIX
			var rowLinks= row[ROW_LINKS_PROP];
			if (!rowLinks) { return; }
			for (var i=0; i<rowLinks.length; i++) {
				rowLinks[i].firstChild.className= "linkMouseover";
			}
		},
		
		_handleTbodyMouseOut: function(e) {
			var row= getRow(e);
			if (!row) { return; }
			this._removeRowLinkHighlighting(row);
		},
		
		_removeRowLinkHighlighting: function(row) {
			if(!row) { return; }
			var rowLinks= row[ROW_LINKS_PROP];
			if (!rowLinks) { return; }
			for (var i=0; i<rowLinks.length; i++) {
				rowLinks[i].firstChild.className= "";
			}
		},
		
		_handleTbodyMouseClick: function(e) {
			if(e.ctrlKey || e.shiftKey || e.altKey ) { // alt clicks open tabs or windows
				return;
			}
			var row= getRow(e);
			if (!row) { return; }

			var id= row[ROW_WORK_ITEM_ID_PROP];
			if(!id) return;
						
			window.location.href= WorkItemUtil.getViewWorkItemUri(id);
		},
		_updateRowHighlighting: function() {
			var workingCopies = com.ibm.team.workitem.web.cache.internal.Cache.getAllItems({filterAttributes: [ "isWorkItem" ]});
			for(var i=0;i<workingCopies.length;i++) {
				this._handleWorkItemChanged(workingCopies[i])
			}
		},
		_handleWorkItemViewChanged: function() {
			this._updateRowHighlighting();
		},
		_handleWorkItemDestroyed: function(id) {
			this._handleWorkItemChanged(null, id);
		},
		_handleWorkItemChanged: function(workItemProxy, id) {
			if((!workItemProxy || !workItemProxy.object ) && id) return;
			if(workItemProxy.isNotCached) return; //temporary item used in bulk operations
			if(workItemProxy.isInBackgroundOperation) return; //skip for now
			if(workItemProxy) {
				id = workItemProxy.getId();
			}
			var row = this._findRow(id);
			var rowCheckboxDisplay = "";
			var rowCheckboxChecked = false;
			if(row) {
				if(row.row && row.row.checkbox && row.row.checkbox.domNode && row.row.checkbox.domNode.style) {
					rowCheckboxDisplay = row.row.checkbox.domNode.style.display; 
					rowCheckboxChecked = row.row.checkbox.checked;
					row.row.checkbox.destroy(); 
					if(row.row.childNodes[0] && row.row.childNodes[0]._isControl) row.row.childNodes[0]._isControl = false;
					row.row.checkbox = null
// breaks IE		delete row.row.checkbox;
				}
				var newRow= document.createElement("tr");
				this._populateRow(newRow,row.row[ROW_QUERY_ROW_PROP], row.row[ROW_HEADERS_PROP], row.row[ROW_QUERY_PROP] );
			    row.row.checkbox = newRow.checkbox;
			    row.row.checkbox.domNode.style.display = rowCheckboxDisplay;
			    row.row.checkbox.domNode.parentNode.style.display = rowCheckboxDisplay;		// TD	    
			    row.row.checkbox.attr('checked',rowCheckboxChecked);
			    
				var cols = [];
				for(var i=0;i<row.row.childNodes.length;i++) {
					if(row.row.childNodes[i]._isControl) {
						cols.push(row.row.childNodes[i]);
					} else {
						cols.push(newRow.childNodes[i]);
						newRow.childNodes[i].style.display = row.row.childNodes[i].style.display;
					}
				}
				dojox.data.dom.removeChildren(row.row);
				var length = newRow.childNodes.length;
				for(var i=0;i<length; i++ ) {
					row.row.appendChild(cols[i]);
				}
				if (workItemProxy) {
					if (workItemProxy.isChanged()) {
						dojo.addClass(row.row, "save-pending");
//						dojo.removeClass(row.row, "item-open");
					}
					else {
						dojo.removeClass(row.row, "save-pending");
//						dojo.addClass(row.row, "item-open");
					}
				}
				else {
					dojo.removeClass(row.row, "save-pending");
//					dojo.removeClass(row.row, "item-open");
				}
			}
		},
		clearSelection: function() {
			if(!this._rowCache) { return; }
			for (var i = 0;i<this._rowCache.length; i++) {
				var value = this._rowCache[i];
				if(value && value.row) {
					this._removeRowLinkHighlighting(value.row);
				}
			};
		},
		unhidePreviewEditor: function(parms) {
			parms.row.style.display="";
			dojo.addClass(parms.parentRow, "double-row");
			this._showEditorTwistie(parms.actionColumn._editorToggle, true);
		},
		hidePreviewEditor: function(parms, keepTwistie, isClose) {
			parms.row.style.display="none";
			dojo.removeClass(parms.parentRow, "double-row");
			if(!keepTwistie) {
				this._showEditorTwistie(parms.actionColumn._editorToggle, false);
			}
			// If we close the preview editor
			// and the working copy use count is zero, unhighlight the row
			// or if the working copy is not changed, unhighlight the row
			if(isClose) {
				var widget = this._inlineEditors["InlineWorkItemEditor-" + parms.actionColumn._workItemId];
				if(widget) {
					var workItemProxy = widget.workingCopy;
//					widget.destroy(); // Done in the editor clsoe
					this._inlineEditors["InlineWorkItemEditor-" + parms.actionColumn._workItemId] = false;
				}
				if(parms.parentRow) {
					if(workItemProxy && workItemProxy.getUseCount() > 0 && workItemProxy.isChanged()) {
						dojo.addClass(parms.parentRow, "save-pending");						
					} else {
						dojo.removeClass(parms.parentRow, "save-pending");
					}		
				} 
			}
		}
	}
);

})();