/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.DateWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.DateTextBox");

dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.client.internal.query.Variable");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function(){
	
var RELATIVE_DATE_VAR_ID= "now";

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var ImagePool= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var Variable= com.ibm.team.workitem.web.client.internal.query.Variable;
var Util= com.ibm.team.workitem.web.client.util;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.DateWidget", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/widgets/templates/DateWidget.html"),
	widgetsInTemplate: true,
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this.textYesterday = this.messages.labelYesterday;
		this.textToday = this.messages.labelToday;
		this.textTomorrow = this.messages.labelTomorrow;
		this.textHoursAgo = this.messages.optionHoursAgo;
		this.textDaysAgo = this.messages.optionDaysAgo;
		this.textMonthsAgo = this.messages.optionMonthsAgo;
		this.textYearsAgo = this.messages.optionYearsAgo;
		this.textHoursAgo = this.messages.optionHoursAgo;
		this.textDaysFromNow = this.messages.optionDaysFromNow;
		this.textHoursFromNow = this.messages.optionHoursFromNow;
		this.textMonthsFromNow = this.messages.optionMonthsFromNow;
		this.textYearsFromNow = this.messages.optionYearsFromNow;
		this.presentation= args.presentation;
		this.selection= args.selection;
		this.variables= args.variables;
		this._parameters= Util.buildMap(args.presentation.properties);
	},
	
	postCreate: function() {
		this._calendarImage.src= ImagePool.CALENDAR;
		this._handleFutureVariables();
		this._setValues(this.selection);
		this._setVariables(this.variables);
		this._connectListeners();
	},
	
	/* String[] */ getValues: function() {
		var selection= new Array();
		var date= this._dateTextBox.getValue();
		if (date && this._getAbsoluteRadio().checked) {
			// Set to noon local
			date.setHours(12);
			date.setMinutes(0);		
			date.setSeconds(0);
			date.setMilliseconds(0);
			var timestamp= date.getTime() + "";
			selection.push(timestamp);
		}
		return selection;
	},
	
	/* Variable[] */ getVariables: function() {
			
		var variables= new Array();
		var self= this;
		
		dojo.query(".values input", this.domNode).forEach(function(input) {
			if (input.checked) {
				if ("custom" == input.value) {
					var custom= self._getCustomVariable();
					if (custom) {
						variables.push(custom);
					}
				} else if ("absolute" != input.value) {
					variables.push(new Variable(RELATIVE_DATE_VAR_ID, input.value));
				}
			}
		});

		return variables;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_connectListeners: function() {
		var radios= dojo.query('input[type="radio"]', this.domNode);
		for (var i in radios) {
			this.connect(radios[i], "onclick", this._handleRadioChange);
		}
		this.connect(this._dateTextBox, "onChange", this._handleDateSelected);
	},
	
	_getCustomVariable: function() {
		var inputValue= this._timeUnitInput.value;
		var duration= inputValue ? parseInt(inputValue) : Number.NaN;
		if (!isNaN(duration)) {
			var unit= this._timeUnitSelect.value;
			var args= this._createArgument(unit, duration);
			return new Variable(RELATIVE_DATE_VAR_ID, args);
		}
		return null;
	},
	
	_setValues: function(values) {
		if (values && values.length > 0) {
			var date= new Date();
			date.setTime(values[0].id);
			this._dateTextBox.setValue(date);
			this._check(this._getAbsoluteRadio());
		}
	},
	
	_setVariables: function(variables) {
		if (variables && variables.length > 0) {
			for (var i= 0; i < variables.length; i++) {
				var inputList= dojo.query('input[value="' + variables[i].arguments + '"]', this.domNode);
				if (inputList.length > 0) {
					this._check(inputList[0]);
				} else {
					var timeUnit= this._parseArgument(variables[i].arguments);
					this._timeUnitInput.value= timeUnit.duration;
					this._timeUnitSelect.value= timeUnit.unit;
					this._checkCustomVariableControl();
				}
			}
		}
	},
	
	_handleFutureVariables: function() {
		if (!this._parameters.futureVariables || this._parameters.futureVariables == "true")
			return;
		dojo.query("[future]", this.domNode).forEach(function(node) {
			node.parentNode.removeChild(node);
		});
	},
	
	_createArgument: function(timeUnit, duration) {
		if (timeUnit.charAt(0) == '-')
			return "-" + duration + timeUnit.substr(1);
		return duration + timeUnit;
	},
	
	_parseArgument: function(arg) {
		
		var unit= "";
		var duration= "";
		var lastIndex= arg.length - 1;
		var firstIndex= 0;

		if (arg.charAt(0) == '-') {
			unit= "-";
			firstIndex= 1;
		}

		duration= arg.substring(firstIndex, lastIndex);
		unit+= arg.substring(lastIndex);
		
		return { duration: duration, unit: unit };
	},
	
	_findRadio: function(value) {
		var inputs= dojo.query('input[value="' + value + '"]', this.domNode);
		return inputs.length > 0 ? inputs[0] : null;
	},
	
	_getAbsoluteRadio: function() {
		return this._findRadio("absolute");
	},
	
	_checkCustomVariableControl: function() {
		var radio= this._findRadio("custom");
		this._check(radio);
	},
	
	_onSelectCalendar: function() {
		this._dateTextBox._open();
	},
	
	_onTimeInputFocus: function() {
		this._checkCustomVariableControl();
	},
	
	_handleRadioChange: function(event) {
		this._check(event.target);
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_handleDateSelected: function() {
		var dateRadio= this._getAbsoluteRadio();
		this._check(dateRadio);
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_onTimeInputChange: function() {
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_onTimeUnitChange: function() {
		this._checkCustomVariableControl();
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_check: function(input) {
		var radios= dojo.query('input[type="radio"]', this.domNode);
		for (var i in radios) {
			radios[i].defaultChecked= false;
		}
		input.checked= input.defaultChecked= true;
	}
});
	
})();
