/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.TextWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.HoverPopup");

dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function(){
	
var STYLE_VALID= "validation-ok"
var STYLE_INVALID= "validation-failed"

var HoverPopup= jazz.ui.HoverPopup;
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.TextWidget", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/widgets/templates/TextWidget.html"),
	
	converters: {
		"integer": function(value) {
			var parsed= parseInt(value);
			if ((parsed < -2147483648) || (parsed > 2147483647))
				return Number.NaN;
			return parsed;
		},
		"long": function(value) {
			var parsed= parseInt(value);
			if ((parsed < -9223372036854775807) || parsed > 9223372036854775807)
				return Number.NaN;
			return parsed; 
		},
		"float": function(value) {
			return parseFloat(value);
		}
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this.presentation= args.presentation;
		this.selection= args.selection;
		
		this._converter= this.converters[this.presentation.attribute.attributeTypeId];
	},
	
	postCreate: function() {

		if (this._converter)
			this._enableTooltip(this.messages.titleEnterNumber);
		
		if (this.selection)
			this._setValues(this.selection);
	},
	
	/* String[] */ getValues: function() {
		var selection= new Array();
		var rawValues= this._getRawValues();
		for (var i= 0; i < rawValues.length; i++) {
			var value= this._convert(rawValues[i]);
			if (value && (!this._converter || !isNaN(value)))
				selection.push(value);
		}
		return selection;
	},
	destroy: function() {
		if(this._toolTip) this._toolTip.destroy();
		this.inherited(arguments);
	},
	doValidate: function() {
		var inputNode= this._textInput;
		if (this._validateInput()) {
			dojo.removeClass(inputNode, STYLE_INVALID);
			dojo.addClass(inputNode, STYLE_VALID);
			this.hideTooltip();
			dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
		} else {
			dojo.removeClass(inputNode, STYLE_VALID);
			dojo.addClass(inputNode, STYLE_INVALID);
			this.showTooltip();
		}
	},
	
	hideTooltip: function() {
		if (this._toolTip) {
			this._toolTip.close();
		}
	},
	
	showTooltip: function() {
		if (this._toolTip && !this._toolTip._isOpen) {
			this._toolTip.open();
		}
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_enableTooltip: function(message) {
		this._toolTip= new HoverPopup({
			around: [this._textInput],
			html: message,
			orientation: "L",
			programmatic: true
		});
	},
	
	_convert: function(value) {
		if (value.length > 0 && this._converter) {
			return this._converter(value, this._converterParms);
		}
		return value;
	},
	
	_validateInput: function() {
		if (this._converter) {
			var values= this._getRawValues();
			for (var i= 0; i < values.length; i++) {
				var converted= this._convert(values[i])
				if (isNaN(converted) || (values[i] != converted))
					return false;			
			}
		}
		return true;	
	},
	
	/* String[] */ _getRawValues: function() {
		var selection= new Array();
		var stringInput= this._textInput.value;
		var parts= stringInput.split(",");
		for (var i= 0; i < parts.length; i++) {
			var trimmed= dojo.trim(parts[i]);
			selection.push(trimmed);
		}
		return selection;
	},
	
	_setValues: function(/* UIItemDTO[] */ selection) {
		var list= new Array();
		for (var i= 0; i < selection.length; i++) {
			list.push(selection[i].id);
		}
		this._textInput.value= list.join(", ");
	}
	
});
	
})();
