/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui2.internal.dialog.CreateFromTemplateDialogLauncher"); 
dojo.require("com.ibm.team.workitem.web.ui2.internal.dialog.PolicyDialog");
dojo.require("com.ibm.team.workitem.web.cache.internal.PolicyRequestProxy"); 
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var WorkItemAttributes= com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient= com.ibm.team.workitem.web.client.internal.WorkItemClient;

dojo.declare("com.ibm.team.workitem.web.ui2.internal.dialog.CreateFromTemplateDialogLauncher", null, {
openCreateFromTemplateDialog: function(parms) {
    if(!this.openCreateFromTemplateDialogMessages) this.openCreateFromTemplateDialogMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

	var templates = parms.templates;
   
    /* construct the list of templates */
    
	/* when a template is selected, show the description and variables */
	
    /* add the attribute variables */
    var self = this;
	var presentations = [
		{
			id: "listWidget",
			widget: "com.ibm.team.workitem.web.ui2.internal.widget.DialogListWidget",
			attributeId: "template",
			possibleValues: templates,
			detailsSelector: "description",
			messageSelector: "label",
			size: 6,
			initialize: function(parms) {
				var policyRequest = parms.workingCopy;
				if(policyRequest) {
					policyRequest.addListener({
						path: "attributes.template", // a template
						event: "onchange",
						listener: this,
						functionName: "_handleTemplateChange"
					});
				}
			},
			_handleTemplateChange: function(parms /* parentWidget: aPolicyDialog, workingCopy: aPolicyRequestProxy */) {
				self._validateRequestAndEnableCreateButton(parms); // self == the launcher
			}
		},
		{
			id: WorkItemAttributes.CATEGORY,
			label: "", // set in initialize
			widget: "com.ibm.team.workitem.web.ui.internal.view.editor.parts.CategoryAttributePart",
			creationArgs: {
				showGuessCategoryLink: false
			},
			attributeId: WorkItemAttributes.CATEGORY,
			initialize: function(parms) {
				parms.layoutWidget.setVisible(false);
				self._initializePossibleValues(parms);
				var policyRequest = parms.workingCopy;
				if(policyRequest) {
					policyRequest.addListener({
						path: "attributes.template", 
						event: "onchange",
						listener: this,
						functionName: "_handleTemplateChange"
					});
				}
			},
			_handleTemplateChange: function(parms) {
				parms.attributeId = WorkItemAttributes.CATEGORY;
				self._handleTemplateChange(parms);
			}
		},
		{   
			id:  WorkItemAttributes.TARGET,
			label: "", // set in initialize
			widget: "com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget",
			attributeId: WorkItemAttributes.TARGET,
			possibleValuesProvider: self._getTargetPossibleValues,
			initialize: function(parms) {
				parms.layoutWidget.setVisible(false);
				self._initializePossibleValues(parms);
				var policyRequest = parms.workingCopy;
				if(policyRequest) {
					policyRequest.addListener({
						path: "attributes.template", 
						event: "onchange",
						listener: this,
						functionName: "_handleTemplateChange"
					});
				}
			},
			_handleTemplateChange: function(parms) {
				parms.attributeId = WorkItemAttributes.TARGET;
				self._handleTemplateChange(parms);
			}
		}
	];
	
	/*
	 * The policy request is initialized with dummy potential attributes
	 */
	var policyRequestObject = {
		attributes: {
			template: false,
			target: false,
			category: false
		}
	}
	var policyRequest =  new com.ibm.team.workitem.web.cache.internal.PolicyRequestProxy({ 
		object: policyRequestObject
		}); 
	
	var dialogArgs = {
		dialogTitle: this.openCreateFromTemplateDialogMessages.labelTemplates,
		secondaryTitle: "",
		height: "18em",		
		workingCopy: policyRequest,
		OkLabel: this.openCreateFromTemplateDialogMessages.labelCreateFromTemplate,
		onOk: function(args) {
				parms.onOk(args);
		},
		onCancel: function(args) {
		// ignore
		},
		presentations: presentations
	}
	new com.ibm.team.workitem.web.ui2.internal.dialog.PolicyDialog(dialogArgs);
},
_validateRequestAndEnableCreateButton: function(parms) {
	// TODO: For now always enabled
},
_handleTemplateChange: function(parms) {
	var policyRequest = parms.workingCopy;
	var template = parms.workingCopy.getValue({path: "attributes.template"});
	var templateMap = parms.caller.parentWidget.getPropertyMapFromArray({
		array: template.attributeVariables, 
		result: {}, 
		keySelector: "id"}).result;
	var widget = parms.caller.parentWidget.getChildWidget(parms.attributeId);
	var layout = parms.caller.parentWidget.getChildWidgetLayout(parms.attributeId);
	if(templateMap[parms.attributeId]) {
		/*
		 * The label is only known after the template is loaded
		 * so only show the widget after the label is known
		 */
		if(layout && !layout.isVisible()) {
			var labelText = templateMap[parms.attributeId].label;
			labelText = dojo.string.substitute(this.openCreateFromTemplateDialogMessages.labelValue, [ labelText ]);
			layout.setLabel({labelText: labelText});
		}
		widget.setEnabled(true);
		if(layout) {
			layout.setVisible(true);
		} else {
			widget.domNode.style.visibility = "visible";
		} 
	} else {
		widget.setEnabled(false);
		if(layout) {
			layout.setVisible(false);
		} else {
			widget.domNode.style.visibility = "hidden";
		}
	}
},
_initializePossibleValues: function(parms) {
		var policyRequest = parms.workingCopy;
		if(this._hasRequestedPossibleValues) return;
		this._hasRequestedPossibleValues = true;
		var op = parms.workingCopy.createOperation({
	        message: this.openCreateFromTemplateDialogMessages.busyLoading,
	        id: "loadingTemplatePossibleValues"
	  	});
	  	parms.workingCopy.startOperation(op);
		var self= this;
		var handlerObj= {
			success: function(/* UIItemSetDTO */ valueSet) {
				parms.workingCopy.finishOperation(op);
				var valueMap = parms.caller.parentWidget.getPropertyMapFromArray({
					array: valueSet, 
					result: {}, 
					keySelector: "attributeName"});
			    var categoryValues = valueMap.result.category ? valueMap.result.category.uiItems : [];
				var categoryWidget = parms.caller.parentWidget.getChildWidget(WorkItemAttributes.CATEGORY);
				categoryWidget.setPossibleValues(categoryValues);
				
			    var targetValues = valueMap.result.target ? valueMap.result.target.uiItems : [];
				var targetWidget = parms.caller.parentWidget.getChildWidget(WorkItemAttributes.TARGET);
				targetWidget.setPossibleValues(targetValues);
			},
			failure: function(errorObj) {
				parms.workingCopy.finishOperation(op);
		 		console.error("Error fetching dialog possible " + errorObj.message); // $NON-NLS-1$
			}
		};

		var handler= new ServiceResponseHandler(handlerObj, "success", "failure");
		WorkItemClient.getAllValues(handler, { newWorkItem: true, ids: [ WorkItemAttributes.CATEGORY, WorkItemAttributes.TARGET ] });
}
}
);

})();
