/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.util.DashboardUtil");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");

(function() {

/*
 * Aliases
 */
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

/*
 * Public utility methods.
 * 
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */
com.ibm.team.dashboard.web.util.DashboardUtil = {
	/*
	 * Returns an HREF that can be used as a link to the given dashboard info. Tab is
	 * optional, and if specified, will take you to that specific tab. The scopeItem
	 * param indicates the target dashboard depending on the indicated scope. For
	 * personal dashboards, scopeItem is the itemId of the dashboard, for team dashboards
	 * it's an array of segments in the team area path and for project dashboards it's
	 * the project area name.
	 * 
	 * If projectName is specified, it will be shown in the context of that project,
	 * otherwise it will be assumed to be in the current project context.
	 */
	/* string */ getDashboardHref: function(/* string */ scope, /* string */ scopeItem, /* string */ tabId, /* string */ projectName) {
		var href = [];
		if (projectName) {
			var href = [net.jazz.ajax._contextRoot];
			href.push("/web/projects/");
			href.push(encodeURIComponent(projectName));
			if (djConfig.isDebug) {
				href.push("?debug=true");
			}
		}
		
		href.push("#action=");
		href.push(DashboardConstants.ACTION_VIEW_DASHBOARD);
		switch(scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				href.push("&");
				href.push(DashboardConstants.URL_PARAM_ID);
				href.push("=");
				href.push(scopeItem);
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				var path = [];
				for (var i = 0; i < scopeItem.length; i++) {
					if (path.length > 0) {
						path.push("/");
					}
					path.push(encodeURIComponent(scopeItem[i]));
				}
				href.push("&");
				href.push(DashboardConstants.URL_PARAM_TEAM);
				href.push("=");
				href.push(path.join(""));
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				break;
		}
		if (tabId) {
			href.push("&");
			href.push(DashboardConstants.URL_PARAM_TAB);
			href.push("=");
			href.push(tabId);
		}
		return href.join("");
	}
};

})();