/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.Duration"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.number"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
	
var estimateConfig= com.ibm.team.apt.web.client.util.EstimateConfig;
var UNSPECIFIED= -1;
var INVALID= -2;

var bind= dojo.string.substitute;
var Messages= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

/**
 * TODO this belongs to work item land
 */
dojo.declare("com.ibm.team.apt.client.Duration", null, { //$NON-NLS-1$
	
	__duration: null,
	
	constructor: function(duration) {
		this.__duration= Math.round(duration);
		this.specified= duration >= 0;
		this.valid= duration != INVALID;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getMilliseconds: function() {
		return this.specified ? this.__duration : 0;
	},

	getLabel: function() {
		if (!this.label) {
			this.label= __generateLabel(this.__duration);
		}
		return this.label;
	},
	
	specified: false,

	valid: false,
	
	compareTo: function(other) {
		if (!this.specified || !other.specified)
			return (this.specified ? 1 : 0) - (other.specified ? 1 : 0);
		if (this.__duration < other.__duration)
			return -1;
		if (this.__duration > other.__duration)
			return 1;
		return 0;
	},

	equals: function(other) {
		if (this === other)
			return true;

		if (this.declaredClass !== other.declaredClass)
			return false;
		
		return this.__duration == other.__duration;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.client.Duration.UNSPECIFIED= new com.ibm.team.apt.client.Duration(UNSPECIFIED); 
com.ibm.team.apt.client.Duration.INVALID= new com.ibm.team.apt.client.Duration(INVALID); 

function __generateLabel(duration) {
	var label;
	if (duration == UNSPECIFIED || duration == 0 || isNaN(duration)) {
		label= Messages['duration_notSpecified']; //$NON-NLS-1$
	} else {
		var patternSelector;
		var normalizedTime;
		if ("Hours" == estimateConfig.estimateOutputFormat) { //$NON-NLS-1$
			patternSelector= "hours"; //$NON-NLS-1$
			normalizedTime= duration / (1000 * 60 * 60);
		} else {
			if (duration < 1000 * 60) {
				patternSelector= "seconds"; //$NON-NLS-1$
				normalizedTime= (duration / 1000);
			} else if (duration < 1000 * 60 * 60) {
				patternSelector= "minutes"; //$NON-NLS-1$
				normalizedTime= duration / (1000 * 60);
			} else if (duration < 1000 * 60 * 60 * 8) {
				patternSelector= "hours"; //$NON-NLS-1$
				normalizedTime= duration / (1000 * 60 * 60);
			} else if (duration < 1000 * 60 * 60 * 8 * 5) {
				patternSelector= "days"; //$NON-NLS-1$
				normalizedTime= duration / (1000 * 60 * 60 * 8);
			} else {
				patternSelector= "weeks"; //$NON-NLS-1$
				normalizedTime= duration / (1000 * 60 * 60 * 8 * 5);
			}
		}

		var quantitySelector= normalizedTime != 1 ? "_plural" : "_singular"; //$NON-NLS-1$ //$NON-NLS-2$
		label= bind(Messages['duration_' + patternSelector + quantitySelector], [dojo.number.format(normalizedTime, { pattern: "#.#" })]); //$NON-NLS-1$ //$NON-NLS-2$
	}
	return label;
}


})();
