/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.PlanDeltaBuilder"); //$NON-NLS-1$

dojo.require("com.ibm.team.foundation.Assert"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.PlanElement"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.PlanElementDelta"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.PlanElementAttributeDelta"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.PlanElementChangeEvent"); //$NON-NLS-1$

(function() {
	
var Assert					= com.ibm.team.foundation.Assert;
	
var PlanElement				= com.ibm.team.apt.client.PlanElement;
var PlanElementDelta		= com.ibm.team.apt.client.PlanElementDelta;
var PlanAttributeDelta		= com.ibm.team.apt.client.PlanElementAttributeDelta;
var PlanElementChangeEvent	= com.ibm.team.apt.client.PlanElementChangeEvent;
	
dojo.declare("com.ibm.team.apt.client.PlanDeltaBuilder", null, { //$NON-NLS-1$
	
	__planRoot: null,
	__deltaRoot: null,
	
	__oldParent: null,
	__newParent: null,
	
	__generation: 0,

	__postRunnables: null,
	
	constructor: function(root) {
		this.__planRoot= root;
		this.__deltaRoot= new PlanElementDelta(null, this.__planRoot);
		this.__postRunnables= [];
	},
	
	newGeneration: function() {
		return this.__generation++;
	},

	isEmpty: function() {
		return this.__deltaRoot.isNoChange() && !this.__deltaRoot.hasChildren();
	},
	
	getEvent: function() {
		return new PlanElementChangeEvent(this.__deltaRoot);
	},
	
	sendPostNotifications: function() {
		for ( var i = 0; i < this.__postRunnables.length; i++) {
			this.__postRunnables[i]();
		}
	},
	
	addPostNotification: function(runnable) {
		Assert.isNotNull(runnable);
		this.__postRunnables.push(runnable);
	},
	
	isInMove: function() {
		return this.__oldParent != null && this.__newParent != null;
	},
	
	aboutToMove: function(oldParent, newParent) {
		Assert.isNotNull(oldParent); 
		Assert.isNotNull(newParent);
		
		this.__oldParent= oldParent;
		this.__newParent= newParent;
	},
	
	moved: function() {
		this.__oldParent= null;
		this.__newParent= null;
	},
	
	changed: function(element, attribute, oldValue, newValue) {
		var delta= this.getDelta(element);
		if (delta.isNoChange()) {
			delta.setChanged();
		}
		delta.setContentChange();
		delta.addAttribute(new PlanAttributeDelta(attribute, oldValue, newValue, this.__generation));
	},
	
	planCheckChanged: function(oldReport, newReport) {
		Assert.isTrue(oldReport != null ? this.__planRoot == oldReport.getPlanElement() : true && this.__planRoot == newReport.getPlanElement());
		this.__deltaRoot.addAttribute(new PlanAttributeDelta(PlanElement.PLANCHECK_REPORT, oldReport, newReport, this.__generation));
		
		var reports= newReport.getReports();
		for (var i= 0; i < reports.length; i++) {
			var newItemReport= reports[i];
			var planItem= newItemReport.getPlanElement();
			var oldItemReport= oldReport != null ? oldReport.getReport(planItem) : null;
			
			if ((oldItemReport == null && newItemReport.hasProblems()) || (oldItemReport != null && !oldItemReport.equals(newItemReport))) {
				var delta= this.getDelta(planItem);
				delta.setChanged();
				delta.addAttribute(new PlanAttributeDelta(PlanElement.PLANCHECK_REPORT, oldItemReport, newItemReport, this.__generation));
			}
		}
	},
	
	added: function(element) {
		var parent= element.getParent();
		Assert.isNotNull(parent);
		var parentDelta= this.getDelta(parent);
		if (parentDelta.isNoChange()) {
			parentDelta.setChanged();
		}
		parentDelta.setStructuralChange();
		var childrenCount= parent.getChildrenCount();
		this.changed(parent, PlanElement.CHILDREN_COUNT, childrenCount - 1, childrenCount);
		if (childrenCount == 1)
			this.changed(parent, PlanElement.CHILDREN, false, true);

		var delta= this.getDelta(element);
		delta.setAdded();
		if (this.__oldParent != null) {
			delta.setMoveParent(this.__oldParent);
			delta.setOtherMoveDelta(this.getDeltaWithPath(this.getPath(this.__oldParent, element)));
		}
	},
	
	removed: function(element) {
		var parent= element.getParent();
		Assert.isNotNull(parent);
		var parentDelta= this.getDelta(parent);
		if (parentDelta.isNoChange()) {
			parentDelta.setChanged();
		}
		parentDelta.setStructuralChange();

		var childrenCount= parent.getChildrenCount();
		this.changed(parent, PlanElement.CHILDREN_COUNT, childrenCount + 1, childrenCount);
		if (childrenCount == 0)
			this.changed(parent, PlanElement.CHILDREN, true, false);		

		var delta= this.getDelta(element);
		if (delta.isAdded()) {
			delta.getParentDelta().removeChild(delta);
			this.__oldParent= null;
			this.__newParent= null;
		} else {
			delta.setRemoved();
			if (this.__newParent != null) {
				delta.setMoveParent(this.__newParent);
				delta.setOtherMoveDelta(this.getDeltaWithPath(this.getPath(this.__newParent, element)));
			} else {
				var children= element.getChildren();
				for (var i= 0; i < children.length; i++) {
					this.removed(children[i]);
				}
			}
		}
	},
	
	getDelta: function(element) {
		Assert.isTrue(this.__planRoot == element.getPlanModel());
		return this.getDeltaWithPath(this.getPath(element, null));
	},
	
	getDeltaWithPath: function(path) {
		if (path.length == 1)
			return this.__deltaRoot;
		
		var current= this.__deltaRoot;
		for (var i= path.length - 2; i >= 0; i--) {
			var child= current.getChild(path[i]);
			if (child == null) {
				child= new PlanElementDelta(current, path[i]);
				current.addChild(child);
			}
			current= child;
		}
		return current;
	},

	getPath: function(element, child) {
		var result= [];
		if (child != null) {
			result.push(child);
		}
		
		result.push(element);
		var parent= element.getParent();
		while (parent != null) {
			result.push(parent);
			parent= parent.getParent();
	
			if (result.length >= 1000) {
				throw new Error("Endless loop in PlanDeltaBuilder#getPath()"); //$NON-NLS-1$
			}
		}
		return result;
	},

	__sentinel: null // terminates this class definition
});


})();

