/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.common.Attribute"); //$NON-NLS-1$

dojo.require("dojo.date.stamp"); //$NON-NLS-1$

// XXX layer breaker -> must go to repository
dojo.require("com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore"); //$NON-NLS-1$

// XXX layer breaker -> must go to workitem
//dojo.require("com.ibm.team.apt.client.Duration"); //$NON-NLS-1$

(function() {

//var Duration= com.ibm.team.apt.client.Duration;
var ReferencedItemStore= com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore;

var NULL_MARKER= "_sM_UI_9bEd24fuo2IY3YYw"; //$NON-NLS-1$
var UNSET_MARKER= "_yJkARv9bEd2L6dd7tp2HxA"; //$NON-NLS-1$
var UNSUPPORTED_MARKER= "_qqX1ZP9cEd2KEOAL-50s4g"; //$NON-NLS-1$

var TYPE_STRING= "STRING"; //$NON-NLS-1$
var TYPE_HTML= "HTML"; //$NON-NLS-1$
var TYPE_INTEGER= "INTEGER"; //$NON-NLS-1$
var TYPE_FLOAT= "FLOAT"; //$NON-NLS-1$
var TYPE_BOOLEAN= "BOOLEAN"; //$NON-NLS-1$
var TYPE_DATE= "DATE"; //$NON-NLS-1$
var TYPE_TAGS= "TAGS"; //$NON-NLS-1$
var TYPE_DURATION= "DURATION"; //$NON-NLS-1$
var TYPE_UUID= "UUID"; //$NON-NLS-1$
var TYPE_TYPE= "TYPE"; //$NON-NLS-1$
var TYPE_ITEM= "ITEM"; //$NON-NLS-1$
var TYPE_ENUMERATION= "ENUMERATION"; //$NON-NLS-1$

dojo.declare("com.ibm.team.workitem.common.Attribute", null, { //$NON-NLS-1$

	__referencedItems: null,
	__valueSet: null,
	__nullValue: null,
	
	constructor: function(attributeDTO) {
		this.itemId= attributeDTO.itemId;
		this.itemType= attributeDTO.itemType;
		this.label= attributeDTO.label;
		this.iconURL= attributeDTO.iconURL;

		this.publicId= attributeDTO.publicId;
		this.internalId= attributeDTO.internalId;

		this.attributeType= attributeDTO.attributeType;
		this.referenceType= attributeDTO.referenceType;

		this.__nullValue= attributeDTO.nullValue;
		this.__valueSet= attributeDTO.valueSet;
		
		this.__referencedItems= attributeDTO._referencedItemStore;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	itemId: null,
	itemType: null,
	label: null,
	iconURL: null,

	publicId: null,
	internalId: null,

	attributeType: null,
	referenceType: null,
	
	getNullValue: function() {
		return this.demarshal(this.__nullValue);
	},

	getValueSet: function() {
		return dojo.map(this.__valueSet, function(value) {
			return this.demarshal(value);
		}, this);
	},

	demarshal: function(value) {
		if (this.attributeType == TYPE_DURATION) {
			if (value === NULL_MARKER)
				return null;
			if (value === UNSET_MARKER)
				return com.ibm.team.apt.client.Duration.UNSPECIFIED;
			return new com.ibm.team.apt.client.Duration(parseInt(value));
		}
		
		if (value === NULL_MARKER)
			return null;
		if (value === UNSET_MARKER)
			return undefined;

		switch (this.attributeType) {
			case TYPE_STRING:
			case TYPE_HTML:
				return value;
			case TYPE_INTEGER:
				return parseInt(value);
			case TYPE_FLOAT:
				return parseFloat(value);
			case TYPE_BOOLEAN:
				return value === "true"; //$NON-NLS-1$
			case TYPE_DATE:
				return dojo.date.stamp.fromISOString(value);
			case TYPE_TAGS:
				return dojo.fromJson(value) || [];
			case TYPE_UUID:
				return value;
			case TYPE_TYPE:
				return this.__referencedItems.getItem(ReferencedItemStore.ITEM_TYPES.WITYPE, value);
			case TYPE_ITEM:
			case TYPE_ENUMERATION:
				return this.__referencedItems.getItem(this.referenceType, value);
		}
		return null;
	},

	marshal: function(value) {
		if (value === null)
			// the foundation infrastructure can't handle null. see defect 42903 *sigh* 
			return NULL_MARKER;
		if (value === undefined)
			return UNSET_MARKER;
		
		switch (this.attributeType) {
			case TYPE_STRING:
			case TYPE_HTML:
			case TYPE_UUID:
				return value;
			case TYPE_INTEGER:
			case TYPE_FLOAT:
			case TYPE_BOOLEAN:
				return String(value);
			case TYPE_DATE:
				return dojo.date.stamp.toISOString(value);
			case TYPE_TYPE:
			case TYPE_ITEM:
			case TYPE_ENUMERATION:
				return value.itemId;
			case TYPE_DURATION:
				return value.__duration;
			case TYPE_TAGS:
				throw new Error("not yet implemented"); //$NON-NLS-1$
		}
		return null;
	},
	
	equals: function(other) {
		if (this === other)
			return true;

		if (this.declaredClass !== other.declaredClass)
			return false;
		
		return this.itemId == other.itemId;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});

com.ibm.team.workitem.common.Attribute.STRING= TYPE_STRING;
com.ibm.team.workitem.common.Attribute.HTML= TYPE_HTML;
com.ibm.team.workitem.common.Attribute.INTEGER= TYPE_INTEGER;
com.ibm.team.workitem.common.Attribute.FLOAT= TYPE_FLOAT;
com.ibm.team.workitem.common.Attribute.BOOLEAN= TYPE_BOOLEAN;
com.ibm.team.workitem.common.Attribute.DATE= TYPE_DATE;
com.ibm.team.workitem.common.Attribute.TAGS= TYPE_TAGS;
com.ibm.team.workitem.common.Attribute.DURATION= TYPE_DURATION;
com.ibm.team.workitem.common.Attribute.UUID= TYPE_UUID;
com.ibm.team.workitem.common.Attribute.TYPE= TYPE_TYPE;
com.ibm.team.workitem.common.Attribute.ITEM= TYPE_ITEM;
com.ibm.team.workitem.common.Attribute.ENUMERATION= TYPE_ENUMERATION;


})();
