/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.ui.structure.TimespanDefinition"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.internal.Instant"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var Instant						= com.ibm.team.apt.client.internal.Instant;
	
var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var LOWEST							= new Instant(new Date(1));
var HIGHEST							= new Instant(new Date(Math.pow(2, 52)));

dojo.declare("com.ibm.team.apt.ui.structure.TimespanDefinition", null, { //$NON-NLS-1$

	__index: -1,
	__isPast: false,
	__startTime: null,
	__endTime: null,
	__referenceTime: 0,

	constructor: function() {
		this.__referenceTime= new Date().getTime();
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	label: null,
	
	isPast: function() {
		return this.__isPast;
	},

	getStart: function() {
		return this.__startTime;
	},
	
	getEnd: function() {
		return this.__endTime;
	},

	refreshTimestamps: function(reference) {
		this.__referenceTime= reference.getDate().getTime();
		this.__startTime= this.__endTime= null;
		this.__ensureInitialized();
	},

	includes: function(when, resolved) {
		this.__ensureInitialized();
		if (when == null || this.__isPast != resolved)
			return false;
		
		return this.__startTime.beforeEq(when) && this.__endTime.after(when);
	},

	compareTo: function(other) {
		return this.__index - other.__index;
	},

	equals: function(other) {
		return other != null && this.declaredClass == other.declaredClass;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_calculateStartTime: function() {
		return new Instant(new Date(this.__referenceTime));
	},

	_calculateEndTime: function() {
		return new Instant(new Date(this.__referenceTime));
	},

	__ensureInitialized: function() {
		if (this.__startTime == null || this.__endTime == null) {
			this.__startTime= this._calculateStartTime();
			this.__endTime= this._calculateEndTime();
		}
	},

	__sentinel: null // terminates this class definition
});
var TimespanDefinition= com.ibm.team.apt.ui.structure.TimespanDefinition;


dojo.declare("com.ibm.team.apt.ui.structure.TimespanDefinition.LaterToday", TimespanDefinition, { //$NON-NLS-1$

	constructor: function() {
		this.__index= 0;
		this.label= MSGS.TIMESPAN_TODAY;
	},

	_calculateEndTime: function() {
		var result= new Date(this.__referenceTime);
		result.setHours(0);
		result.setMinutes(0);
		result.setSeconds(0);
		result.setMilliseconds(0);
		result.setDate(result.getDate() + 1);
		return new Instant(result);
	},

	__sentinel: null // terminates this class definition
});

dojo.declare("com.ibm.team.apt.ui.structure.TimespanDefinition.LaterThisWeek", TimespanDefinition, { //$NON-NLS-1$
	
	constructor: function() {
		this.__index= 10;
		this.label= MSGS.TIMESPAN_LATERTHISWEEK;
	},

	_calculateEndTime: function() {
		var result= new Date(this.__referenceTime);
		result.setHours(0);
		result.setMinutes(0);
		result.setSeconds(0);
		result.setMilliseconds(0);
		result.setDate(result.getDate() + (7 - result.getDay()));
		return new Instant(result);
	},

	__sentinel: null // terminates this class definition
});

dojo.declare("com.ibm.team.apt.ui.structure.TimespanDefinition.Future", TimespanDefinition, { //$NON-NLS-1$
	
	constructor: function() {
		this.__index= 20;
		this.label= MSGS.TIMESPAN_FUTURE;
	},

	_calculateEndTime: function() {
		return HIGHEST;
	},

	__sentinel: null // terminates this class definition
});

dojo.declare("com.ibm.team.apt.ui.structure.TimespanDefinition.Past", TimespanDefinition, { //$NON-NLS-1$
	
	constructor: function() {
		this.__index= -10;
		this.label= MSGS.TIMESPAN_PAST;
		this.__isPast= true;
	},

	_calculateStartTime: function() {
		return LOWEST;
	},

	__sentinel: null // terminates this class definition
});

dojo.declare("com.ibm.team.apt.ui.structure.TimespanDefinition.__Unscheduled", TimespanDefinition, { //$NON-NLS-1$

	constructor: function() {
		this.__index= Number.MAX_VALUE;
		this.label= MSGS.TIMESPAN_UNSCHEDULED;
	},
	
	includes: function(when, resolved) {
		return when == null && !resolved;
	},

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.ui.structure.TimespanDefinition.UNSCHEDULED= new com.ibm.team.apt.ui.structure.TimespanDefinition.__Unscheduled();

})();
